@@ -31,3 +31,10 @@ def test_package_storage_get_file(storage: PackageStorage) -> None:
     result = storage.get_file("test", ext=PurePath("should-use-my-extension.mkv"))
     assert result.name == "test.mkv"
     assert result.suffix == ".mkv"
+
+
+def test_package_storage_default_folder(storage: PackageStorage) -> None:
+    # Have `cwd` be a file just to pretend it's the result of `get_script_path()`.
+    storage = PackageStorage(cwd=PurePath(__file__))
+
+    assert storage.folder.is_dir()
