@@ -3,11 +3,11 @@ from __future__ import annotations
 from abc import ABC, abstractmethod
 from dataclasses import dataclass, replace
 from enum import IntFlag, auto
-from typing import TYPE_CHECKING, Any, Sequence
+from typing import TYPE_CHECKING, Any, Mapping, Self, Sequence
 
-from jetpytools import MISSING, copy_signature, fallback
-from typing_extensions import Self, TypeVar
+from jetpytools import CustomNotImplementedError, CustomValueError, copy_signature, fallback, normalize_seq
 
+from vsjetpack import TypeVar
 from vskernels import (
     Bobber,
     Catrom,
@@ -19,19 +19,7 @@ from vskernels import (
     Scaler,
     TopShift,
 )
-from vstools import (
-    ChromaLocation,
-    ConstantFormatVideoNode,
-    FieldBased,
-    FieldBasedLike,
-    VideoNodeT,
-    VSFunctionAllArgs,
-    VSFunctionNoArgs,
-    check_variable,
-    core,
-    normalize_seq,
-    vs,
-)
+from vstools import ChromaLocation, FieldBased, FieldBasedLike, VSFunctionAllArgs, VSFunctionNoArgs, core, vs
 
 __all__ = [
     "BWDIF",
@@ -65,12 +53,12 @@ class DeinterlacerKwargs(dict[str, Any]):
             return super().__setitem__(key, value)
         setattr(self.deinterlacer, key, value)
 
-    @copy_signature(dict.update)
+    @copy_signature(dict[str, Any].update)
     def update(self, *args: Any, **kwargs: Any) -> None:
         for k, v in dict(*args, **kwargs).items():
             self[k] = v
 
-    @copy_signature(dict.setdefault)
+    @copy_signature(dict[str, Any].setdefault)
     def setdefault(self, key: str, default: Any = None) -> Any:
         if key not in self:
             self[key] = default
@@ -83,7 +71,7 @@ class Deinterlacer(Bobber, ABC):
     Abstract base class for deinterlacing operations.
     """
 
-    tff: bool = False
+    tff: bool | None = None
     """The field order."""
 
     double_rate: bool = True
@@ -93,17 +81,74 @@ class Deinterlacer(Bobber, ABC):
         self.kwargs = DeinterlacerKwargs()
         self.kwargs.deinterlacer = self
 
+    def deinterlace(
+        self,
+        clip: vs.VideoNode,
+        *,
+        tff: FieldBasedLike | bool | None = None,
+        double_rate: bool | None = None,
+        **kwargs: Any,
+    ) -> vs.VideoNode:
+        """
+        Apply deinterlacing to the given clip.
+
+        Args:
+            clip: The input clip.
+            tff: Field order of the clip.
+            double_rate: Whether to double the frame rate (True) or retain the original rate (False).
+            **kwargs: Additional arguments passed to the plugin function.
+
+        Returns:
+            Deinterlaced clip.
+        """
+        field_based = FieldBased.from_param_or_video(fallback(tff, self.tff, default=None), clip, True, self.__class__)
+
+        return self._interpolate(clip, field_based.is_tff, fallback(double_rate, self.double_rate), False, **kwargs)
+
+    def bob(self, clip: vs.VideoNode, *, tff: FieldBasedLike | bool | None = None, **kwargs: Any) -> vs.VideoNode:
+        """
+        Apply bob deinterlacing to the given clip.
+
+        Args:
+            clip: The input clip.
+            tff: Field order of the clip.
+            **kwargs: Additional arguments passed to the plugin function.
+
+        Returns:
+            Deinterlaced clip.
+        """
+        field_based = FieldBased.from_param_or_video(fallback(tff, self.tff, default=None), clip, True, self.__class__)
+
+        return self._interpolate(clip, field_based.is_tff, True, False, **kwargs)
+
+    def copy(self, **kwargs: Any) -> Self:
+        """
+        Returns a new Antialiaser class replacing specified fields with new values
+        """
+        return replace(self, **kwargs)
+
+    @abstractmethod
+    def get_deint_args(self, **kwargs: Any) -> dict[str, Any]:
+        """
+        Retrieves arguments for deinterlacing processing.
+
+        Args:
+            **kwargs: Additional arguments.
+
+        Returns:
+            Passed keyword arguments.
+        """
+        return kwargs
+
     @property
     @abstractmethod
-    def _deinterlacer_function(self) -> VSFunctionAllArgs[vs.VideoNode, ConstantFormatVideoNode]:
+    def _deinterlacer_function(self) -> VSFunctionAllArgs:
         """
         Get the plugin function.
         """
 
     @abstractmethod
-    def _interpolate(
-        self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
+    def _interpolate(self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any) -> vs.VideoNode:
         """
         Performs deinterlacing if dh is False or doubling if dh is True.
 
@@ -120,71 +165,6 @@ class Deinterlacer(Bobber, ABC):
             Interpolated clip.
         """
 
-    @abstractmethod
-    def get_deint_args(self, **kwargs: Any) -> dict[str, Any]:
-        """
-        Retrieves arguments for deinterlacing processing.
-
-        Args:
-            **kwargs: Additional arguments.
-
-        Returns:
-            Passed keyword arguments.
-        """
-        return kwargs
-
-    def deinterlace(
-        self,
-        clip: vs.VideoNode,
-        *,
-        tff: FieldBasedLike | bool | None = None,
-        double_rate: bool | None = None,
-        **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
-        """
-        Apply deinterlacing to the given clip.
-
-        Args:
-            clip: The input clip.
-            tff: Field order of the clip.
-            double_rate: Whether to double the frame rate (True) or retain the original rate (False).
-            **kwargs: Additional arguments passed to the plugin function.
-
-        Returns:
-            Deinterlaced clip.
-        """
-        return self._interpolate(
-            clip,
-            FieldBased.from_param(fallback(tff, self.tff), self.__class__).is_tff,
-            fallback(double_rate, self.double_rate),
-            False,
-            **kwargs,
-        )
-
-    def bob(
-        self, clip: vs.VideoNode, *, tff: FieldBasedLike | bool | None = None, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
-        """
-        Apply bob deinterlacing to the given clip.
-
-        Args:
-            clip: The input clip.
-            tff: Field order of the clip.
-            **kwargs: Additional arguments passed to the plugin function.
-
-        Returns:
-            Deinterlaced clip.
-        """
-        return self._interpolate(
-            clip, FieldBased.from_param(fallback(tff, self.tff), self.__class__).is_tff, True, False, **kwargs
-        )
-
-    def copy(self, **kwargs: Any) -> Self:
-        """
-        Returns a new Antialiaser class replacing specified fields with new values
-        """
-        return replace(self, **kwargs)
-
 
 @dataclass(kw_only=True)
 class AntiAliaser(Deinterlacer, ABC):
@@ -215,9 +195,7 @@ class AntiAliaser(Deinterlacer, ABC):
         Apply anti-aliasing in both horizontal and vertical directions.
         """
 
-    def antialias(
-        self, clip: vs.VideoNode, direction: AADirection = AADirection.BOTH, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
+    def antialias(self, clip: vs.VideoNode, direction: AADirection = AADirection.BOTH, **kwargs: Any) -> vs.VideoNode:
         """
         Apply anti-aliasing to the given clip.
 
@@ -229,24 +207,26 @@ class AntiAliaser(Deinterlacer, ABC):
         Returns:
             Anti-aliased clip.
         """
-        assert check_variable(clip, self.antialias)
+        tff = fallback(kwargs.pop("tff", self.tff), True)
 
-        for y in sorted((aa_dir for aa_dir in self.AADirection), key=lambda x: x.value, reverse=self.transpose_first):
+        for y in sorted(self.AADirection, key=lambda x: x.value, reverse=self.transpose_first):
             if direction in (y, self.AADirection.BOTH):
                 if y == self.AADirection.HORIZONTAL:
-                    clip = self.transpose(clip)
+                    clip, tclips = self.transpose(clip, **kwargs)
+                    kwargs |= tclips
 
-                clip = self.deinterlace(clip, **kwargs)
+                clip = self._interpolate(clip, tff, self.double_rate, False, **kwargs)
 
                 if self.double_rate:
                     clip = core.std.Merge(clip[::2], clip[1::2])
 
                 if y == self.AADirection.HORIZONTAL:
-                    clip = self.transpose(clip)
+                    clip, tclips = self.transpose(clip, **kwargs)
+                    kwargs |= tclips
 
         return clip
 
-    def transpose(self, clip: vs.VideoNode) -> ConstantFormatVideoNode:
+    def transpose(self, clip: vs.VideoNode, **kwargs: Any) -> tuple[vs.VideoNode, Mapping[str, vs.VideoNode | None]]:
         """
         Transpose the input clip by swapping its horizontal and vertical axes.
 
@@ -256,7 +236,7 @@ class AntiAliaser(Deinterlacer, ABC):
         Returns:
             The transposed clip.
         """
-        return clip.std.Transpose()
+        return clip.std.Transpose(), {}
 
 
 @dataclass(kw_only=True)
@@ -283,7 +263,7 @@ class SuperSampler(Scaler, AntiAliaser, ABC):
         height: int | None = None,
         shift: tuple[TopShift, LeftShift] = (0, 0),
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Scale the given clip using super sampling method.
 
@@ -299,7 +279,7 @@ class SuperSampler(Scaler, AntiAliaser, ABC):
         Returns:
             The scaled clip.
         """
-        assert check_variable(clip, self.scale)
+        tff_fallback = fallback(kwargs.pop("tff", self.tff), True)
 
         dims = self._wh_norm(clip, width, height)
         dest_dimensions = list(dims)
@@ -323,11 +303,11 @@ class SuperSampler(Scaler, AntiAliaser, ABC):
             is_width = (not x and self.transpose_first) or (not self.transpose_first and x)
 
             if is_width:
-                clip = self.transpose(clip)
+                clip, _ = self.transpose(clip)
 
             while clip.height < dim:
                 delta = max(nshift[x], key=lambda y: abs(y))
-                tff = False if delta < 0 else True if delta > 0 else self.tff
+                tff = False if delta < 0 else True if delta > 0 else tff_fallback
                 offset = -0.25 if tff else 0.25
 
                 for y in range(clip.format.num_planes):
@@ -339,7 +319,7 @@ class SuperSampler(Scaler, AntiAliaser, ABC):
                 clip = self._interpolate(clip, tff, False, True, **kwargs)
 
             if is_width:
-                clip = self.transpose(clip)
+                clip, _ = self.transpose(clip)
 
         if not self.transpose_first:
             nshift.reverse()
@@ -356,15 +336,13 @@ class SuperSampler(Scaler, AntiAliaser, ABC):
                 for i in range(len(ns)):
                     ns[i] *= not noshift[i]
 
-        return ComplexScaler.ensure_obj(self.scaler, self.__class__).scale(  # type: ignore[return-value]
-            clip, width, height, (nshift[1], nshift[0])
-        )
+        return ComplexScaler.ensure_obj(self.scaler, self.__class__).scale(clip, width, height, (nshift[1], nshift[0]))
 
     if TYPE_CHECKING:
 
         def supersample(
-            self, clip: VideoNodeT, rfactor: float = 2.0, shift: tuple[TopShift, LeftShift] = (0, 0), **kwargs: Any
-        ) -> VideoNodeT:
+            self, clip: vs.VideoNode, rfactor: float = 2.0, shift: tuple[TopShift, LeftShift] = (0, 0), **kwargs: Any
+        ) -> vs.VideoNode:
             """
             Supersample a clip by a given scaling factor.
 
@@ -390,7 +368,7 @@ class NNEDI3(SuperSampler):
     """
     Neural Network Edge Directed Interpolation (3rd gen.)
 
-    More informations: https://github.com/sekrit-twc/znedi3
+    More information: https://github.com/sekrit-twc/znedi3
     """
 
     nsize: int = 0
@@ -437,7 +415,7 @@ class NNEDI3(SuperSampler):
     - 1: Weights trained to minimise squared error.
     """
 
-    pscrn: int | None = 1
+    pscrn: int | None = None
     """
     The prescreener used to decide which pixels should be processed by the predictor neural network,
     and which can be handled by simple cubic interpolation.
@@ -451,7 +429,8 @@ class NNEDI3(SuperSampler):
 
     The new prescreener is not available with float input.
 
-    Wrapper default is 1, plugin default is 2 for integer input and 1 for float input.
+    - Wrapper default is 4 for integer input and 1 for float input. When `opencl=True` it is always 1.
+    - Plugin default is 2 for integer input and 1 for float input.
     """
 
     opencl: bool = False
@@ -459,26 +438,6 @@ class NNEDI3(SuperSampler):
     Enables the use of the OpenCL variant.
     """
 
-    @property
-    def _deinterlacer_function(self) -> VSFunctionAllArgs[vs.VideoNode, ConstantFormatVideoNode]:
-        return core.lazy.sneedif.NNEDI3 if self.opencl else core.lazy.znedi3.nnedi3
-
-    def _interpolate(
-        self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
-        field = int(tff) + int(double_rate) * 2
-
-        return self._deinterlacer_function(clip, field, dh, **self.get_deint_args(**kwargs))
-
-    def get_deint_args(self, **kwargs: Any) -> dict[str, Any]:
-        return {
-            "nsize": self.nsize,
-            "nns": self.nns,
-            "qual": self.qual,
-            "etype": self.etype,
-            "pscrn": self.pscrn,
-        } | kwargs
-
     @Scaler.cachedproperty
     def kernel_radius(self) -> int:
         match self.nsize:
@@ -491,6 +450,28 @@ class NNEDI3(SuperSampler):
             case _:
                 return 32
 
+    def get_deint_args(self, *, clip: vs.VideoNode | None = None, **kwargs: Any) -> dict[str, Any]:
+        pscrn = (
+            fallback(self.pscrn, 1 if self.opencl or clip.format.sample_type is vs.FLOAT else 4) if clip else self.pscrn
+        )
+
+        return {
+            "nsize": self.nsize,
+            "nns": self.nns,
+            "qual": self.qual,
+            "etype": self.etype,
+            "pscrn": pscrn,
+        } | kwargs
+
+    @property
+    def _deinterlacer_function(self) -> VSFunctionAllArgs:
+        return core.lazy.sneedif.NNEDI3 if self.opencl else core.lazy.znedi3.nnedi3
+
+    def _interpolate(self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any) -> vs.VideoNode:
+        field = tff + double_rate * 2
+
+        return self._deinterlacer_function(clip, field, dh, **self.get_deint_args(clip=clip, **kwargs))
+
 
 @dataclass
 class EEDI2(SuperSampler):
@@ -563,29 +544,11 @@ class EEDI2(SuperSampler):
     - 1 = Check for spatial consistency of final interpolation directions
     - 2 = Check for junctions and corners
     - 3 = Apply both checks from 1 and 2
-
-    Only `pp=0` and `pp=1` is implemented for the CUDA variant.
     """
 
-    cuda: bool = False
-    """Enables the use of the CUDA variant for processing."""
-
-    @property
-    def _deinterlacer_function(self) -> VSFunctionAllArgs[vs.VideoNode, ConstantFormatVideoNode]:
-        return core.lazy.eedi2cuda.EEDI2 if self.cuda else core.lazy.eedi2.EEDI2
-
-    def _interpolate(
-        self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
-        field = int(tff) + int(double_rate) * 2
-
-        if not dh:
-            clip = clip.std.SeparateFields(tff)
-
-            if not double_rate:
-                clip = clip[::2]
-
-        return self._deinterlacer_function(clip, field, **self.get_deint_args(**kwargs))
+    @Scaler.cachedproperty
+    def kernel_radius(self) -> int:
+        return self.maxd
 
     def get_deint_args(self, **kwargs: Any) -> dict[str, Any]:
         return {
@@ -600,9 +563,20 @@ class EEDI2(SuperSampler):
             "pp": self.pp,
         } | kwargs
 
-    @Scaler.cachedproperty
-    def kernel_radius(self) -> int:
-        return self.maxd
+    @property
+    def _deinterlacer_function(self) -> VSFunctionAllArgs:
+        return core.lazy.eedi2.EEDI2
+
+    def _interpolate(self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any) -> vs.VideoNode:
+        field = tff + double_rate * 2
+
+        if not dh:
+            clip = clip.std.SeparateFields(tff)
+
+            if not double_rate:
+                clip = clip[::2]
+
+        return self._deinterlacer_function(clip, field, **self.get_deint_args(**kwargs))
 
 
 @dataclass
@@ -687,13 +661,15 @@ class EEDI3(SuperSampler):
     - vthresh[2]: Controls the weighting of the interpolation direction.
     """
 
-    sclip: vs.VideoNode | VSFunctionNoArgs[vs.VideoNode, ConstantFormatVideoNode] | None = None
+    sclip: vs.VideoNode | Deinterlacer | VSFunctionNoArgs | None = None
     """
     Provides additional control over the interpolation by using a reference clip.
     If set to None, vertical cubic interpolation is used as a fallback method instead.
+
+    Passing a Deinterlacer object is only supported for pure deinterlacing.
     """
 
-    mclip: vs.VideoNode | VSFunctionNoArgs[vs.VideoNode, ConstantFormatVideoNode] | None = None
+    mclip: vs.VideoNode | VSFunctionNoArgs | None = None
     """
     A mask used to apply edge-directed interpolation only to specified pixels.
     Pixels where the mask value is 0 will be interpolated using cubic linear
@@ -702,71 +678,51 @@ class EEDI3(SuperSampler):
     by limiting edge-directed interpolation to certain pixels.
     """
 
-    def _set_sclip_mclip(self, kwargs: dict[str, Any]) -> dict[str, Any]:
-        sclip, mclip = kwargs.pop("sclip", MISSING), kwargs.pop("mclip", MISSING)
+    @Scaler.cachedproperty
+    def kernel_radius(self) -> int:
+        return self.mdis
 
-        if sclip is not MISSING:
-            self.sclip = sclip
-
-        if mclip is not MISSING:
-            self.mclip = mclip
-
-        return kwargs
-
-    @property
-    def _deinterlacer_function(self) -> VSFunctionAllArgs[vs.VideoNode, ConstantFormatVideoNode]:
-        return core.lazy.eedi3m.EEDI3
-
-    def _interpolate(
-        self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
-        field = int(tff) + int(double_rate) * 2
-
-        kwargs = self.get_deint_args(**kwargs)
-
-        if callable(self.sclip):
-            kwargs.update(sclip=self.sclip(clip))
-
-        if callable(self.mclip):
-            kwargs.update(mclip=self.mclip(clip))
-
-        return self._deinterlacer_function(clip, field, dh, **kwargs)
-
-    def antialias(
-        self, clip: vs.VideoNode, direction: AntiAliaser.AADirection = AntiAliaser.AADirection.BOTH, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
-        kwargs = self._set_sclip_mclip(kwargs)
-
-        if self.sclip and self.double_rate:
-            if callable(self.sclip):
-                self.sclip = self.sclip(clip)
-
-            self.sclip = core.std.Interleave([self.sclip, self.sclip])
-
-        return super().antialias(clip, direction, **kwargs)
-
-    def transpose(self, clip: vs.VideoNode) -> ConstantFormatVideoNode:
-        if isinstance(self.sclip, vs.VideoNode):
-            self.sclip = self.sclip.std.Transpose()
-
-        if isinstance(self.mclip, vs.VideoNode):
-            self.mclip = self.mclip.std.Transpose()
-
-        return super().transpose(clip)
-
-    def scale(
+    def deinterlace(
         self,
         clip: vs.VideoNode,
-        width: int | None = None,
-        height: int | None = None,
-        shift: tuple[TopShift, LeftShift] = (0, 0),
+        *,
+        tff: FieldBasedLike | bool | None = None,
+        double_rate: bool | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
-        return super().scale(clip, width, height, shift, **self._set_sclip_mclip(kwargs))
+    ) -> vs.VideoNode:
+        kwargs = self.get_deint_args(**kwargs)
+
+        sclip, mclip = kwargs.pop("sclip"), kwargs.pop("mclip")
+
+        if isinstance(sclip, Deinterlacer):
+            sclip = sclip.deinterlace(clip, tff=tff, double_rate=double_rate)
+
+        if callable(sclip):
+            sclip = sclip(clip)
+
+        if callable(mclip):
+            mclip = mclip(clip)
+
+        return super().deinterlace(clip, tff=tff, double_rate=double_rate, sclip=sclip, mclip=mclip, **kwargs)
+
+    def bob(self, clip: vs.VideoNode, *, tff: FieldBasedLike | bool | None = None, **kwargs: Any) -> vs.VideoNode:
+        kwargs = self.get_deint_args(**kwargs)
+
+        sclip, mclip = kwargs.pop("sclip"), kwargs.pop("mclip")
+
+        if isinstance(sclip, Deinterlacer):
+            sclip = sclip.bob(clip, tff=tff)
+
+        if callable(sclip):
+            sclip = sclip(clip)
+
+        if callable(mclip):
+            mclip = mclip(clip)
+
+        return super().bob(clip, tff=tff, sclip=sclip, mclip=mclip, **kwargs)
 
     def get_deint_args(self, **kwargs: Any) -> dict[str, Any]:
-        if self.vthresh is None:
-            self.vthresh = (None, None, None)
+        vthresh = (None, None, None) if self.vthresh is None else self.vthresh
 
         return {
             "alpha": self.alpha,
@@ -777,20 +733,72 @@ class EEDI3(SuperSampler):
             "ucubic": self.ucubic,
             "cost3": self.cost3,
             "vcheck": self.vcheck,
-            "vthresh0": self.vthresh[0],
-            "vthresh1": self.vthresh[1],
-            "vthresh2": self.vthresh[2],
+            "vthresh0": vthresh[0],
+            "vthresh1": vthresh[1],
+            "vthresh2": vthresh[2],
             "sclip": self.sclip,
             "mclip": self.mclip,
         } | kwargs
 
-    @Scaler.cachedproperty
-    def kernel_radius(self) -> int:
-        return self.mdis
+    def antialias(
+        self, clip: vs.VideoNode, direction: AntiAliaser.AADirection = AntiAliaser.AADirection.BOTH, **kwargs: Any
+    ) -> vs.VideoNode:
+        kwargs = self.get_deint_args(**kwargs)
 
-    def __vs_del__(self, core_id: int) -> None:
-        self.sclip = None
-        self.mclip = None
+        sclip, mclip = kwargs.pop("sclip"), kwargs.pop("mclip")
+
+        if isinstance(sclip, Deinterlacer):
+            raise CustomValueError("sclip must be a callable or VideoNode", self.antialias)
+
+        if sclip and self.double_rate:
+            if isinstance(sclip, VSFunctionNoArgs):
+                sclip = sclip(clip)
+
+            sclip = core.std.Interleave([sclip, sclip])
+
+        if isinstance(mclip, VSFunctionNoArgs):
+            mclip = mclip(clip)
+
+        return super().antialias(clip, direction, sclip=sclip, mclip=mclip, **kwargs)
+
+    def transpose(
+        self,
+        clip: vs.VideoNode,
+        *,
+        sclip: vs.VideoNode | None = None,
+        mclip: vs.VideoNode | None = None,
+        **kwargs: Any,
+    ) -> tuple[vs.VideoNode, Mapping[str, vs.VideoNode | None]]:
+        # At this point, sclip and mclip can only be a VideoNode or None
+        if isinstance(sclip, vs.VideoNode):
+            sclip = sclip.std.Transpose()
+
+        if isinstance(mclip, vs.VideoNode):
+            mclip = mclip.std.Transpose()
+
+        return clip.std.Transpose(), kwargs | {"sclip": sclip, "mclip": mclip}
+
+    def scale(
+        self,
+        clip: vs.VideoNode,
+        width: int | None = None,
+        height: int | None = None,
+        shift: tuple[TopShift, LeftShift] = (0, 0),
+        **kwargs: Any,
+    ) -> vs.VideoNode:
+        kwargs = self.get_deint_args(**kwargs)
+
+        if kwargs["sclip"] or kwargs["mclip"]:
+            raise CustomNotImplementedError("sclip and mclip are currently not supported.", self.scale)
+
+        return super().scale(clip, width, height, shift, **kwargs)
+
+    @property
+    def _deinterlacer_function(self) -> VSFunctionAllArgs:
+        return core.lazy.eedi3m.EEDI3
+
+    def _interpolate(self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any) -> vs.VideoNode:
+        return self._deinterlacer_function(clip, tff + double_rate * 2, dh, **kwargs)
 
 
 @dataclass
@@ -805,13 +813,16 @@ class SangNom(SuperSampler):
     Must be an integer between 0 and 128, inclusive.
     """
 
+    _static_kernel_radius = 3
+
+    def get_deint_args(self, **kwargs: Any) -> dict[str, Any]:
+        return {"aa": self.aa} | kwargs
+
     @property
-    def _deinterlacer_function(self) -> VSFunctionAllArgs[vs.VideoNode, ConstantFormatVideoNode]:
+    def _deinterlacer_function(self) -> VSFunctionAllArgs:
         return core.lazy.sangnom.SangNom
 
-    def _interpolate(
-        self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
+    def _interpolate(self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any) -> vs.VideoNode:
         if double_rate:
             order = 0
             clip = clip.std.SeparateFields(tff).std.DoubleWeave(tff)
@@ -820,11 +831,6 @@ class SangNom(SuperSampler):
 
         return self._deinterlacer_function(clip, order, dh, **self.get_deint_args(**kwargs))
 
-    def get_deint_args(self, **kwargs: Any) -> dict[str, Any]:
-        return {"aa": self.aa} | kwargs
-
-    _static_kernel_radius = 3
-
 
 @dataclass
 class BWDIF(Deinterlacer):
@@ -832,56 +838,58 @@ class BWDIF(Deinterlacer):
     Motion adaptive deinterlacing based on yadif with the use of w3fdif and cubic interpolation algorithms.
     """
 
-    edeint: vs.VideoNode | VSFunctionNoArgs[vs.VideoNode, ConstantFormatVideoNode] | None = None
+    edeint: vs.VideoNode | Deinterlacer | VSFunctionNoArgs | None = None
     """
     Allows the specification of an external clip from which to take spatial predictions
     instead of having Bwdif use cubic interpolation.
+
     This clip must be the same width, height, and colorspace as the input clip.
+
     If using same rate output, this clip should have the same number of frames as the input.
     If using double rate output, this clip should have twice as many frames as the input.
     """
 
-    @property
-    def _deinterlacer_function(self) -> VSFunctionAllArgs[vs.VideoNode, ConstantFormatVideoNode]:
-        return core.lazy.bwdif.Bwdif
-
-    def _interpolate(
-        self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
-        field = int(tff) + int(double_rate) * 2
-
-        if callable(self.edeint):
-            kwargs.update(edeint=self.edeint(clip))
-
-        return self._deinterlacer_function(clip, field, **self.get_deint_args(**kwargs))
+    _static_kernel_radius = 2
 
     def get_deint_args(self, **kwargs: Any) -> dict[str, Any]:
         return {"edeint": self.edeint} | kwargs
 
-    def __vs_del__(self, core_id: int) -> None:
-        self.edeint = None
+    @property
+    def _deinterlacer_function(self) -> VSFunctionAllArgs:
+        return core.lazy.bwdif.Bwdif
 
-    _static_kernel_radius = 2
+    def _interpolate(self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any) -> vs.VideoNode:
+        field = tff + double_rate * 2
+
+        kwargs = self.get_deint_args(**kwargs)
+
+        edeint = kwargs.pop("edeint")
+
+        if isinstance(edeint, Deinterlacer):
+            kwargs["edeint"] = edeint._interpolate(clip, tff, double_rate, False)
+
+        if callable(edeint):
+            kwargs["edeint"] = edeint(clip)
+
+        return self._deinterlacer_function(clip, field, **kwargs)
 
 
 if TYPE_CHECKING:
     # Let's assume the specialized SuperSampler isn't abstract
     class _ConcreteSuperSampler(SuperSampler):
+        def get_deint_args(self, **kwargs: Any) -> dict[str, Any]: ...
         @property
-        def _deinterlacer_function(self) -> VSFunctionAllArgs[vs.VideoNode, ConstantFormatVideoNode]: ...
+        def _deinterlacer_function(self) -> VSFunctionAllArgs: ...
         def _interpolate(
             self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any
-        ) -> ConstantFormatVideoNode: ...
-        def get_deint_args(self, **kwargs: Any) -> dict[str, Any]: ...
+        ) -> vs.VideoNode: ...
 else:
     _ConcreteSuperSampler = SuperSampler
 
 _SuperSamplerWithNNEDI3DefaultT = TypeVar("_SuperSamplerWithNNEDI3DefaultT", bound=SuperSampler, default=NNEDI3)
 
 
-class SuperSamplerProcess(
-    MixedScalerProcess[_SuperSamplerWithNNEDI3DefaultT, Point], _ConcreteSuperSampler, partial_abstract=True
-):
+class SuperSamplerProcess(MixedScalerProcess[_SuperSamplerWithNNEDI3DefaultT, Point], _ConcreteSuperSampler):
     """
     A utility SuperSampler class that applies a given function to a supersampled clip,
     then downsamples it back using Point.
@@ -889,15 +897,9 @@ class SuperSamplerProcess(
     If used without a specified scaler, it defaults to inheriting from `NNEDI3`.
     """
 
-    _default_scaler = NNEDI3
+    default_scaler = NNEDI3
 
-    def __init__(
-        self,
-        *,
-        function: VSFunctionNoArgs[vs.VideoNode, vs.VideoNode],
-        noshift: bool | Sequence[bool] = True,
-        **kwargs: Any,
-    ) -> None:
+    def __init__(self, *, function: VSFunctionNoArgs, noshift: bool | Sequence[bool] = True, **kwargs: Any) -> None:
         """
         Initialize the SuperSamplerProcess.
 
@@ -930,7 +932,7 @@ class SuperSamplerProcess(
         height: int | None = None,
         shift: tuple[TopShift, LeftShift] = (0, 0),
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         ss_clip = super().scale(clip, width, height, shift, **kwargs)
 
         processed = self.function(ss_clip)
@@ -941,7 +943,7 @@ class SuperSamplerProcess(
                 processed,
                 clip.width,
                 clip.height,
-                tuple([round(s - 1e-6) for s in dim_shifts] for dim_shifts in reversed(self._ss_shifts)),  # type: ignore[return-value, arg-type]
+                tuple([round(s - 1e-6) for s in dim_shifts] for dim_shifts in reversed(self._ss_shifts)),  # type: ignore[arg-type]
             )
             .std.CopyFrameProps(processed)
         )
