@@ -1,25 +1,23 @@
 from __future__ import annotations
 
 from types import NoneType
-from typing import Any, Callable, Generic, Literal, Protocol, Sequence, TypeVar, overload
+from typing import TYPE_CHECKING, Any, Callable, Literal, Protocol, Sequence, overload
 
-from jetpytools import CustomValueError, P, R, to_arr
+from jetpytools import CustomIntEnum, CustomValueError, normalize_seq, to_arr
+
+if TYPE_CHECKING:
+    from vsdenoise import PrefilterLike
 
-from vsdenoise import PrefilterLike
 from vsexprtools import norm_expr
 from vsrgtools import gauss_blur
 from vstools import (
-    ConstantFormatVideoNode,
-    CustomIntEnum,
-    InvalidColorFamilyError,
     Planes,
-    check_variable,
+    UnsupportedColorFamilyError,
     core,
     depth,
     expect_bits,
     join,
     normalize_param_planes,
-    normalize_seq,
     split,
     vs,
 )
@@ -32,7 +30,7 @@ __all__ = [
 ]
 
 
-class F3KDeband(Generic[P, R]):
+class F3KDeband[**P, R]:
     """
     Class decorator that wraps the [f3k_deband][vsdeband.debanders.f3k_deband] function
     and extends its functionality.
@@ -252,10 +250,11 @@ class F3KDeband(Generic[P, R]):
             Configure `MEAN_DIFF` with either individual values or sequences.
 
             You can use either:
+
             - Individual thresholds: ``y1, cb1, cr1, y2, cb2, cr2``
             - Sequences: ``thr_max`` and ``thr_mid``
 
-            ### y1 / cb1 / cr1
+            ## y1 / cb1 / cr1
             These are detail protection thresholds (max difference).
 
             This threshold applies to the maxDif check.
@@ -265,7 +264,7 @@ class F3KDeband(Generic[P, R]):
 
             It helps protect sharp edges and fine details from being blurred by the debanding process.
 
-            ### y2 / cb2 / cr2
+            ## y2 / cb2 / cr2
             These are gradient / texture protection threshold (mid-pair difference).
 
             This threshold applies to the midDif checks.
@@ -366,7 +365,7 @@ def f3k_deband(
     clip: vs.VideoNode,
     radius: int = 16,
     thr: int | Sequence[int] = 96,
-    grain: float | Sequence[float] = 0.0,
+    grain: int | Sequence[int] = 0,
     planes: Planes = None,
     *,
     sample_mode: F3KDeband.SampleMode = F3KDeband.SampleMode.SQUARE,
@@ -379,14 +378,14 @@ def f3k_deband(
     """
     Debanding filter wrapper using the `neo_f3kdb` plugin.
 
-    More informations can be found in the plugin documentation: https://github.com/HomeOfAviSynthPlusEvolution/neo_f3kdb
+    More information can be found in the plugin documentation: <https://github.com/HomeOfAviSynthPlusEvolution/neo_f3kdb>
 
     Args:
         clip: Input clip.
         radius: Radius used for banding detection. Valid range is [1, 255].
         thr: Banding detection threshold(s) for each plane (Y, Cb, Cr). A pixel is considered banded if the difference
             with its reference pixel(s) is less than the corresponding threshold.
-        grain: Amount of grain to add after debanding. Accepts a float or sequence of floats.
+        grain: Amount of grain to add after debanding. Accepts a int or sequence of ints.
         planes: Specifies which planes to process. Default is all planes.
         sample_mode: Strategy used to sample reference pixels. See
             [SampleMode][vsdeband.debanders.F3KDeband.SampleMode].
@@ -400,18 +399,17 @@ def f3k_deband(
 
     Raises:
         CustomValueError: If inconsistent grain parameters are provided across chroma planes.
-        InvalidColorFamilyError: If input clip is not YUV or GRAY.
+        UnsupportedColorFamilyError: If input clip is not YUV or GRAY.
 
     Returns:
         Debanded and optionally grained clip.
     """
-
-    InvalidColorFamilyError.check(clip, (vs.GRAY, vs.YUV), f3k_deband)
+    UnsupportedColorFamilyError.check(clip, (vs.GRAY, vs.YUV), f3k_deband)
 
     clip, bits = expect_bits(clip, 16)
 
-    y, cb, cr = normalize_param_planes(clip, normalize_seq(thr, 3), planes, 0, f3k_deband)
-    grainy, *ngrainc = normalize_param_planes(clip, normalize_seq(grain, 2), planes, 0, f3k_deband)
+    y, cb, cr = normalize_param_planes(clip, normalize_seq(thr, 3), planes, 0)
+    grainy, *ngrainc = normalize_param_planes(clip, normalize_seq(grain, 2), planes, 0)
 
     if len(set(ngrainc)) > 1:
         raise CustomValueError("Inconsistent grain parameters across chroma planes.", f3k_deband)
@@ -436,8 +434,8 @@ def f3k_deband(
         y,
         cb,
         cr,
-        round(grainy * 255 * 0.8),
-        round(ngrainc[0] * 255 * 0.8),
+        grainy,
+        ngrainc[0],
         sample_mode,
         seed,
         blur_first,
@@ -462,7 +460,7 @@ def placebo_deband(
     Debanding wrapper around the `placebo.Deband` filter from the VapourSynth `vs-placebo` plugin.
 
     For full plugin documentation, see:
-    https://github.com/sgt0/vs-placebo?tab=readme-ov-file#deband
+    <https://github.com/sgt0/vs-placebo?tab=readme-ov-file#deband>
 
     Args:
         clip: Input clip.
@@ -480,15 +478,10 @@ def placebo_deband(
     Returns:
         Debanded and optionally grained clip.
     """
+    thr = normalize_param_planes(clip, thr, planes, 0)
+    ngrain = normalize_param_planes(clip, grain, planes, 0)
 
-    assert check_variable(clip, placebo_deband)
-
-    thr = normalize_param_planes(clip, normalize_seq(thr), planes, 0, placebo_deband)
-    ngrain = normalize_param_planes(clip, normalize_seq(grain), planes, 0, placebo_deband)
-
-    def _placebo(
-        clip: ConstantFormatVideoNode, threshold: float, grain_val: float, planes: Sequence[int]
-    ) -> ConstantFormatVideoNode:
+    def _placebo(clip: vs.VideoNode, threshold: float, grain_val: float, planes: Sequence[int]) -> vs.VideoNode:
         plane = 0
 
         if threshold == grain_val == 0:
@@ -497,7 +490,7 @@ def placebo_deband(
         for p in planes:
             plane |= pow(2, p)
 
-        return clip.placebo.Deband(plane, iterations, threshold, radius, grain_val * (1 << 5) * 0.8, **kwargs)
+        return clip.placebo.Deband(plane, iterations, threshold, radius, grain_val, **kwargs)
 
     set_grn = set(ngrain)
 
@@ -523,10 +516,7 @@ def placebo_deband(
     return debanded
 
 
-_Nb = TypeVar("_Nb", int, float, contravariant=True)
-
-
-class _DebanderFunc(Protocol[_Nb]):
+class _DebanderFunc[_Nb: (float, int)](Protocol):
     """
     Protocol for debanding functions.
     """
@@ -536,7 +526,7 @@ class _DebanderFunc(Protocol[_Nb]):
         clip: vs.VideoNode,
         radius: int = ...,
         thr: _Nb | Sequence[_Nb] = ...,
-        grain: float | Sequence[float] = ...,
+        grain: _Nb | Sequence[_Nb] = ...,
         planes: Planes = ...,
     ) -> vs.VideoNode: ...
 
@@ -557,12 +547,11 @@ def mdb_bilateral(
     This function is more of a last resort for extreme banding.
 
     Example usage:
-    ```py
-    from vsdeband import mdb_bilateral, f3k_deband
-    from functools import partial
+        ```py
+        from vsdeband import mdb_bilateral
 
-    debanded = mdb_bilateral(clip, 17, 320)
-    ```
+        debanded = mdb_bilateral(clip, 22, 320)
+        ```
 
     Args:
         clip: Input clip.
@@ -577,8 +566,6 @@ def mdb_bilateral(
     Returns:
         Debanded clip.
     """
-    assert check_variable(clip, mdb_bilateral)
-
     clip, bits = expect_bits(clip, 16)
 
     rad1, rad2, rad3 = round(radius * 4 / 3), round(radius * 2 / 3), round(radius / 3)
