@@ -1,21 +1,21 @@
 from __future__ import annotations
 
+from abc import ABC, abstractmethod
 from enum import auto
-from typing import Any, Callable, Iterable, Literal, Protocol, Sequence, TypeAlias, TypeVar, overload
+from typing import Any, Callable, Iterable, Literal, Protocol, Sequence, overload
 
-from jetpytools import MISSING, CustomEnum, CustomValueError, FuncExcept, MissingT, mod_x, to_arr
+from jetpytools import MISSING, CustomEnum, CustomValueError, EnumABCMeta, FuncExcept, MissingT, mod_x, to_arr
 
 from vsexprtools import norm_expr
-from vskernels import BaseScalerSpecializer, BicubicAuto, Lanczos, LeftShift, Scaler, ScalerLike, TopShift
-from vsmasktools import adg_mask
+from vsjetpack import TypeVar
+from vskernels import BicubicAuto, Lanczos, LeftShift, Scaler, ScalerLike, ScalerSpecializer, TopShift
 from vsrgtools import BlurMatrix
 from vstools import (
     ColorRange,
-    ConstantFormatVideoNode,
     ConvMode,
     Planes,
     UnsupportedColorFamilyError,
-    check_variable,
+    check_variable_resolution,
     core,
     get_lowest_values,
     get_neutral_values,
@@ -39,7 +39,7 @@ __all__ = [
 ]
 
 
-EdgeLimits = tuple[float | Sequence[float] | bool, float | Sequence[float] | bool]
+type EdgeLimits = tuple[float | Sequence[float] | bool, float | Sequence[float] | bool]
 """
 Tuple representing lower and upper edge limits for each plane.
 
@@ -64,20 +64,13 @@ class _GrainerFunc(Protocol):
     ) -> vs.VideoNode: ...
 
 
-class _PostProcessFunc(Protocol):
+_ScalerWithLanczosDefaultT = TypeVar("_ScalerWithLanczosDefaultT", bound=Scaler, default=Lanczos)
+
+
+# TODO: class ScalerTwoPasses[_ScalerT: Scaler = Lanczos] python 3.13
+class ScalerTwoPasses(ScalerSpecializer[_ScalerWithLanczosDefaultT]):
     """
-    Protocol for a post-processing function applied after graining.
-    """
-
-    def __call__(self, grained: vs.VideoNode) -> vs.VideoNode: ...
-
-
-_ScalerT = TypeVar("_ScalerT", bound=Scaler)
-
-
-class ScalerTwoPasses(BaseScalerSpecializer[_ScalerT], Scaler, partial_abstract=True):
-    """
-    Abstract scaler class that applies scaling in two passes.
+    Scaler class that applies scaling in two passes.
     """
 
     _default_scaler = Lanczos
@@ -89,17 +82,22 @@ class ScalerTwoPasses(BaseScalerSpecializer[_ScalerT], Scaler, partial_abstract=
         height: int | None = None,
         shift: tuple[TopShift, LeftShift] = (0, 0),
         **kwargs: Any,
-    ) -> vs.VideoNode | ConstantFormatVideoNode:
-        assert check_variable(clip, self.__class__)
+    ) -> vs.VideoNode:
+        assert check_variable_resolution(clip, self.__class__)
+
+        if any(shift):
+            raise CustomValueError("Shifting is unsupported.", self.__class__, shift)
 
         width, height = self._wh_norm(clip, width, height)
 
         if width / clip.width > 1.5 or height / clip.height > 1.5:
             # If the scale is too big, we need to scale it in two passes, else the window
             # will be too big and the grain will be dampened down too much
-            mod = max(clip.format.subsampling_w, clip.format.subsampling_h) << 1
             clip = super().scale(
-                clip, mod_x((width + clip.width) / 2, mod), mod_x((height + clip.height) / 2, mod), **kwargs
+                clip,
+                mod_x((width + clip.width) / 2, 2**clip.format.subsampling_w),
+                mod_x((height + clip.height) / 2, 2**clip.format.subsampling_h),
+                **kwargs,
             )
 
         return super().scale(clip, width, height, (0, 0), **kwargs)
@@ -127,16 +125,16 @@ class GrainFactoryBicubic(BicubicAuto):
         super().__init__(sharp / -50 + 1, None, **kwargs)
 
 
-class AbstractGrainer:
+class AbstractGrainer(ABC):
     """
     Abstract grainer base class.
     """
 
-    def __call__(self, clip: vs.VideoNode, /, **kwargs: Any) -> vs.VideoNode | GrainerPartial:
-        raise NotImplementedError
+    @abstractmethod
+    def __call__(self, clip: vs.VideoNode, /, **kwargs: Any) -> vs.VideoNode | GrainerPartial: ...
 
 
-class Grainer(AbstractGrainer, CustomEnum):
+class Grainer(AbstractGrainer, CustomEnum, metaclass=EnumABCMeta):
     """
     Enum representing different grain/noise generation algorithms.
     """
@@ -182,7 +180,9 @@ class Grainer(AbstractGrainer, CustomEnum):
         scale: float | tuple[float, float] = 1.0,
         scaler: ScalerLike = LanczosTwoPasses,
         temporal: float | tuple[float, int] = (0.0, 0),
-        post_process: _PostProcessFunc | Iterable[_PostProcessFunc] | None = None,
+        post_process: Callable[[vs.VideoNode], vs.VideoNode]
+        | Iterable[Callable[[vs.VideoNode], vs.VideoNode]]
+        | None = None,
         protect_edges: bool | EdgeLimits = True,
         protect_neutral_chroma: bool | None = None,
         luma_scaling: float | None = None,
@@ -200,7 +200,9 @@ class Grainer(AbstractGrainer, CustomEnum):
         scale: float | tuple[float, float] = 1.0,
         scaler: ScalerLike = LanczosTwoPasses,
         temporal: float | tuple[float, int] = (0.0, 0),
-        post_process: _PostProcessFunc | Iterable[_PostProcessFunc] | None = None,
+        post_process: Callable[[vs.VideoNode], vs.VideoNode]
+        | Iterable[Callable[[vs.VideoNode], vs.VideoNode]]
+        | None = None,
         protect_edges: bool | EdgeLimits = True,
         protect_neutral_chroma: bool | None = None,
         luma_scaling: float | None = None,
@@ -218,7 +220,9 @@ class Grainer(AbstractGrainer, CustomEnum):
         scale: float | tuple[float, float] = 1.0,
         scaler: ScalerLike = LanczosTwoPasses,
         temporal: float | tuple[float, int] = (0.0, 0),
-        post_process: _PostProcessFunc | Iterable[_PostProcessFunc] | None = None,
+        post_process: Callable[[vs.VideoNode], vs.VideoNode]
+        | Iterable[Callable[[vs.VideoNode], vs.VideoNode]]
+        | None = None,
         protect_edges: bool | EdgeLimits = True,
         protect_neutral_chroma: bool | None = None,
         luma_scaling: float | None = None,
@@ -238,7 +242,9 @@ class Grainer(AbstractGrainer, CustomEnum):
         scale: float | tuple[float, float] = 1.0,
         scaler: ScalerLike = LanczosTwoPasses,
         temporal: float | tuple[float, int] = (0.0, 0),
-        post_process: _PostProcessFunc | Iterable[_PostProcessFunc] | None = None,
+        post_process: Callable[[vs.VideoNode], vs.VideoNode]
+        | Iterable[Callable[[vs.VideoNode], vs.VideoNode]]
+        | None = None,
         protect_edges: bool | EdgeLimits = True,
         protect_neutral_chroma: bool | None = None,
         luma_scaling: float | None = None,
@@ -257,7 +263,9 @@ class Grainer(AbstractGrainer, CustomEnum):
         scale: float | tuple[float, float] = 1.0,
         scaler: ScalerLike = LanczosTwoPasses,
         temporal: float | tuple[float, int] = (0.0, 0),
-        post_process: _PostProcessFunc | Iterable[_PostProcessFunc] | None = None,
+        post_process: Callable[[vs.VideoNode], vs.VideoNode]
+        | Iterable[Callable[[vs.VideoNode], vs.VideoNode]]
+        | None = None,
         protect_edges: bool | EdgeLimits = True,
         protect_neutral_chroma: bool | None = None,
         luma_scaling: float | None = None,
@@ -274,7 +282,9 @@ class Grainer(AbstractGrainer, CustomEnum):
         scale: float | tuple[float, float] = 1.0,
         scaler: ScalerLike = LanczosTwoPasses,
         temporal: float | tuple[float, int] = (0.0, 0),
-        post_process: _PostProcessFunc | Iterable[_PostProcessFunc] | None = None,
+        post_process: Callable[[vs.VideoNode], vs.VideoNode]
+        | Iterable[Callable[[vs.VideoNode], vs.VideoNode]]
+        | None = None,
         protect_edges: bool | EdgeLimits = True,
         protect_neutral_chroma: bool | None = None,
         luma_scaling: float | None = None,
@@ -292,7 +302,9 @@ class Grainer(AbstractGrainer, CustomEnum):
         scale: float | tuple[float, float] = 1.0,
         scaler: ScalerLike = LanczosTwoPasses,
         temporal: float | tuple[float, int] = (0.0, 0),
-        post_process: _PostProcessFunc | Iterable[_PostProcessFunc] | None = None,
+        post_process: Callable[[vs.VideoNode], vs.VideoNode]
+        | Iterable[Callable[[vs.VideoNode], vs.VideoNode]]
+        | None = None,
         protect_edges: bool | EdgeLimits = True,
         protect_neutral_chroma: bool | None = None,
         luma_scaling: float | None = None,
@@ -310,7 +322,9 @@ class Grainer(AbstractGrainer, CustomEnum):
         scale: float | tuple[float, float] = 1.0,
         scaler: ScalerLike = LanczosTwoPasses,
         temporal: float | tuple[float, int] = (0.0, 0),
-        post_process: _PostProcessFunc | Iterable[_PostProcessFunc] | None = None,
+        post_process: Callable[[vs.VideoNode], vs.VideoNode]
+        | Iterable[Callable[[vs.VideoNode], vs.VideoNode]]
+        | None = None,
         protect_edges: bool | EdgeLimits = True,
         protect_neutral_chroma: bool | None = None,
         luma_scaling: float | None = None,
@@ -327,7 +341,9 @@ class Grainer(AbstractGrainer, CustomEnum):
         scale: float | tuple[float, float] = 1.0,
         scaler: ScalerLike = LanczosTwoPasses,
         temporal: float | tuple[float, int] = (0.0, 0),
-        post_process: _PostProcessFunc | Iterable[_PostProcessFunc] | None = None,
+        post_process: Callable[[vs.VideoNode], vs.VideoNode]
+        | Iterable[Callable[[vs.VideoNode], vs.VideoNode]]
+        | None = None,
         protect_edges: bool | EdgeLimits = True,
         protect_neutral_chroma: bool | None = None,
         luma_scaling: float | None = None,
@@ -370,7 +386,7 @@ class Grainer(AbstractGrainer, CustomEnum):
 
                    - True: Use legal range based on clip format.
                    - False: Disable edge protection.
-                   - Tuple: Specify custom edge limits per plane (see `EdgeLimits`).
+                   - Tuple: Specify custom edge limits per plane (see [EdgeLimits][vsdeband.noise.EdgeLimits]).
 
             protect_neutral_chroma:
                 Whether to disable graining on neutral chroma.
@@ -388,7 +404,7 @@ class Grainer(AbstractGrainer, CustomEnum):
                    - ``neutral_out``: (Boolean) Output the neutral layer instead of the merged clip.
 
         Returns:
-            Grained video clip, or a `GrainerPartial` if `clip` is not provided.
+            Grained video clip, or a [GrainerPartial][vsdeband.noise.GrainerPartial] if `clip` is not provided.
         """
         kwargs.update(
             strength=strength,
@@ -405,8 +421,6 @@ class Grainer(AbstractGrainer, CustomEnum):
         if clip is MISSING:
             return GrainerPartial(self, **kwargs)
 
-        assert check_variable(clip, self.name)
-
         if self == Grainer.PLACEBO:
             assert static is False, "PlaceboGrain does not support static noise!"
 
@@ -462,14 +476,14 @@ class Grainer(AbstractGrainer, CustomEnum):
 
 
 def _apply_grainer(
-    clip: ConstantFormatVideoNode,
+    clip: vs.VideoNode,
     grainer_function: _GrainerFunc,
     strength: float | Sequence[float],
     scale: float | tuple[float, float],
     scaler: ScalerLike,
     temporal: float | tuple[float, int],
     protect_edges: bool | EdgeLimits,
-    post_process: Callable[..., vs.VideoNode] | Iterable[Callable[..., vs.VideoNode]] | None,
+    post_process: Callable[[vs.VideoNode], vs.VideoNode] | Iterable[Callable[[vs.VideoNode], vs.VideoNode]] | None,
     protect_neutral_chroma: bool | None,
     luma_scaling: float | None,
     planes: Planes,
@@ -492,12 +506,10 @@ def _apply_grainer(
     protect_neutral_chroma_blend = kwargs.pop("protect_neutral_chroma_blend", scale_delta(2, 8, clip))
     neutral_out = kwargs.pop("neutral_out", False)
 
-    (scalex, scaley), mod = scale, max(clip.format.subsampling_w, clip.format.subsampling_h) << 1
-
     # Making a neutral blank clip
     base_clip = clip.std.BlankClip(
-        mod_x(clip.width / scalex, mod),
-        mod_x(clip.height / scaley, mod),
+        mod_x(clip.width / scale[0], 2**clip.format.subsampling_w),
+        mod_x(clip.height / scale[1], 2**clip.format.subsampling_h),
         length=clip.num_frames + temporal_rad * 2,
         color=get_neutral_values(clip),
         keep=True,
@@ -551,13 +563,15 @@ def _apply_grainer(
             grained = _protect_neutral_chroma(clip, grained, base_clip, protect_neutral_chroma_blend, planes, func)
 
         if luma_scaling is not None:
+            from vsmasktools import adg_mask
+
             grained = core.std.MaskedMerge(base_clip, grained, adg_mask(clip, luma_scaling), planes)
 
     return core.std.MergeDiff(clip, grained, planes) if not neutral_out else grained
 
 
 def _protect_pixel_range(
-    clip: ConstantFormatVideoNode,
+    clip: vs.VideoNode,
     grained: vs.VideoNode,
     low: list[float],
     high: list[float],
@@ -580,7 +594,7 @@ def _protect_pixel_range(
 
 
 def _protect_neutral_chroma(
-    clip: ConstantFormatVideoNode,
+    clip: vs.VideoNode,
     grained: vs.VideoNode,
     base_clip: vs.VideoNode,
     blend: float,
@@ -606,8 +620,10 @@ def _protect_neutral_chroma(
             return core.std.MaskedMerge(
                 grained, base_clip, norm_expr(split(clip), "x y = x z = and mask_max *", planes, func=func)
             )
-        case _:
-            raise UnsupportedColorFamilyError("Can't use `protect_neutral_chroma=True` when input clip is GRAY", func)
+
+    raise UnsupportedColorFamilyError(
+        func, clip, (vs.YUV, vs.RGB), "Can't use `protect_neutral_chroma=True` when input clip is {wrong}"
+    )
 
 
 class GrainerPartial(AbstractGrainer):
@@ -623,8 +639,8 @@ class GrainerPartial(AbstractGrainer):
             grainer: [Grainer][vsdeband.noise.Grainer] enumeration.
             **kwargs: Arguments for the specified grainer.
         """
-        self.grainer = grainer
-        self.kwargs = kwargs
+        self._grainer = grainer
+        self._kwargs = kwargs
 
     def __call__(self, clip: vs.VideoNode, /, **kwargs: Any) -> vs.VideoNode:
         """
@@ -637,10 +653,10 @@ class GrainerPartial(AbstractGrainer):
         Returns:
             Processed clip.
         """
-        return self.grainer(clip, **self.kwargs | kwargs)
+        return self._grainer(clip, **self._kwargs | kwargs)
 
 
-GrainerLike: TypeAlias = Grainer | GrainerPartial
+type GrainerLike = Grainer | GrainerPartial
 """
 Grainer-like type, which can be a single grainer or a partial grainer.
 """
