@@ -4,10 +4,9 @@ This module implements dehalo functions with complex masking abilities.
 
 from __future__ import annotations
 
-from contextlib import suppress
-from typing import Any, Callable, Generic, Iterator, Mapping, Sequence
+from typing import Any, Callable, Iterator, Mapping, Sequence
 
-from jetpytools import CustomIndexError, P, R, normalize_seq
+from jetpytools import CustomIndexError, FuncExcept, normalize_seq
 
 from vsaa import NNEDI3, SuperSamplerProcess
 from vsdenoise import Prefilter
@@ -24,15 +23,14 @@ from vsmasktools import (
 from vsrgtools import BlurMatrixBase, box_blur, contrasharpening_dehalo
 from vsscale import pre_ss as pre_supersample
 from vstools import (
-    ConstantFormatVideoNode,
     ConvMode,
-    FuncExcept,
     FunctionUtil,
-    InvalidColorFamilyError,
     OneDimConvMode,
     Planes,
+    UnsupportedColorFamilyError,
+    VSFunctionPlanesArgs,
+    VSObjectABC,
     check_progressive,
-    check_variable,
     get_y,
     join,
     limiter,
@@ -40,15 +38,14 @@ from vstools import (
     scale_mask,
     split,
     vs,
-    vs_object,
 )
 
-from .alpha import IterArr, VSFunctionPlanesArgs, dehalo_alpha
+from .alpha import IterArr, dehalo_alpha
 
 __all__ = ["fine_dehalo", "fine_dehalo2"]
 
 
-class FineDehalo(Generic[P, R]):
+class FineDehalo[**P, R]:
     """
     Class decorator that wraps the [fine_dehalo][vsdehalo.fine_dehalo] function
     and extends its functionality.
@@ -83,7 +80,7 @@ class FineDehalo(Generic[P, R]):
         # Misc params
         planes: Planes = 0,
         func: FuncExcept | None = None,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         The fine_dehalo mask.
 
@@ -108,9 +105,9 @@ class FineDehalo(Generic[P, R]):
         """
         return self.Masks(clip, rx, ry, edgemask, thmi, thma, thlimi, thlima, exclude, edgeproc, planes, func).MAIN
 
-    class Masks(Mapping[str, ConstantFormatVideoNode], vs_object):
+    class Masks(VSObjectABC, Mapping[str, vs.VideoNode]):
         """
-        Class for creating and storing intermediate masks used in the `fine_dehalo` function.
+        Class for creating and storing intermediate masks used in the [fine_dehalo][vsdehalo.fine_dehalo] function.
 
         Each step of the masking pipeline is stored with a descriptive key, allowing for
         debugging or further processing.
@@ -156,14 +153,14 @@ class FineDehalo(Generic[P, R]):
             """
             func = func or self.__class__
 
-            InvalidColorFamilyError.check(clip, (vs.GRAY, vs.YUV), func)
+            UnsupportedColorFamilyError.check(clip, (vs.GRAY, vs.YUV), func)
 
             thmif, thmaf, thlimif, thlimaf = [
                 [scale_mask(x, 8, clip) for x in th]
-                for th in [normalize_seq(th, clip.format.num_planes) for th in (thmi, thma, thlimi, thlima)]  # type: ignore[union-attr]
+                for th in [normalize_seq(th, clip.format.num_planes) for th in (thmi, thma, thlimi, thlima)]
             ]
-            exclude = normalize_seq(exclude, clip.format.num_planes)  # type: ignore[union-attr]
-            edgeproc = normalize_seq(edgeproc, clip.format.num_planes)  # type: ignore[union-attr]
+            exclude = normalize_seq(exclude, clip.format.num_planes)
+            edgeproc = normalize_seq(edgeproc, clip.format.num_planes)
             planes = normalize_planes(clip, planes)
             work_clip = get_y(clip) if planes == [0] else clip
 
@@ -251,7 +248,7 @@ class FineDehalo(Generic[P, R]):
             self._shr_med = shr_med
             self._main = mask
 
-        def __getitem__(self, index: str) -> ConstantFormatVideoNode:
+        def __getitem__(self, index: str) -> vs.VideoNode:
             index = index.upper()
 
             if index in self._names:
@@ -266,38 +263,33 @@ class FineDehalo(Generic[P, R]):
             return len(self._names)
 
         @property
-        def EDGES(self) -> ConstantFormatVideoNode:  # noqa: N802
+        def EDGES(self) -> vs.VideoNode:  # noqa: N802
             return self._edges
 
         @property
-        def SHARP_EDGES(self) -> ConstantFormatVideoNode:  # noqa: N802
+        def SHARP_EDGES(self) -> vs.VideoNode:  # noqa: N802
             return self._strong
 
         @property
-        def LARGE_EDGES(self) -> ConstantFormatVideoNode:  # noqa: N802
+        def LARGE_EDGES(self) -> vs.VideoNode:  # noqa: N802
             return self._large
 
         @property
-        def IGNORE_DETAILS(self) -> ConstantFormatVideoNode:  # noqa: N802
+        def IGNORE_DETAILS(self) -> vs.VideoNode:  # noqa: N802
             return self._light
 
         @property
-        def SHRINK(self) -> ConstantFormatVideoNode:  # noqa: N802
+        def SHRINK(self) -> vs.VideoNode:  # noqa: N802
             return self._shrink
 
         @property
-        def SHRINK_EDGES_EXCL(self) -> ConstantFormatVideoNode:  # noqa: N802
+        def SHRINK_EDGES_EXCL(self) -> vs.VideoNode:  # noqa: N802
             return self._shr_med
 
         @property
-        def MAIN(self) -> ConstantFormatVideoNode:  # noqa: N802
+        def MAIN(self) -> vs.VideoNode:  # noqa: N802
             return self._main
 
-        def __vs_del__(self, core_id: int) -> None:
-            for name in ["_edges", "_strong", "_large", "_light", "_shrink", "_shr_med", "_main"]:
-                with suppress(AttributeError):
-                    delattr(self, name)
-
 
 @FineDehalo
 def fine_dehalo(
@@ -337,15 +329,16 @@ def fine_dehalo(
     The parameter `ss` can be configured per iteration while `blur`, `lowsens`, `highsens`, `darkstr` and `brightstr`
     can be configured per plane and per iteration. You can specify:
 
-        - A single value: applies to all iterations and all planes.
-        - A tuple of values: interpreted as iteration-wise.
-        - A list inside the tuple: interpreted as per-plane for a specific iteration.
+    - A single value: applies to all iterations and all planes.
+    - A tuple of values: interpreted as iteration-wise.
+    - A list inside the tuple: interpreted as per-plane for a specific iteration.
 
     For example:
         `blur=(1.4, [1.4, 1.65], [1.5, 1.4, 1.45])` implies 3 iterations:
-            - 1st: 1.4 for all planes
-            - 2nd: 1.4 for luma, 1.65 for both chroma planes
-            - 3rd: 1.5 for luma, 1.4 for U, 1.45 for V
+
+        - 1st: 1.4 for all planes
+        - 2nd: 1.4 for luma, 1.65 for both chroma planes
+        - 3rd: 1.5 for luma, 1.4 for U, 1.45 for V
 
     The parameters `thmi`, `thma`, `thlimi`, `thlima`, `exclude` and `edgeproc` can be configured per plane.
 
@@ -367,7 +360,7 @@ def fine_dehalo(
         brightstr: Strength factor for suppressing bright halos.
         rx: Horizontal radius for halo removal.
         ry: Vertical radius for halo removal. Defaults to `rx` if not set.
-        edgemask: Edge detection object to use. Defaults to `Robinson3`.
+        edgemask: Edge detection object to use. Defaults to [Robinson3][vsmasktools.Robinson3].
         thmi: Minimum threshold for sharp edge selection; isolates only the strongest (line-like) edges.
         thma: Maximum threshold for sharp edge selection; filters out weaker edges.
         thlimi: Minimum threshold for including edges that were previously ignored.
@@ -381,7 +374,7 @@ def fine_dehalo(
         attach_masks: Stores the masks as frame properties in the output clip.
             The prop names are `FineDehaloMask` + the masking step.
         func: An optional function to use for error handling.
-        **kwargs: Additionnal advanced parameters.
+        **kwargs: Additional advanced parameters.
 
     Returns:
         Dehaloed clip.
@@ -482,8 +475,6 @@ def fine_dehalo2(
     """
     func = fine_dehalo2
 
-    assert check_variable(clip, func)
-
     work_clip, *chroma = split(clip)
 
     mask_h = mask_v = None
