@@ -5,17 +5,7 @@ from typing import Any
 from jetpytools import FuncExcept
 
 from vsexprtools import norm_expr
-from vstools import (
-    ConstantFormatVideoNode,
-    VSFunctionNoArgs,
-    check_ref_clip,
-    check_variable,
-    core,
-    join,
-    shift_clip,
-    shift_clip_multi,
-    vs,
-)
+from vstools import VSFunctionNoArgs, check_ref_clip, core, join, shift_clip, shift_clip_multi, vs
 
 from .funcs import vinverse
 from .utils import telecine_patterns
@@ -25,7 +15,7 @@ __all__ = ["deblend", "deblend_bob", "deblend_fix_kf", "deblending_helper"]
 
 def deblending_helper(
     deblended: vs.VideoNode, fieldmatched: vs.VideoNode, length: int = 5, func: FuncExcept | None = None
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Helper function to select a deblended clip pattern from a fieldmatched clip.
 
@@ -40,8 +30,6 @@ def deblending_helper(
     """
     func = func or deblending_helper
 
-    assert check_variable(deblended, func)
-    assert check_variable(fieldmatched, func)
     check_ref_clip(fieldmatched, deblended, func)
 
     inters = telecine_patterns(fieldmatched, deblended, length, func)
@@ -59,16 +47,16 @@ def deblending_helper(
         length=length,
     )
 
-    return core.std.FrameEval(fieldmatched, lambda n, f: inters[f[0][0, 0]], index_src)
+    return core.std.FrameEval(fieldmatched, lambda n, f: inters[int(f[0][0, 0])], index_src)
 
 
 def deblend(
     clip: vs.VideoNode,
     fieldmatched: vs.VideoNode | None = None,
-    decomber: VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | None = vinverse,
+    decomber: VSFunctionNoArgs | None = vinverse,
     func: FuncExcept | None = None,
     **kwargs: Any,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Automatically deblends if normal field matching leaves 2 blends every 5 frames. Adopted from jvsfunc.
 
@@ -96,9 +84,9 @@ def deblend(
 
 def deblend_bob(
     bobbed: vs.VideoNode, fieldmatched: vs.VideoNode | None = None, func: FuncExcept | None = None
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
-    Stronger version of `deblend` that uses a bobbed clip to deblend. Adopted from jvsfunc.
+    Stronger version of [deblend][vsdeinterlace.deblend] that uses a bobbed clip to deblend. Adopted from jvsfunc.
 
     Args:
         bobbed: Bobbed source.
@@ -110,8 +98,6 @@ def deblend_bob(
     """
     func = func or deblend_bob
 
-    assert check_variable(bobbed, func)
-
     ab0, bc0, c0 = shift_clip_multi(bobbed[::2], (0, 2))
     bc1, ab1, a1 = shift_clip_multi(bobbed[1::2])
 
@@ -123,11 +109,9 @@ def deblend_bob(
     return deblended
 
 
-def deblend_fix_kf(
-    deblended: vs.VideoNode, fieldmatched: vs.VideoNode, func: FuncExcept | None = None
-) -> ConstantFormatVideoNode:
+def deblend_fix_kf(deblended: vs.VideoNode, fieldmatched: vs.VideoNode, func: FuncExcept | None = None) -> vs.VideoNode:
     """
-    Should be used after deblend/_bob to fix scene changes. Adopted from jvsfunc.
+    Should be used after [deblend_bob][vsdeinterlace.deblend_bob] to fix scene changes. Adopted from jvsfunc.
 
     Args:
         deblended: Deblended clip.
@@ -139,8 +123,6 @@ def deblend_fix_kf(
     """
     func = func or deblend_fix_kf
 
-    assert check_variable(deblended, func)
-
     shifted_clips = shift_clip_multi(deblended)
     prop_srcs = shift_clip_multi(fieldmatched, (0, 1))
 
@@ -148,4 +130,4 @@ def deblend_fix_kf(
         prop_srcs, "x._Combed x.VFMSceneChange and y.VFMSceneChange 2 0 ? 1 ?", format=vs.GRAY8, func=func
     )
 
-    return core.std.FrameEval(deblended, lambda n, f: shifted_clips[f[0][0, 0]], index_src)
+    return core.std.FrameEval(deblended, lambda n, f: shifted_clips[int(f[0][0, 0])], index_src)
