@@ -3,11 +3,11 @@ from __future__ import annotations
 from typing import Any
 
 from vstools import (
-    ConstantFormatVideoNode,
     FieldBased,
     FieldBasedLike,
+    FramerateMismatchError,
     FunctionUtil,
-    InvalidFramerateError,
+    UnsupportedFramerateError,
     VSFunctionKwArgs,
     VSFunctionNoArgs,
     core,
@@ -27,7 +27,7 @@ def sivtc(
     pattern: int = 0,
     tff: FieldBasedLike | bool | None = None,
     ivtc_cycle: IVTCycles = IVTCycles.CYCLE_10,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Simplest form of a fieldmatching function.
 
@@ -56,12 +56,12 @@ def jivtc(
     pattern: int,
     tff: FieldBasedLike | bool | None = None,
     chroma_only: bool = True,
-    postprocess: VSFunctionKwArgs[vs.VideoNode, vs.VideoNode] = deblend,
+    postprocess: VSFunctionKwArgs = deblend,
     postdecimate: IVTCycles | None = IVTCycles.CYCLE_05,
     ivtc_cycle: IVTCycles = IVTCycles.CYCLE_10,
     final_ivtc_cycle: IVTCycles = IVTCycles.CYCLE_08,
     **kwargs: Any,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     This function should only be used when a normal ivtc or ivtc + bobber leaves chroma blend to every fourth frame.
     You can disable chroma_only to use it for luma as well, but it is not recommended.
@@ -80,7 +80,7 @@ def jivtc(
 
     tff = FieldBased.from_param_or_video(tff, clip, True, jivtc).is_tff
 
-    InvalidFramerateError.check(jivtc, clip, (30000, 1001))
+    UnsupportedFramerateError.check(clip, (30000, 1001), jivtc)
 
     ivtced = clip.std.SeparateFields(tff).std.DoubleWeave(tff)
     ivtced = ivtc_cycle.decimate(ivtced, pattern)
@@ -102,9 +102,9 @@ def vfm(
     clip: vs.VideoNode,
     tff: FieldBasedLike | bool | None = None,
     mode: VFMMode = VFMMode.TWO_WAY_MATCH_THIRD_COMBED,
-    postprocess: vs.VideoNode | VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | None = None,
+    postprocess: vs.VideoNode | VSFunctionNoArgs | None = None,
     **kwargs: Any,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Perform field matching using VFM.
 
@@ -113,22 +113,31 @@ def vfm(
     You can pass a post-processing clip or function that will act on leftover combed frames.
     If you pass a clip, it will replace combed frames with that clip. If you pass a function,
     it will run that function on your input clip and replace combed frames with it.
+    The output of the clip or function must have the same framerate as the input clip.
 
-    Example usage:
-
-    ```py
-        # Run vsaa.NNEDI3 on combed frames
-        vfm(clip, postprocess=NNEDI3().deinterlace)
-    ```
+    Example:
+        ```py
+        # Run vsaa.NNEDI3 on leftover combed frames
+        vfm(clip, postprocess=NNEDI3(double_rate=False).deinterlace)
+        ```
 
     Args:
         clip: Input clip to field matching telecine on.
-        tff: Field order of the input clip. If None, it will be automatically detected.
-        mode: VFM matching mode. For more information, see [VFMMode][vsdeinterlace.VFMMode]. Default:
-            VFMMode.TWO_WAY_MATCH_THIRD_COMBED.
-        postprocess: Optional function or clip to process combed frames. If a function is passed, it should take a clip
-            as input and return a clip as output. If a clip is passed, it will be used as the postprocessed clip.
-        **kwargs: Additional keyword arguments to pass to VFM. For a list of parameters, see the VIVTC documentation.
+        tff:
+            Field order of the input clip.
+            If None, it will be automatically detected.
+        mode:
+            VFM matching mode. For more information, see [VFMMode][vsdeinterlace.VFMMode].
+            Default: VFMMode.TWO_WAY_MATCH_THIRD_COMBED.
+        postprocess:
+            Optional function or clip to process combed frames.
+                If a function is passed, it should take a clip as input and return a clip as output.
+                If a clip is passed, it will be used as the postprocessed clip.
+
+                The output of the clip or function must have the same framerate as the input clip.
+        **kwargs:
+            Additional keyword arguments to pass to VFM.
+            For a list of parameters, see the VIVTC documentation.
 
     Returns:
         Field matched clip with progressive frames.
@@ -158,12 +167,19 @@ def vfm(
         if callable(postprocess):
             postprocess = postprocess(kwargs.get("clip2", clip))
 
+        FramerateMismatchError.check(
+            vfm,
+            clip,
+            postprocess,
+            message="The post-processing function must return a clip with the same framerate as the input clip!",
+        )
+
         fieldmatch = find_prop_rfs(fieldmatch, postprocess, "_Combed", "==", 1)
 
     return func.return_clip(fieldmatch)
 
 
-def vdecimate(clip: vs.VideoNode, weight: float = 0.0, **kwargs: Any) -> ConstantFormatVideoNode:
+def vdecimate(clip: vs.VideoNode, weight: float = 0.0, **kwargs: Any) -> vs.VideoNode:
     """
     Perform frame decimation using VDecimate.
 
