@@ -1,17 +1,14 @@
 from __future__ import annotations
 
 from functools import cache
-from inspect import signature
 from types import MappingProxyType
-from typing import TYPE_CHECKING, Any, Callable, Generic, Sequence
+from typing import Any, Callable, Sequence
 
 from jetpytools import (
     CustomEnum,
     CustomRuntimeError,
     CustomStrEnum,
     CustomValueError,
-    P,
-    R,
     cachedproperty,
     fallback,
     interleave_arr,
@@ -21,13 +18,11 @@ from jetpytools import (
 from vsexprtools import norm_expr
 from vskernels import Point
 from vstools import (
-    ConstantFormatVideoNode,
+    ColorRange,
     FunctionUtil,
     Planes,
-    UnsupportedVideoFormatError,
     check_progressive,
     check_ref_clip,
-    check_variable,
     core,
     depth,
     get_y,
@@ -52,13 +47,14 @@ def wnnm(
     """
     Weighted Nuclear Norm Minimization Denoise algorithm.
 
-    Block matching, which is popularized by BM3D, finds similar blocks and then stacks together in a 3-D group.
+    Block matching, popularized by BM3D, finds similar blocks and stacks them together in a 3-D group.
     The similarity between these blocks allows details to be preserved during denoising.
 
-    In contrast to BM3D, which denoises the 3-D group based on frequency domain filtering,
-    WNNM utilizes weighted nuclear norm minimization, a kind of low rank matrix approximation.
-    Because of this, WNNM exhibits less blocking and ringing artifact compared to BM3D,
-    but the computational complexity is much higher. This stage is called collaborative filtering in BM3D.
+    In contrast to BM3D, which denoises the 3-D group via frequency domain filtering,
+    WNNM (Weighted Nuclear Norm Minimization) uses a low-rank matrix approximation approach.
+    Because of this, WNNM tends to exhibit fewer blocking and ringing artifacts than BM3D,
+    at the cost of much higher computational complexity.
+    This stage is analogous to the "collaborative filtering" step in BM3D.
 
     For more information, see the [WNNM README](https://github.com/WolframRhodium/VapourSynth-WNNM).
 
@@ -90,7 +86,6 @@ def wnnm(
     Returns:
         Denoised clip.
     """
-
     assert check_progressive(clip, wnnm)
 
     func = FunctionUtil(clip, wnnm, planes, bitdepth=32)
@@ -111,11 +106,7 @@ def wnnm(
             previous = denoised
         else:
             previous = norm_expr(
-                [func.work_clip, previous, denoised],
-                "x y - {merge_factor} * z +",
-                planes,
-                merge_factor=merge_factor,
-                func=func.func,
+                [func.work_clip, previous, denoised], f"x y - {merge_factor} * z +", planes, func=func.func
             )
 
         denoised = core.wnnm.WNNM(previous, sigma, **dkwargs)
@@ -123,21 +114,8 @@ def wnnm(
     return func.return_clip(denoised)
 
 
-# TODO: remove this when vs-stubs will be a thing™
-if TYPE_CHECKING:
-    from vapoursynth import Function, Plugin
-
-    class _VSFunction(Function):
-        def __call__(self, *args: Any, **kwargs: Any) -> ConstantFormatVideoNode: ...
-
-    class _VSPlugin(Plugin):
-        BM3D: _VSFunction
-        BM3Dv2: _VSFunction
-        VAggregate: _VSFunction
-
-
-def _clean_keywords(kwargs: dict[str, Any], function: _VSFunction) -> dict[str, Any]:
-    return {k: v for k, v in kwargs.items() if k in signature(function).parameters}
+def _clean_keywords(kwargs: dict[str, Any], function: vs.Function) -> dict[str, Any]:
+    return {k: v for k, v in kwargs.items() if k in function.__signature__.parameters}
 
 
 class UnsupportedProfileError(CustomValueError):
@@ -146,7 +124,7 @@ class UnsupportedProfileError(CustomValueError):
     """
 
 
-class BM3D(Generic[P, R]):
+class BM3D[**P, R]:
     """
     Class decorator that wraps the [bm3d][vsdenoise.blockmatch.bm3d] function
     and extends its functionality.
@@ -168,7 +146,9 @@ class BM3D(Generic[P, R]):
         AUTO = "auto"
         """
         Automatically selects the best available backend.
+
         Selection priority: "CUDA_RTC" → "CUDA" → "HIP" → "SYCL" → "CPU" → "OLD".
+
         When the filter chain is executed within vspreview, the priority of "cuda_rtc" and "cuda" is reversed.
         """
 
@@ -207,7 +187,7 @@ class BM3D(Generic[P, R]):
             """
             Resolves the appropriate BM3D backend to use based on availability and context.
 
-            If the current instance is not BM3D.Backend.AUTO, it returns itself.
+            If the current instance is not [AUTO][vsdenoise.blockmatch.BM3D.Backend.AUTO], it returns itself.
             Otherwise, it attempts to select the best available backend.
 
             Raises:
@@ -240,7 +220,7 @@ class BM3D(Generic[P, R]):
             )
 
         @property
-        def plugin(self) -> _VSPlugin:
+        def plugin(self) -> vs.Plugin:
             """
             Returns the appropriate BM3D plugin based on the current backend.
 
@@ -254,7 +234,7 @@ class BM3D(Generic[P, R]):
         Enum representing the available BM3D profiles, each with default parameter settings.
 
         For more detailed information on these profiles,
-        refer to the original [documentation](https://github.com/HomeOfVapourSynthEvolution/VapourSynth-BM3D#profile-default)
+        refer to the original [documentation](https://github.com/HomeOfVapourSynthEvolution/VapourSynth-BM3D#profile-default).
         """
 
         FAST = "fast"
@@ -539,7 +519,7 @@ def bm3d(
     final_args: dict[str, Any] | None = None,
     planes: Planes = None,
     **kwargs: Any,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Block-Matching and 3D filtering (BM3D) is a state-of-the-art algorithm for image denoising.
 
@@ -576,7 +556,7 @@ def bm3d(
         ref: A clip to be used as the basic estimate. It replaces BM3D's internal basic estimate and serves as the
             reference for the final estimate. Must be of the same format and dimensions as the input clip. Either `ref`
             or `pre` can be specified, not both. Defaults to None.
-        backend: The backend to use for processing. Defaults to BM3D.Backend.A
+        backend: The backend to use for processing. Defaults to BM3D.Backend.AUTO.
         basic_args: Additional arguments to pass to the basic estimate step. Defaults to None.
         final_args: Additional arguments to pass to the final estimate step. Defaults to None.
         planes: Planes to process. Default to all.
@@ -603,7 +583,7 @@ def bm3d(
         ref = check_ref_clip(clip, ref, func)
 
     radius_basic, radius_final = normalize_seq(tr, 2)
-    nsigma = normalize_param_planes(clip, sigma, planes, 0, func)
+    nsigma = normalize_param_planes(clip, sigma, planes, 0)
 
     backend = backend.resolve()
     nbasic_args = fallback(basic_args, {})
@@ -616,11 +596,11 @@ def bm3d(
         raise UnsupportedProfileError("The VERY_NOISY profile is only supported with BM3D.Backend.OLD.", func)
 
     def _bm3d_wolfram(
-        preclip: ConstantFormatVideoNode,
-        pre: ConstantFormatVideoNode | None,
-        ref: ConstantFormatVideoNode | None,
+        preclip: vs.VideoNode,
+        pre: vs.VideoNode | None,
+        ref: vs.VideoNode | None,
         chroma: bool = False,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Internal function for WolframRhodium implementation.
         """
@@ -647,21 +627,33 @@ def bm3d(
         return final
 
     def _bm3d_mawen(
-        preclip: ConstantFormatVideoNode,
-        pre: ConstantFormatVideoNode | None,
-        ref: ConstantFormatVideoNode | None,
-    ) -> ConstantFormatVideoNode:
+        preclip: vs.VideoNode,
+        pre: vs.VideoNode | None,
+        ref: vs.VideoNode | None,
+    ) -> vs.VideoNode:
         """
         Internal function for mawen1250 implementation.
         """
 
-        preclip = core.bm3d.RGB2OPP(preclip, preclip.format.sample_type)
+        preclip = (
+            core.bm3d.RGB2OPP(preclip, preclip.format.sample_type)
+            if preclip.format.color_family != vs.GRAY
+            else depth(preclip, range_out=ColorRange.FULL)
+        )
 
         if pre:
-            pre = core.bm3d.RGB2OPP(pre, pre.format.sample_type)
+            pre = (
+                core.bm3d.RGB2OPP(pre, pre.format.sample_type)
+                if pre.format.color_family != vs.GRAY
+                else depth(pre, range_out=ColorRange.FULL)
+            )
 
         if ref:
-            ref = core.bm3d.RGB2OPP(ref, ref.format.sample_type)
+            ref = (
+                core.bm3d.RGB2OPP(ref, ref.format.sample_type)
+                if ref.format.color_family != vs.GRAY
+                else depth(ref, range_out=ColorRange.FULL)
+            )
 
         if not ref:
             b_args = profile.basic_args(radius_basic) | nbasic_args | kwargs
@@ -695,9 +687,11 @@ def bm3d(
         if 0 in nsigma:
             final = join({p: preclip if s == 0 else final for p, s in zip(range(3), nsigma)}, vs.YUV)
 
-        return core.bm3d.OPP2RGB(final, preclip.format.sample_type)
-
-    assert check_variable(clip, func)
+        return (
+            core.bm3d.OPP2RGB(final, preclip.format.sample_type)
+            if preclip.format.color_family != vs.GRAY
+            else depth(final, range_out=preclip)
+        )
 
     if clip.format.color_family == vs.RGB:
         if backend == BM3D.Backend.OLD:
@@ -723,6 +717,7 @@ def bm3d(
     prepre = depth(pre, 32) if pre else pre
     preref = depth(ref, 32) if ref else ref
 
+    # YUV444 path
     if clip.format.color_family == vs.YUV and {clip.format.subsampling_w, clip.format.subsampling_h} == {0}:
         if backend == BM3D.Backend.OLD:
             point = Point()
@@ -747,12 +742,10 @@ def bm3d(
 
         return depth(denoised, clip)
 
+    # GRAY or subsampled YUV
     if backend == BM3D.Backend.OLD:
-        raise UnsupportedVideoFormatError(
-            "When using `BM3D.Backend.OLD`, the input clip must be in YUV444 or RGB format.",
-            func,
-            clip.format.color_family,
-        )
+        # Will error if clip is subsampled.
+        return _bm3d_mawen(clip, pre, ref)
 
     denoised = _bm3d_wolfram(preclip, prepre, preref)
 
