@@ -6,38 +6,32 @@ from __future__ import annotations
 
 import math
 from functools import cache
-from typing import TYPE_CHECKING, Any, Iterator, Literal, Mapping, Sequence, TypeAlias, overload
+from typing import TYPE_CHECKING, Any, Iterator, Literal, Mapping, Self, Sequence, overload
 
-from jetpytools import KwargsNotNone, MismatchError, classproperty, fallback
-from typing_extensions import Self, deprecated
-from vapoursynth import Plugin
-
-from vstools import (
-    ConstantFormatVideoNode,
+from jetpytools import (
     CustomEnum,
     CustomIntEnum,
     CustomOverflowError,
     CustomRuntimeError,
     CustomValueError,
-    FieldBased,
     FuncExcept,
-    Planes,
+    KwargsNotNone,
+    MismatchError,
     SupportsFloatOrIndex,
-    check_progressive,
-    core,
-    flatten,
-    get_depth,
-    get_sample_type,
-    vs,
+    classproperty,
+    fallback,
 )
 
-__all__ = ["DFTTest", "SLocationLike", "fft3d"]
+from vsjetpack import require_jet_dependency
+from vstools import Planes, check_progressive, core, flatten, vs
+
+__all__ = ["DFTTest", "SLocationLike"]
 
 
 class _BackendBase(CustomEnum):
     kwargs: dict[str, Any]
 
-    def DFTTest(self, clip: vs.VideoNode, *args: Any, **kwargs: Any) -> ConstantFormatVideoNode:  # noqa: N802
+    def DFTTest(self, clip: vs.VideoNode, *args: Any, **kwargs: Any) -> vs.VideoNode:  # noqa: N802
         self = self.resolve()
 
         if self == DFTTest.Backend.OLD:
@@ -68,17 +62,17 @@ class _BackendBase(CustomEnum):
         )
 
     @property
-    def plugin(self) -> Plugin:
+    def plugin(self) -> vs.Plugin:
         return getattr(core.lazy, self.resolve().value)
 
 
-Frequency: TypeAlias = float
+type Frequency = float
 """
 Represents the frequency value in the frequency domain.
 
 The value is a `float` that represents the magnitude or position in the frequency spectrum.
 """
-Sigma: TypeAlias = float
+type Sigma = float
 """
 Represents the sigma value, which is typically associated with noise standard deviation.
 
@@ -104,7 +98,7 @@ class DFTTest:
 
         class InterMode(CustomEnum):
             """
-            Interpolation modes for sigma values in `SLocation`.
+            Interpolation modes for sigma values in [SLocation][vsdenoise.DFTTest.SLocation].
 
             Defines how sigma values should be interpolated across frequency locations.
             """
@@ -201,7 +195,7 @@ class DFTTest:
                     digits: The precision of the frequency values (default is 3 decimal places).
 
                 Returns:
-                    The interpolated `SLocation` object.
+                    The interpolated [SLocation][vsdenoise.DFTTest.SLocation] object.
                 """
 
             @overload
@@ -246,7 +240,8 @@ class DFTTest:
                     digits: The precision of the frequency values (default is 3 decimal places).
 
                 Returns:
-                    The interpolated `SLocation` object or a `MultiDim` object if multiple locations are provided.
+                    The interpolated [SLocation][vsdenoise.DFTTest.SLocation] object or a `MultiDim` object
+                        if multiple locations are provided.
                 """
                 if not locations and location is not None:
                     return DFTTest.SLocation.from_param(location).interpolate(self, res, digits)
@@ -374,7 +369,7 @@ class DFTTest:
             Reverses the frequency-sigma pairs, inverting the frequency range and reversing the order of sigma values.
 
             Returns:
-                A new `SLocation` instance with reversed frequency-sigma pairs.
+                A new [SLocation][vsdenoise.DFTTest.SLocation] instance with reversed frequency-sigma pairs.
             """
             return self.__class__(dict(zip((1 - f for f in reversed(self.frequencies)), list(reversed(self.sigmas)))))
 
@@ -424,13 +419,14 @@ class DFTTest:
         @classmethod
         def from_param(cls, location: SLocationLike | Literal[False]) -> Self:
             """
-            Converts a frequency-sigma pair or a literal `False` to an `SLocation` instance.
+            Converts a frequency-sigma pair or a literal `False`
+            to an [SLocation][vsdenoise.DFTTest.SLocation] instance.
 
             Args:
                 location: A frequency-sigma pair, `False` for no processing.
 
             Returns:
-                An `SLocation` instance`.
+                An [SLocation][vsdenoise.DFTTest.SLocation] instance`.
             """
             if location is False:
                 return cls.NoProcess
@@ -441,6 +437,7 @@ class DFTTest:
 
             return cls(location)
 
+        @require_jet_dependency("scipy")
         def interpolate(
             self, method: InterMode = InterMode.LINEAR, res: int = 20, digits: int = 3
         ) -> DFTTest.SLocation:
@@ -453,7 +450,7 @@ class DFTTest:
                 digits: The precision of the frequency values (default is 3 decimal places).
 
             Returns:
-                A new `SLocation` instance with interpolated sigma values.
+                A new [SLocation][vsdenoise.DFTTest.SLocation] instance with interpolated sigma values.
             """
             from scipy.interpolate import interp1d
 
@@ -468,11 +465,11 @@ class DFTTest:
         @classmethod
         def NoProcess(cls) -> Self:  # noqa: N802
             """
-            Returns a pre-defined `SLocation` instance that performs no processing
+            Returns a pre-defined [SLocation][vsdenoise.DFTTest.SLocation] instance that performs no processing
             (i.e., sigma is zero for all locations).
 
             Returns:
-                A `SLocation` instance with no processing.
+                A [SLocation][vsdenoise.DFTTest.SLocation] instance with no processing.
             """
             return cls({0: 0, 1: 0})
 
@@ -601,35 +598,35 @@ class DFTTest:
         """
         Hanning window (0). A raised cosine window with good frequency resolution and low spectral leakage.
 
-        See: https://en.wikipedia.org/wiki/Window_function#Hann_(Hanning)_window
+        See: <https://en.wikipedia.org/wiki/Window_function#Hann_(Hanning)_window>
         """
 
         HAMMING = 1
         """
         Hamming window (1). Similar to Hanning but with slightly reduced side lobe attenuation.
 
-        See: https://en.wikipedia.org/wiki/Window_function#Hamming_window
+        See: <https://en.wikipedia.org/wiki/Window_function#Hamming_window>
         """
 
         BLACKMAN = 2
         """
         Blackman window (2). Better side lobe attenuation than Hamming or Hanning at the cost of wider main lobe.
 
-        See: https://en.wikipedia.org/wiki/Window_function#Blackman_window
+        See: <https://en.wikipedia.org/wiki/Window_function#Blackman_window>
         """
 
         BLACKMAN_HARRIS_4TERM = 3
         """
         4-term Blackman-Harris window (3). Low side lobes, ideal for high dynamic range applications.
 
-        See: https://en.wikipedia.org/wiki/Window_function#Blackman%E2%80%93Harris_window
+        See: <https://en.wikipedia.org/wiki/Window_function#Blackman%E2%80%93Harris_window>
         """
 
         KAISER_BESSEL = 4
         """
         Kaiser-Bessel window (4). Adjustable window (beta parameter) balancing resolution and leakage.
 
-        See: https://en.wikipedia.org/wiki/Window_function#Kaiser_window
+        See: <https://en.wikipedia.org/wiki/Window_function#Kaiser_window>
         """
 
         BLACKMAN_HARRIS_7TERM = 5
@@ -637,50 +634,51 @@ class DFTTest:
         7-term Blackman-Harris window (5). Extended version with even greater side lobe suppression.
 
         See:
-            - https://ccrma.stanford.edu/~jos/Windows/Blackman_Harris_Window_Family.html
-            - https://github.com/hukenovs/blackman_harris_win
+
+        - <https://ccrma.stanford.edu/~jos/Windows/Blackman_Harris_Window_Family.html>
+        - <https://github.com/hukenovs/blackman_harris_win>
         """
 
         FLAT_TOP = 6
         """
         Flat top window (6). Optimized for amplitude accuracy and minimizing scalloping loss.
 
-        See: https://en.wikipedia.org/wiki/Window_function#Flat_top_window
+        See: <https://en.wikipedia.org/wiki/Window_function#Flat_top_window>
         """
 
         RECTANGULAR = 7
         """
         Rectangular window (7). Equivalent to no windowing; highest resolution, but worst leakage.
 
-        See: https://en.wikipedia.org/wiki/Window_function#Rectangular_window
+        See: <https://en.wikipedia.org/wiki/Window_function#Rectangular_window>
         """
 
         BARLETT = 8
         """
         Bartlett window (8). Also known as a triangular window; moderate leakage reduction.
 
-        See: https://en.wikipedia.org/wiki/Window_function#Bartlett_window
+        See: <https://en.wikipedia.org/wiki/Window_function#Bartlett_window>
         """
 
         BARLETT_HANN = 9
         """
         Bartlett-Hann window (9). Hybrid of Bartlett and Hanning with a smoother taper.
 
-        See: https://en.wikipedia.org/wiki/Window_function#Bartlett%E2%80%93Hann_window
+        See: <https://en.wikipedia.org/wiki/Window_function#Bartlett%E2%80%93Hann_window>
         """
 
         NUTTALL = 10
         """
         Nuttall window (10). Four-term cosine window with very low side lobes.
 
-        See: https://en.wikipedia.org/wiki/Window_function#Nuttall_window,_continuous_first_derivative
+        See: <https://en.wikipedia.org/wiki/Window_function#Nuttall_window,_continuous_first_derivative>
         """
 
         BLACKMAN_NUTTALL = 11
         """
         Blackman-Nuttall window (11). Variant of Nuttall with even stronger side lobe suppression.
 
-        See: https://en.wikipedia.org/wiki/Window_function#Blackman%E2%80%93Nuttall_window
+        See: <https://en.wikipedia.org/wiki/Window_function#Blackman%E2%80%93Nuttall_window>
         """
 
     class Backend(_BackendBase):
@@ -850,7 +848,7 @@ class DFTTest:
             Additional configuration parameters for the backend.
             """
 
-            def DFTTest(self, clip: vs.VideoNode, *args: Any, **kwargs: Any) -> ConstantFormatVideoNode:  # noqa: N802
+            def DFTTest(self, clip: vs.VideoNode, *args: Any, **kwargs: Any) -> vs.VideoNode:  # noqa: N802
                 """
                 Applies the DFTTest denoising filter using the plugin associated with the selected backend.
 
@@ -872,7 +870,7 @@ class DFTTest:
                 """
                 Resolves the appropriate DFTTest backend to use based on availability.
 
-                If the current instance is not DFTTest.Backend.AUTO, it returns itself.
+                If the current instance is not [AUTO][vsdenoise.DFTTest.Backend.AUTO], it returns itself.
                 Otherwise, it attempts to select the best available backend.
 
                 Raises:
@@ -884,7 +882,7 @@ class DFTTest:
                 ...
 
             @property
-            def plugin(self) -> Plugin:
+            def plugin(self) -> vs.Plugin:
                 """
                 Returns the appropriate DFTTest plugin based on the current backend.
 
@@ -973,7 +971,7 @@ class DFTTest:
         """
         Denoises a clip using Discrete Fourier Transform (DFT).
 
-        More informations:
+        More information:
             - [VapourSynth DFTTest plugin](https://github.com/HomeOfVapourSynthEvolution/VapourSynth-DFTTest/blob/master/README.md)
             - [AviSynth DFTTest docs](http://avisynth.nl/index.php/Dfttest)
             - [vs-dfttest2 docstring](https://github.com/AmusementClub/vs-dfttest2/blob/573bb36c53df93c46a38926c7c654569e3679732/dfttest2.py#L614-L764)
@@ -1100,7 +1098,7 @@ The `SLocationLike` type can be one of the following:
 
 - `int` or `float`:
   A single frequency value (for 1D frequency location).
-- `DFTTest.SLocation`:
+- [DFTTest.SLocation][vsdenoise.DFTTest.SLocation]:
   A structured class for defining frequency locations in a more complex manner.
 - `Sequence[Frequency, Sigma]`:
   A sequence (e.g., list or tuple) that alternates between `Frequency` and `Sigma` values.
@@ -1118,36 +1116,3 @@ ensuring every frequency has an associated sigma.
 
 SLocationT = SLocationLike
 """Deprecated alias of SLocationLike."""
-
-
-@deprecated("`fft3d` is permanently deprecated and known to contain many bugs. Use with caution.")
-def fft3d(clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
-    """
-    Applies FFT3DFilter, a 3D frequency-domain filter used for strong denoising and mild sharpening.
-
-    This filter processes frames using the Fast Fourier Transform (FFT) in the frequency domain.
-    Unlike local filters, FFT3DFilter performs block-based, non-local processing.
-
-       - [Official documentation](https://github.com/myrsloik/VapourSynth-FFT3DFilter/blob/master/doc/fft3dfilter.md)
-       - [Possibly faster implementation](https://github.com/AmusementClub/VapourSynth-FFT3DFilter/releases)
-
-    Note: Sigma values are internally scaled according to bit depth, unlike when using the plugin directly.
-
-    Args:
-        clip: Input video clip.
-        **kwargs: Additional parameters passed to the FFT3DFilter plugin.
-
-    Returns:
-        A heavily degraded version of DFTTest, with added banding and color shifts.
-    """
-    kwargs |= {"interlaced": FieldBased.from_video(clip, False, fft3d).is_inter}
-
-    # fft3dfilter requires sigma values to be scaled to bit depth
-    # https://github.com/myrsloik/VapourSynth-FFT3DFilter/blob/master/doc/fft3dfilter.md#scaling-parameters-according-to-bit-depth
-    sigma_multiplier = 1.0 / 256.0 if get_sample_type(clip) is vs.FLOAT else 1 << (get_depth(clip) - 8)
-
-    for sigma in ["sigma", "sigma2", "sigma3", "sigma4", "smin ", "smax"]:
-        if sigma in kwargs:
-            kwargs[sigma] *= sigma_multiplier
-
-    return core.fft3dfilter.FFT3DFilter(clip, **kwargs)
