@@ -2,10 +2,10 @@ from __future__ import annotations
 
 from typing import Any, overload
 
-from jetpytools import CustomRuntimeError
+from jetpytools import CustomRuntimeError, CustomValueError
 
 from vsrgtools import MeanMode
-from vstools import CustomValueError, FormatsMismatchError, Planes, VSFunctionKwArgs, VSFunctionNoArgs, vs
+from vstools import FormatsMismatchError, Planes, VSFunctionKwArgs, VSFunctionNoArgs, vs
 
 __all__ = [
     "frequency_merge",
@@ -14,10 +14,7 @@ __all__ = [
 
 @overload
 def frequency_merge(
-    *_clips: vs.VideoNode
-    | tuple[
-        vs.VideoNode, VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | VSFunctionKwArgs[vs.VideoNode, vs.VideoNode] | None
-    ],
+    *_clips: vs.VideoNode | tuple[vs.VideoNode, VSFunctionNoArgs | VSFunctionKwArgs | None],
     mode_high: MeanMode | vs.VideoNode = MeanMode.LEHMER,
     mode_low: MeanMode | vs.VideoNode = MeanMode.ARITHMETIC,
     lowpass: None = None,
@@ -31,20 +28,17 @@ def frequency_merge(
     *_clips: vs.VideoNode,
     mode_high: MeanMode | vs.VideoNode = MeanMode.LEHMER,
     mode_low: MeanMode | vs.VideoNode = MeanMode.ARITHMETIC,
-    lowpass: VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | VSFunctionKwArgs[vs.VideoNode, vs.VideoNode],
+    lowpass: VSFunctionNoArgs | VSFunctionKwArgs,
     planes: Planes = None,
     **kwargs: Any,
 ) -> vs.VideoNode: ...
 
 
 def frequency_merge(
-    *_clips: vs.VideoNode
-    | tuple[
-        vs.VideoNode, VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | VSFunctionKwArgs[vs.VideoNode, vs.VideoNode] | None
-    ],
+    *_clips: vs.VideoNode | tuple[vs.VideoNode, VSFunctionNoArgs | VSFunctionKwArgs | None],
     mode_high: MeanMode | vs.VideoNode = MeanMode.LEHMER,
     mode_low: MeanMode | vs.VideoNode = MeanMode.ARITHMETIC,
-    lowpass: VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | VSFunctionKwArgs[vs.VideoNode, vs.VideoNode] | None = None,
+    lowpass: VSFunctionNoArgs | VSFunctionKwArgs | None = None,
     planes: Planes = None,
     **kwargs: Any,
 ) -> vs.VideoNode:
@@ -75,10 +69,10 @@ def frequency_merge(
             lowpass function can either be a standard function or one that supports per-plane filtering.
         mode_high: The mean mode to use for the high frequency components or specifying the clip with the high frequency
             components. If a clip is passed, it must be one of the clips provided in `_clips`. Defaults to
-            `MeanMode.LEHMER`.
+            [MeanMode.LEHMER][vsrgtools.MeanMode.LEHMER].
         mode_low: The mean mode to use for the low frequency components or specifying the clip with the low frequency
             components. If a clip is passed, it must be one of the clips provided in `_clips`. Defaults to
-            `MeanMode.ARITHMETIC`.
+            [MeanMode.ARITHMETIC][vsrgtools.MeanMode.ARITHMETIC].
         lowpass: A global lowpass function applied to all provided clips. If set, ``_clips`` must consist solely of
             VideoNodes.
         planes: Planes to process. If specified, this will be passed to the lowpass functions. If None, it won't be
@@ -90,9 +84,7 @@ def frequency_merge(
     """
 
     clips = list[vs.VideoNode]()
-    lowpass_funcs = list[
-        VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | VSFunctionKwArgs[vs.VideoNode, vs.VideoNode] | None
-    ]()
+    lowpass_funcs = list[VSFunctionNoArgs | VSFunctionKwArgs | None]()
 
     for c in _clips:
         if isinstance(c, tuple):
