@@ -4,29 +4,13 @@ This module contains general denoising functions built on top of base denoisers.
 
 from __future__ import annotations
 
-from typing import Any, Callable, Literal, Sequence, overload
+from typing import Any, Literal, Sequence, overload
 
-from jetpytools import MISSING, CustomRuntimeError, FuncExcept, MissingT
+from jetpytools import MISSING, CustomRuntimeError, FuncExcept, KwargsNotNone, MissingT, fallback, normalize_seq
 
 from vsexprtools import ExprOp, ExprVars, combine_expr, norm_expr
 from vskernels import Catrom, Kernel, KernelLike, Lanczos, Scaler, ScalerLike
-from vstools import (
-    ConstantFormatVideoNode,
-    HoldsVideoFormat,
-    KwargsNotNone,
-    Planes,
-    VideoFormatLike,
-    VSFunctionNoArgs,
-    check_ref_clip,
-    check_variable_format,
-    fallback,
-    get_color_family,
-    join,
-    normalize_planes,
-    normalize_seq,
-    scale_delta,
-    vs,
-)
+from vstools import Planes, VSFunctionNoArgs, check_ref_clip, get_color_family, join, normalize_planes, scale_delta, vs
 
 from .mvtools import MotionVectors, MVTools, MVToolsPreset, refine_blksize
 from .prefilters import PrefilterLike
@@ -42,15 +26,14 @@ __all__ = [
 def mc_degrain(
     clip: vs.VideoNode,
     vectors: MotionVectors | None = None,
-    prefilter: vs.VideoNode | PrefilterLike | VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | None = None,
-    mfilter: vs.VideoNode | VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | None = None,
+    prefilter: vs.VideoNode | PrefilterLike | VSFunctionNoArgs | None = None,
+    mfilter: vs.VideoNode | VSFunctionNoArgs | None = None,
     preset: MVToolsPreset = ...,
     tr: int = 1,
     blksize: int | tuple[int, int] = 16,
     overlap: int | tuple[int, int] = 2,
     refine: int = 1,
     thsad: int | tuple[int, int] = 400,
-    thsad2: int | tuple[int | None, int | None] | None = None,
     thsad_recalc: int | None = None,
     limit: int | tuple[int | None, int | None] | None = None,
     thscd: int | tuple[int | None, int | None] | None = None,
@@ -63,15 +46,14 @@ def mc_degrain(
 def mc_degrain(
     clip: vs.VideoNode,
     vectors: MotionVectors | None = None,
-    prefilter: vs.VideoNode | PrefilterLike | VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | None = None,
-    mfilter: vs.VideoNode | VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | None = None,
+    prefilter: vs.VideoNode | PrefilterLike | VSFunctionNoArgs | None = None,
+    mfilter: vs.VideoNode | VSFunctionNoArgs | None = None,
     preset: MVToolsPreset = ...,
     tr: int = 1,
     blksize: int | tuple[int, int] = 16,
     overlap: int | tuple[int, int] = 2,
     refine: int = 1,
     thsad: int | tuple[int, int] = 400,
-    thsad2: int | tuple[int | None, int | None] | None = None,
     thsad_recalc: int | None = None,
     limit: int | tuple[int | None, int | None] | None = None,
     thscd: int | tuple[int | None, int | None] | None = None,
@@ -85,15 +67,14 @@ def mc_degrain(
 def mc_degrain(
     clip: vs.VideoNode,
     vectors: MotionVectors | None = None,
-    prefilter: vs.VideoNode | PrefilterLike | VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | None = None,
-    mfilter: vs.VideoNode | VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | None = None,
+    prefilter: vs.VideoNode | PrefilterLike | VSFunctionNoArgs | None = None,
+    mfilter: vs.VideoNode | VSFunctionNoArgs | None = None,
     preset: MVToolsPreset = ...,
     tr: int = 1,
     blksize: int | tuple[int, int] = 16,
     overlap: int | tuple[int, int] = 2,
     refine: int = 1,
     thsad: int | tuple[int, int] = 400,
-    thsad2: int | tuple[int | None, int | None] | None = None,
     thsad_recalc: int | None = None,
     limit: int | tuple[int | None, int | None] | None = None,
     thscd: int | tuple[int | None, int | None] | None = None,
@@ -105,15 +86,14 @@ def mc_degrain(
 def mc_degrain(
     clip: vs.VideoNode,
     vectors: MotionVectors | None = None,
-    prefilter: vs.VideoNode | PrefilterLike | VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | None = None,
-    mfilter: vs.VideoNode | VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | None = None,
+    prefilter: vs.VideoNode | PrefilterLike | VSFunctionNoArgs | None = None,
+    mfilter: vs.VideoNode | VSFunctionNoArgs | None = None,
     preset: MVToolsPreset = MVToolsPreset.HQ_SAD,
     tr: int = 1,
     blksize: int | tuple[int, int] = 16,
     overlap: int | tuple[int, int] = 2,
     refine: int = 1,
     thsad: int | tuple[int, int] = 400,
-    thsad2: int | tuple[int | None, int | None] | None = None,
     thsad_recalc: int | None = None,
     limit: int | tuple[int | None, int | None] | None = None,
     thscd: int | tuple[int | None, int | None] | None = None,
@@ -139,9 +119,6 @@ def mc_degrain(
         thsad: Defines the soft threshold of block sum absolute differences. Blocks with SAD above this threshold have
             zero weight for averaging (denoising). Blocks with low SAD have highest weight. The remaining weight is
             taken from pixels of source clip.
-        thsad2: Define the SAD soft threshold for frames with the largest temporal distance. The actual SAD threshold
-            for each reference frame is interpolated between thsad (nearest frames) and thsad2 (furthest frames). Only
-            used with the FLOAT MVTools plugin.
         thsad_recalc: Only bad quality new vectors with a SAD above this will be re-estimated by search. thsad value is
             scaled to 8x8 block size.
         limit: Maximum allowed change in pixel values.
@@ -171,7 +148,7 @@ def mc_degrain(
             blksize = refine_blksize(blksize)
             mv.recalculate(thsad=thsad_recalc, blksize=blksize, overlap=refine_blksize(blksize, overlap))
 
-    den = mv.degrain(mfilter, mv.clip, None, tr, thsad, thsad2, limit, thscd, planes=planes)
+    den = mv.degrain(mfilter, mv.clip, None, tr, thsad, limit, thscd, planes)
 
     return (den, mv) if export_globals else den
 
@@ -180,10 +157,10 @@ def mc_clamp(
     flt: vs.VideoNode,
     src: vs.VideoNode,
     mv_obj: MVTools,
-    clamp: int | float | tuple[int | float, int | float] = 0,
+    clamp: float | tuple[float, float] = 0,
     func: FuncExcept | None = None,
     **kwargs: Any,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Motion-compensated clamping of a filtered clip against the source.
 
@@ -205,7 +182,7 @@ def mc_clamp(
             Values are scaled according to clip bit depth.
             Defaults to 0 (no additional clamping margin).
         func: Function returned for custom error handling. This should only be set by VS package developers.
-        **kwargs: Additional keyword arguments passed to `mv_obj.compensate`.
+        **kwargs: Additional keyword arguments passed to [mv_obj.compensate][vsdenoise.MVTools.compensate].
 
     Returns:
         The motion-compensated clamped clip.
@@ -232,28 +209,6 @@ def mc_clamp(
     )
 
 
-class _LanczosChroma(Lanczos):
-    def __init__(
-        self,
-        taps: float = 3,
-        *,
-        format: int
-        | VideoFormatLike
-        | HoldsVideoFormat
-        | None
-        | Callable[[ConstantFormatVideoNode], int | VideoFormatLike | HoldsVideoFormat | None] = None,
-        **kwargs: Any,
-    ) -> None:
-        super().__init__(taps, format=format, **kwargs)
-
-    def get_params_args(
-        self, is_descale: bool, clip: vs.VideoNode, width: int | None = None, height: int | None = None, **kwargs: Any
-    ) -> dict[str, Any]:
-        if callable((fmt := self.kwargs.pop("format", None))):
-            kwargs["format"] = fmt(clip)
-        return super().get_params_args(is_descale, clip, width, height, **kwargs)
-
-
 def ccd(
     clip: vs.VideoNode,
     thr: float = 4,
@@ -261,7 +216,7 @@ def ccd(
     ref_points: Sequence[bool] = (True, True, False),
     scale: float | None = None,
     pscale: float = 0.0,
-    chroma_upscaler: ScalerLike = _LanczosChroma(
+    chroma_upscaler: ScalerLike = Lanczos(
         format=lambda clip: clip.format.replace(color_family=vs.RGB, subsampling_w=0, subsampling_h=0)
     ),
     chroma_downscaler: KernelLike = Catrom,
@@ -274,7 +229,6 @@ def ccd(
     It works best on analog or low-quality digital sources such as VHS or DVD captures.
 
     Example usage:
-
         ```py
         denoised = ccd(clip, thr=6, tr=1, chroma_upscaler=Lanczos(format=vs.RGB48))
         ```
@@ -288,8 +242,9 @@ def ccd(
             respectively, in the processing matrix. The default uses the low and medium, but excludes the high points.
             See [zsmooth.CCD](https://github.com/adworacz/zsmooth?tab=readme-ov-file#ccd) for details.
         scale: Matrix size multiplier.
-            `1` = 25x25 matrix (original CCD)
-            `2` = 50x50, and so on.
+
+               - `1` = 25x25 matrix (original CCD)
+               - `2` = 50x50, and so on.
         pscale: Correction strength for chroma ringing introduced during downscaling.
         chroma_upscaler: Upscaler for converting YUV input to full-resolution RGB before processing.
         chroma_downscaler: Kernel used to downscale the denoised RGB result back to the input format.
@@ -304,8 +259,6 @@ def ccd(
     """
     func = func or ccd
 
-    assert check_variable_format(clip, func)
-
     if planes is MISSING:
         planes = [1, 2] if clip.format.color_family == vs.YUV else None
 
@@ -317,11 +270,11 @@ def ccd(
     else:
         full = Scaler.ensure_obj(chroma_upscaler, func).scale(clip, clip.width, clip.height)
 
-        if (full.format.subsampling_w, full.format.subsampling_h) != (0, 0):  # type: ignore[union-attr]
+        if (full.format.subsampling_w, full.format.subsampling_h) != (0, 0):
             raise CustomRuntimeError("`chroma_upscaler` didn't upscale chroma planes.", func, repr(full))
 
     if get_color_family(full) != vs.RGB:
-        rgb = vs.core.resize.Point(full, format=full.format.replace(color_family=vs.RGB).id)  # type: ignore[union-attr]
+        rgb = vs.core.resize.Point(full, format=full.format.replace(color_family=vs.RGB).id)
     else:
         rgb = full
 
