@@ -4,27 +4,30 @@ This module implements prefilters for denoisers.
 
 from __future__ import annotations
 
+from abc import ABC, abstractmethod
 from enum import auto
 from typing import Any, Literal, Sequence, cast, overload
 
-from jetpytools import CustomEnum, CustomNotImplementedError, FuncExcept, KwargsT
+from jetpytools import (
+    MISSING,
+    CustomEnum,
+    CustomNotImplementedError,
+    CustomValueError,
+    EnumABCMeta,
+    FuncExcept,
+    normalize_seq,
+)
 
 from vsexprtools import norm_expr
 from vsrgtools import bilateral, flux_smooth, gauss_blur, min_blur
 from vstools import (
-    MISSING,
     ColorRange,
-    ConstantFormatVideoNode,
-    CustomValueError,
-    InvalidColorFamilyError,
     MissingT,
     Planes,
-    check_variable,
-    check_variable_format,
+    UnsupportedColorFamilyError,
     get_peak_value,
     get_y,
     normalize_planes,
-    normalize_seq,
     scale_value,
     vs,
 )
@@ -44,9 +47,6 @@ def _run_prefilter(pref_type: Prefilter, clip: vs.VideoNode, planes: Planes, **k
     """
     Internal function for applying a prefilter to a clip.
     """
-
-    assert check_variable(clip, pref_type)
-
     if pref_type == Prefilter.NONE:
         return clip
 
@@ -88,7 +88,7 @@ def _run_prefilter(pref_type: Prefilter, clip: vs.VideoNode, planes: Planes, **k
     if pref_type == Prefilter.NLMEANS:
         from .nlm import nl_means
 
-        return nl_means(clip, **KwargsT(h=7.0, s=2, planes=planes) | kwargs)
+        return nl_means(clip, **{"h": 7.0, "s": 2, "planes": planes} | kwargs)
 
     if pref_type == Prefilter.BM3D:
         from .blockmatch import bm3d
@@ -97,7 +97,7 @@ def _run_prefilter(pref_type: Prefilter, clip: vs.VideoNode, planes: Planes, **k
 
         sigmas = kwargs.pop("sigma", [10 if 0 in planes else 0, 10 if (1 in planes or 2 in planes) else 0])
 
-        return bm3d(clip, **KwargsT(sigma=sigmas, radius=1) | kwargs)
+        return bm3d(clip, **{"sigma": sigmas, "radius": 1} | kwargs)
 
     if pref_type is Prefilter.BILATERAL:
         planes = normalize_planes(clip, planes)
@@ -130,14 +130,14 @@ def _run_prefilter(pref_type: Prefilter, clip: vs.VideoNode, planes: Planes, **k
     raise CustomNotImplementedError(func=pref_type, reason=pref_type)
 
 
-class AbstractPrefilter:
+class AbstractPrefilter(ABC):
+    @abstractmethod
     def __call__(
         self, clip: vs.VideoNode, planes: Planes = None, full_range: bool | float = False, **kwargs: Any
-    ) -> vs.VideoNode | PrefilterPartial:
-        raise NotImplementedError
+    ) -> vs.VideoNode | PrefilterPartial: ...
 
 
-class Prefilter(AbstractPrefilter, CustomEnum):
+class Prefilter(AbstractPrefilter, CustomEnum, metaclass=EnumABCMeta):
     """
     Enum representing available filters.
 
@@ -579,7 +579,7 @@ PrefilterLike = Prefilter | PrefilterPartial | MultiPrefilter
 
 def prefilter_to_full_range(
     clip: vs.VideoNode, slope: float = 2.0, smooth: float = 0.0625, func: FuncExcept | None = None
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Converts a clip to full range if necessary and amplifies dark areas.
     Essentially acts like a luma-based multiplier on the SAD when used as an mvtools prefilter.
@@ -594,9 +594,7 @@ def prefilter_to_full_range(
     """
     func = func or prefilter_to_full_range
 
-    assert check_variable_format(clip, func)
-
-    InvalidColorFamilyError.check(clip, (vs.YUV, vs.GRAY), func)
+    UnsupportedColorFamilyError.check(clip, (vs.YUV, vs.GRAY), func)
 
     if smooth < 0 or not 0 <= slope <= (1 + 2 * smooth) / smooth:
         raise CustomValueError("Curve parameters out of range", func, (slope, smooth))
