@@ -10,16 +10,15 @@ from typing import (
     Iterable,
     Literal,
     NoReturn,
+    Self,
     SupportsIndex,
-    TypeAlias,
     cast,
 )
 
 from jetpytools import SupportsString, to_arr
 from jetpytools.types.utils import SingletonMeta
-from typing_extensions import Self
 
-from vstools import ConvMode, OnePassConvMode, flatten, vs, vs_object
+from vstools import ConvMode, OnePassConvMode, VSObjectABC, flatten, vs
 
 from ..exprop import ExprOp, ExprToken
 
@@ -241,8 +240,7 @@ class Operators(metaclass=SingletonMeta):
             exclude: Optional set of (x, y) coordinates to exclude from the matrix.
             include: Optional set of (x, y) coordinates to include in the matrix.
 
-        Returns:
-            A list of [LiteralVar][vsexprtools.inline.helpers.LiteralVar] instances
+        Returns: A list of [LiteralVar][vsexprtools.inline.helpers.LiteralVar] instances
             representing the matrix of expressions.
         """
         matrix, *_ = ExprOp.matrix(char, radius, mode, exclude, include)
@@ -291,7 +289,7 @@ op = Operators()
 
 
 class ExprVar(ABC):
-    """Base interface for variables used in RPN expression"""
+    """Base interface for variables used in RPN expression."""
 
     def __add__(self, other: ExprVarLike) -> ComputedVar:
         if other == 0:
@@ -466,7 +464,7 @@ class ExprVar(ABC):
 
     def as_var(self) -> ComputedVar:
         """
-        Converts the expression variable to a ComputedVar.
+        Converts the expression variable to a [ComputedVar][vsexprtools.inline.helpers.ComputedVar].
 
         Returns:
             A ComputedVar.
@@ -476,13 +474,16 @@ class ExprVar(ABC):
         return ComputedVar(self)
 
 
-ExprVarLike: TypeAlias = int | float | str | ExprVar
+type ExprVarLike = int | float | str | ExprVar
 """Type alias representing any expression-compatible variable or literal."""
 
 
 class LiteralVar(ExprVar):
     """Literal value wrapper for use in RPN expressions."""
 
+    value: ExprVarLike
+    """The underlying raw value."""
+
     def __init__(self, value: ExprVarLike):
         """
         Initializes a new LiteralVar.
@@ -700,7 +701,7 @@ class ClipVarProps:
         return ComputedVar(f"{self._var.char}.{name}")
 
 
-class ClipVar(ExprVar, vs_object):
+class ClipVar(VSObjectABC, ExprVar):
     """
     Expression variable that wraps a VideoNode and provides symbolic and numeric access.
     """
@@ -779,9 +780,6 @@ class ClipVar(ExprVar, vs_object):
                     f"'{self.__class__.__name__}' object has no attribute '{name}'", name=name, obj=self
                 )
 
-    def __vs_del__(self, core_id: int) -> None:
-        del self._node
-
 
 class Token(LiteralVar):
     """An expression token wrapping [ExprToken][vsexprtools.ExprToken]."""
