@@ -9,12 +9,11 @@ from contextlib import contextmanager
 from functools import cache
 from inspect import currentframe
 from types import FrameType
-from typing import Any, Iterable, Iterator, Sequence, SupportsIndex, cast, overload
+from typing import Any, Iterable, Iterator, Self, Sequence, SupportsIndex, cast, overload
 
 from jetpytools import CustomValueError, to_arr
-from typing_extensions import Self
 
-from vstools import HoldsVideoFormat, VideoFormatLike, get_video_format, vs, vs_object
+from vstools import HoldsVideoFormat, VideoFormatLike, VSObject, get_video_format, vs
 
 from ..error import CustomExprError
 from ..funcs import norm_expr
@@ -244,7 +243,7 @@ def inline_expr(
 
     Args:
         clips: Input clip(s).
-        format: format: Output format, defaults to the first clip format.
+        format: Output format, defaults to the first clip format.
         enable_polyfills: Enable monkey-patching built-in methods. Maybe more than that, nobody knows.
         **kwargs: Additional keyword arguments passed to [norm_expr][vsexprtools.norm_expr].
 
@@ -268,7 +267,7 @@ def inline_expr(
             You must assign the final [ComputedVar][vsexprtools.inline.helpers.ComputedVar]
             (the result of your expression) to `ie.out`.
 
-            Additionnaly, you can use `print(ie.out)` to see the computed expression string
+            Additionally, you can use `print(ie.out)` to see the computed expression string
             or `print(ie.out.to_str_per_plane())` or `print(ie.out.to_str(plane=...))` to see the expression per plane.
     """
     clips = to_arr(clips)
@@ -305,7 +304,7 @@ def inline_expr(
     raise error.with_traceback(catcher.tb)
 
 
-class InlineExprWrapper(vs_object):
+class InlineExprWrapper(VSObject):
     """
     A wrapper class for constructing and evaluating VapourSynth expressions inline using Python syntax.
 
@@ -444,9 +443,9 @@ class InlineExprWrapper(vs_object):
         raise CustomValueError("You can only get the output clip out of the context manager!", self.__class__)
 
     def __vs_del__(self, core_id: int) -> None:
-        del self._final_clip
         del self._nodes
         del self._format
+        del self._final_clip
 
 
 def _capture_locals(frame: FrameType | None, level: int = 0) -> dict[str, Any]:
