@@ -4,10 +4,7 @@ This module defines the base abstract interfaces for general scaling operations.
 
 from __future__ import annotations
 
-from abc import ABC, ABCMeta
-from contextlib import suppress
 from functools import cache, wraps
-from inspect import Signature
 from math import ceil
 from types import NoneType
 from typing import (
@@ -18,41 +15,46 @@ from typing import (
     Concatenate,
     Literal,
     NoReturn,
-    TypeVar,
-    Union,
+    Self,
+    SupportsInt,
     get_origin,
     overload,
 )
 
-from jetpytools import P, R, R_co, classproperty
-from jetpytools import cachedproperty as jetpytools_cachedproperty
-from typing_extensions import Self
-
-from vstools import (
-    ConstantFormatVideoNode,
+from jetpytools import (
     CustomNotImplementedError,
     CustomRuntimeError,
     CustomValueError,
+    FuncExcept,
+    classproperty,
+    fallback,
+    get_subclasses,
+    normalize_seq,
+)
+from jetpytools import cachedproperty as jetpytools_cachedproperty
+
+from vsjetpack import TypeIs
+from vstools import (
+    ChromaLocation,
+    ColorRange,
     FieldBased,
     FieldBasedLike,
-    FuncExcept,
     HoldsVideoFormat,
     Matrix,
     MatrixLike,
+    Primaries,
+    PrimariesLike,
+    PropEnum,
+    Transfer,
+    TransferLike,
     VideoFormatLike,
-    VideoNodeT,
-    check_correct_subsampling,
-    check_variable,
-    check_variable_format,
+    VSObjectABC,
+    VSObjectABCMeta,
     check_variable_resolution,
     core,
-    fallback,
-    get_subclasses,
     get_video_format,
-    normalize_seq,
     split,
     vs,
-    vs_object,
 )
 from vstools.enums.color import _norm_props_enums
 
@@ -66,6 +68,7 @@ from ..exceptions import (
 from ..types import LeftShift, TopShift
 
 __all__ = [
+    "BaseScaler",
     "Bobber",
     "BobberLike",
     "Descaler",
@@ -79,9 +82,9 @@ __all__ = [
 ]
 
 
-def _add_init_kwargs(method: Callable[Concatenate[_BaseScalerT, P], R]) -> Callable[Concatenate[_BaseScalerT, P], R]:
-    signature = Signature.from_callable(method)
-
+def _add_init_kwargs[_BaseScalerT: BaseScaler, **P, R](
+    method: Callable[Concatenate[_BaseScalerT, P], R],
+) -> Callable[Concatenate[_BaseScalerT, P], R]:
     @wraps(method)
     def _wrapped(self: _BaseScalerT, *args: P.args, **kwargs: P.kwargs) -> R:
         # TODO: remove this
@@ -118,7 +121,7 @@ def _add_init_kwargs(method: Callable[Concatenate[_BaseScalerT, P], R]) -> Calla
                 frame_infos.clear()
                 del frame_info, f0, f1
 
-        init_kwargs = {k: self.kwargs.pop(k) for k in self.kwargs.keys() & signature.parameters.keys()}
+        init_kwargs = {k: self.kwargs.pop(k) for k in self.kwargs.keys() & method.__annotations__.keys()}
 
         returned = method(self, *args, **init_kwargs | kwargs)
 
@@ -131,7 +134,7 @@ def _add_init_kwargs(method: Callable[Concatenate[_BaseScalerT, P], R]) -> Calla
     return _wrapped
 
 
-def _base_from_param(
+def _base_from_param[_BaseScalerT: BaseScaler](
     cls: type[_BaseScalerT],
     value: str | type[_BaseScalerT] | _BaseScalerT | None,
     exception_cls: type[_UnknownBaseScalerError],
@@ -164,7 +167,7 @@ def _base_from_param(
     raise exception_cls(func_except or cls.from_param, str(value))
 
 
-def _base_ensure_obj(
+def _base_ensure_obj[_BaseScalerT: BaseScaler](
     cls: type[_BaseScalerT], value: str | type[_BaseScalerT] | _BaseScalerT | None, func_except: FuncExcept | None
 ) -> _BaseScalerT:
     if isinstance(value, cls):
@@ -188,6 +191,36 @@ def _check_kernel_radius(cls: type[BaseScaler]) -> Literal[True]:
     )
 
 
+def _is_format_resolver(
+    value: Any,
+) -> TypeIs[Callable[[vs.VideoNode], SupportsInt | VideoFormatLike | HoldsVideoFormat]]:
+    return callable(value)
+
+
+def _resolve_video_spec_args(clip: vs.VideoNode, **kwargs: Any) -> dict[str, Any]:
+    if _is_format_resolver((fmt := kwargs.get("format"))):
+        kwargs["format"] = get_video_format(fmt(clip))
+
+    spec_map: dict[str, type[PropEnum]] = {
+        "matrix": Matrix,
+        "matrix_in": Matrix,
+        "transfer": Transfer,
+        "transfer_in": Transfer,
+        "primaries": Primaries,
+        "primaries_in": Primaries,
+        "range": ColorRange,
+        "range_in": ColorRange,
+        "chromaloc": ChromaLocation,
+        "chromaloc_in": ChromaLocation,
+    }
+
+    for name, prop_enum in spec_map.items():
+        if callable((resolver := kwargs.get(name))):
+            kwargs[name] = prop_enum.from_param_with_fallback(resolver(clip))
+
+    return kwargs
+
+
 abstract_kernels: list[BaseScalerMeta] = []
 """
 List of fully abstract kernel classes.
@@ -203,7 +236,7 @@ These may implement some but not all kernel functionality.
 """
 
 
-class BaseScalerMeta(ABCMeta):
+class BaseScalerMeta(VSObjectABCMeta):
     """
     Metaclass for scaler classes.
 
@@ -217,14 +250,16 @@ class BaseScalerMeta(ABCMeta):
       still allowed to be instantiated. It is added to ``partial_abstract_kernels``.
     """
 
-    class cachedproperty(jetpytools_cachedproperty[R_co]):  # noqa: N801
+    class cachedproperty[R](jetpytools_cachedproperty[R]):  # noqa: N801
         """
         Read only version of jetpytools.cachedproperty.
         """
 
         if TYPE_CHECKING:
 
-            def __init__(self, func: Callable[Concatenate[_BaseScalerT, P], R_co]) -> None: ...
+            def __init__[_BaseScalerT: BaseScaler, **P](
+                self, func: Callable[Concatenate[_BaseScalerT, P], R]
+            ) -> None: ...
 
         def __set__(self, instance: None, value: Any) -> NoReturn:
             """
@@ -234,8 +269,8 @@ class BaseScalerMeta(ABCMeta):
 
     cached_property = cachedproperty
 
-    def __new__(
-        mcls: type[_BaseScalerMetaT],
+    def __new__[MetaSelf: BaseScalerMeta](  # noqa: PYI019
+        mcls: type[MetaSelf],
         name: str,
         bases: tuple[type, ...],
         namespace: dict[str, Any],
@@ -244,7 +279,7 @@ class BaseScalerMeta(ABCMeta):
         abstract: bool = False,
         partial_abstract: bool = False,
         **kwargs: Any,
-    ) -> _BaseScalerMetaT:
+    ) -> MetaSelf:
         """
         Makes a new BaseScalerMeta type class.
 
@@ -256,7 +291,7 @@ class BaseScalerMeta(ABCMeta):
 
         obj = super().__new__(mcls, name, bases, namespace, **kwargs)
 
-        # Decorate the `implemented_funcs` with `_add_init_kwargs` to add the init kwargs to the funcs.
+        # Decorate the implemented_funcs with `_add_init_kwargs` to add the init kwargs to the funcs.
         for impl_func_name in getattr(obj, "_implemented_funcs"):
             func = getattr(obj, impl_func_name)
 
@@ -287,15 +322,18 @@ def _partial_abstract_kernel_radius(self: BaseScaler) -> int:
     raise CustomNotImplementedError("kernel_radius is not implemented!", self.__class__)
 
 
+setattr(_partial_abstract_kernel_radius, "__name__", "kernel_radius")
+
+
 @BaseScalerMeta.cachedproperty
 def _static_kernel_radius_property(self: BaseScaler) -> int:
     return ceil(self._static_kernel_radius)
 
 
-_BaseScalerMetaT = TypeVar("_BaseScalerMetaT", bound=BaseScalerMeta)
+setattr(_static_kernel_radius_property, "__name__", "kernel_radius")
 
 
-class BaseScaler(vs_object, ABC, metaclass=BaseScalerMeta, abstract=True):
+class BaseScaler(VSObjectABC, metaclass=BaseScalerMeta, abstract=True):
     """
     Base abstract scaling interface for VapourSynth scalers.
     """
@@ -313,7 +351,7 @@ class BaseScaler(vs_object, ABC, metaclass=BaseScalerMeta, abstract=True):
     """
     Tuple of function names that are implemented in the current class.
 
-    These functions determine which keyword arguments will be extracted from the __init__ method.
+    These functions determine which keyword arguments will be extracted from the `__init__` method.
     """
 
     if not TYPE_CHECKING:
@@ -329,10 +367,11 @@ class BaseScaler(vs_object, ABC, metaclass=BaseScalerMeta, abstract=True):
         """
         Initialize the scaler with optional keyword arguments.
 
-        These keyword arguments are automatically forwarded to the `implemented_funcs` methods
-        but only if the method explicitly accepts them as named parameters.
-        If the same keyword is passed to both `__init__` and one of the `implemented_funcs`,
-        the one passed to `func` takes precedence.
+        These keyword arguments are automatically forwarded to
+        the [implemented_funcs][vskernels.BaseScaler.implemented_funcs] methods but only
+        if the method explicitly accepts them as named parameters.
+        If the same keyword is passed to both `__init__` and one of the
+        [implemented_funcs][vskernels.BaseScaler.implemented_funcs], the one passed to `func` takes precedence.
 
         Args:
             **kwargs: Keyword arguments that configure the internal scaling behavior.
@@ -450,20 +489,13 @@ class BaseScaler(vs_object, ABC, metaclass=BaseScalerMeta, abstract=True):
         """
         Returns a set of function names that are implemented in the current class and the parent classes.
 
-        These functions determine which keyword arguments will be extracted from the __init__ method.
+        These functions determine which keyword arguments will be extracted from the `__init__` method.
 
         Returns:
             Frozen set of function names.
         """
         return frozenset(func for klass in cls.mro() for func in getattr(klass, "_implemented_funcs", ()))
 
-    def __vs_del__(self, core_id: int) -> None:
-        with suppress(AttributeError):
-            self.kwargs.clear()
-
-
-_BaseScalerT = TypeVar("_BaseScalerT", bound=BaseScaler)
-
 
 class Scaler(BaseScaler):
     """
@@ -486,7 +518,7 @@ class Scaler(BaseScaler):
         height: int | None = None,
         shift: tuple[TopShift, LeftShift] = (0, 0),
         **kwargs: Any,
-    ) -> vs.VideoNode | ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Scale a clip to a specified resolution.
 
@@ -505,13 +537,12 @@ class Scaler(BaseScaler):
             The scaled clip.
         """
         width, height = self._wh_norm(clip, width, height)
-        check_correct_subsampling(clip, width, height)
 
         return self.scale_function(clip, **_norm_props_enums(self.get_scale_args(clip, shift, width, height, **kwargs)))
 
     def supersample(
-        self, clip: VideoNodeT, rfactor: float = 2.0, shift: tuple[TopShift, LeftShift] = (0, 0), **kwargs: Any
-    ) -> VideoNodeT:
+        self, clip: vs.VideoNode, rfactor: float = 2.0, shift: tuple[TopShift, LeftShift] = (0, 0), **kwargs: Any
+    ) -> vs.VideoNode:
         """
         Supersample a clip by a given scaling factor.
 
@@ -542,7 +573,7 @@ class Scaler(BaseScaler):
                 rfactor,
             )
 
-        return self.scale(clip, dst_width, dst_height, shift, **kwargs)  # type: ignore[return-value]
+        return self.scale(clip, dst_width, dst_height, shift, **kwargs)
 
     def get_scale_args(
         self,
@@ -577,10 +608,10 @@ class Descaler(BaseScaler):
 
     _err_class: ClassVar[type[_UnknownBaseScalerError]] = UnknownDescalerError
 
-    descale_function: Callable[..., ConstantFormatVideoNode]
+    descale_function: Callable[..., vs.VideoNode]
     """Descale function called internally when performing descaling operations."""
 
-    rescale_function: Callable[..., ConstantFormatVideoNode]
+    rescale_function: Callable[..., vs.VideoNode]
     """Rescale function called internally when performing upscaling operations."""
 
     _implemented_funcs: ClassVar[tuple[str, ...]] = ("descale", "rescale")
@@ -592,7 +623,7 @@ class Descaler(BaseScaler):
         height: int | None,
         shift: tuple[TopShift, LeftShift] = (0, 0),
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Descale a clip to the given resolution.
 
@@ -610,7 +641,6 @@ class Descaler(BaseScaler):
             The descaled clip.
         """
         width, height = self._wh_norm(clip, width, height)
-        check_correct_subsampling(clip, width, height)
 
         return self.descale_function(
             clip, **_norm_props_enums(self.get_descale_args(clip, shift, width, height, **kwargs))
@@ -623,7 +653,7 @@ class Descaler(BaseScaler):
         height: int | None,
         shift: tuple[TopShift, LeftShift] = (0, 0),
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Rescale a clip to the given resolution from a previously descaled clip.
 
@@ -641,7 +671,6 @@ class Descaler(BaseScaler):
             The scaled clip.
         """
         width, height = self._wh_norm(clip, width, height)
-        check_correct_subsampling(clip, width, height)
 
         return self.rescale_function(
             clip, **_norm_props_enums(self.get_rescale_args(clip, shift, width, height, **kwargs))
@@ -703,7 +732,7 @@ class Resampler(BaseScaler):
 
     _err_class: ClassVar[type[_UnknownBaseScalerError]] = UnknownResamplerError
 
-    resample_function: Callable[..., ConstantFormatVideoNode]
+    resample_function: Callable[..., vs.VideoNode]
     """Resample function called internally when performing resampling operations."""
 
     _implemented_funcs: ClassVar[tuple[str, ...]] = ("resample",)
@@ -711,11 +740,15 @@ class Resampler(BaseScaler):
     def resample(
         self,
         clip: vs.VideoNode,
-        format: int | VideoFormatLike | HoldsVideoFormat,
+        format: SupportsInt | VideoFormatLike | HoldsVideoFormat,
         matrix: MatrixLike | None = None,
         matrix_in: MatrixLike | None = None,
+        transfer: TransferLike | None = None,
+        transfer_in: TransferLike | None = None,
+        primaries: PrimariesLike | None = None,
+        primaries_in: PrimariesLike | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Resample a video clip to the given format.
 
@@ -732,21 +765,34 @@ class Resampler(BaseScaler):
                    - or a source from which a valid `VideoFormat` can be extracted.
             matrix: An optional color transformation matrix to apply.
             matrix_in: An optional input matrix for color transformations.
+            transfer: An optional color transformation transfer to apply.
+            transfer_in: An optional input transfer for color transformations.
+            primaries: Optional color transformation primaries to apply.
+            primaries_in: Optional input primaries for color transformations.
             **kwargs: Additional keyword arguments passed to the `resample_function`.
 
         Returns:
             The resampled clip.
         """
         return self.resample_function(
-            clip, **_norm_props_enums(self.get_resample_args(clip, format, matrix, matrix_in, **kwargs))
+            clip,
+            **_norm_props_enums(
+                self.get_resample_args(
+                    clip, format, matrix, matrix_in, transfer, transfer_in, primaries, primaries_in, **kwargs
+                )
+            ),
         )
 
     def get_resample_args(
         self,
         clip: vs.VideoNode,
-        format: int | VideoFormatLike | HoldsVideoFormat,
+        format: SupportsInt | VideoFormatLike | HoldsVideoFormat,
         matrix: MatrixLike | None,
         matrix_in: MatrixLike | None,
+        transfer: TransferLike | None = None,
+        transfer_in: TransferLike | None = None,
+        primaries: PrimariesLike | None = None,
+        primaries_in: PrimariesLike | None = None,
         **kwargs: Any,
     ) -> dict[str, Any]:
         """
@@ -761,19 +807,30 @@ class Resampler(BaseScaler):
                    - or a source from which a valid `VideoFormat` can be extracted.
             matrix: The matrix for color transformation.
             matrix_in: The input matrix for color transformation.
+            transfer: The transfer for color transformation.
+            transfer_in: The input transfer for color transformation.
+            primaries: The primaries for color transformation.
+            primaries_in: The input primaries for color transformation.
             **kwargs: Additional keyword arguments for resampling.
 
         Returns:
             A dictionary containing the resampling arguments.
         """
-        return (
-            {
-                "format": get_video_format(format).id,
-                "matrix": Matrix.from_param(matrix),
-                "matrix_in": Matrix.from_param(matrix_in),
-            }
-            | self.kwargs
-            | kwargs
+        return _resolve_video_spec_args(
+            clip,
+            **(
+                {
+                    "format": get_video_format(format).id,
+                    "matrix": Matrix.from_param_with_fallback(matrix),
+                    "matrix_in": Matrix.from_param_with_fallback(matrix_in),
+                    "transfer": Transfer.from_param_with_fallback(transfer),
+                    "transfer_in": Transfer.from_param_with_fallback(transfer_in),
+                    "primaries": Primaries.from_param_with_fallback(primaries),
+                    "primaries_in": Primaries.from_param_with_fallback(primaries_in),
+                }
+                | self.kwargs
+                | kwargs
+            ),
         )
 
 
@@ -792,7 +849,7 @@ class Kernel(Scaler, Descaler, Resampler):
     _implemented_funcs: ClassVar[tuple[str, ...]] = ("shift",)
 
     @overload
-    def shift(self, clip: vs.VideoNode, shift: tuple[TopShift, LeftShift], /, **kwargs: Any) -> ConstantFormatVideoNode:
+    def shift(self, clip: vs.VideoNode, shift: tuple[TopShift, LeftShift], /, **kwargs: Any) -> vs.VideoNode:
         """
         Apply a subpixel shift to the clip using the kernel's scaling logic.
 
@@ -815,7 +872,7 @@ class Kernel(Scaler, Descaler, Resampler):
     @overload
     def shift(
         self, clip: vs.VideoNode, shift_top: float | list[float], shift_left: float | list[float], /, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Apply a subpixel shift to the clip using the kernel's scaling logic.
 
@@ -843,7 +900,7 @@ class Kernel(Scaler, Descaler, Resampler):
         shift_left: float | list[float] | None = None,
         /,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Apply a subpixel shift to the clip using the kernel's scaling logic.
 
@@ -867,12 +924,9 @@ class Kernel(Scaler, Descaler, Resampler):
             VariableFormatError: If the input clip has variable format.
             CustomValueError: If the input clip is GRAY but lists of shift has been passed.
         """
-        assert check_variable_format(clip, self.shift)
 
-        n_planes = clip.format.num_planes
-
-        def _shift(src: vs.VideoNode, shift: tuple[TopShift, LeftShift] = (0, 0)) -> ConstantFormatVideoNode:
-            return self.scale(src, shift=shift, **kwargs)  # type: ignore[return-value]
+        def _shift(src: vs.VideoNode, shift: tuple[TopShift, LeftShift] = (0, 0)) -> vs.VideoNode:
+            return self.scale(src, shift=shift, **kwargs)
 
         if isinstance(shifts_or_top, tuple):
             return _shift(clip, shifts_or_top)
@@ -883,10 +937,10 @@ class Kernel(Scaler, Descaler, Resampler):
         if shift_left is None:
             shift_left = 0.0
 
-        shifts_top = normalize_seq(shifts_or_top, n_planes)
-        shifts_left = normalize_seq(shift_left, n_planes)
+        shifts_top = normalize_seq(shifts_or_top, clip.format.num_planes)
+        shifts_left = normalize_seq(shift_left, clip.format.num_planes)
 
-        if n_planes == 1:
+        if clip.format.num_planes == 1:
             return _shift(clip, (shifts_top[0], shifts_left[0]))
 
         shifted_planes = [
@@ -912,7 +966,7 @@ class Kernel(Scaler, Descaler, Resampler):
         Returns:
             Dictionary of combined parameters.
         """
-        return {"width": width, "height": height} | self.kwargs | kwargs
+        return _resolve_video_spec_args(clip, **{"width": width, "height": height} | self.kwargs | kwargs)
 
     def get_scale_args(
         self,
@@ -986,9 +1040,13 @@ class Kernel(Scaler, Descaler, Resampler):
     def get_resample_args(
         self,
         clip: vs.VideoNode,
-        format: int | VideoFormatLike | HoldsVideoFormat,
+        format: SupportsInt | VideoFormatLike | HoldsVideoFormat,
         matrix: MatrixLike | None,
         matrix_in: MatrixLike | None,
+        transfer: TransferLike | None = None,
+        transfer_in: TransferLike | None = None,
+        primaries: PrimariesLike | None = None,
+        primaries_in: PrimariesLike | None = None,
         **kwargs: Any,
     ) -> dict[str, Any]:
         """
@@ -1003,6 +1061,10 @@ class Kernel(Scaler, Descaler, Resampler):
                    - or a source from which a valid `VideoFormat` can be extracted.
             matrix: Target color matrix.
             matrix_in: Source color matrix.
+            transfer: Target color transfer.
+            transfer_in: Source color transfer.
+            primaries: Target color primaries.
+            primaries_in: Source color primaries.
             **kwargs: Additional arguments to pass to the resample function.
 
         Returns:
@@ -1010,8 +1072,12 @@ class Kernel(Scaler, Descaler, Resampler):
         """
         return {
             "format": get_video_format(format).id,
-            "matrix": Matrix.from_param(matrix),
-            "matrix_in": Matrix.from_param(matrix_in),
+            "matrix": Matrix.from_param_with_fallback(matrix),
+            "matrix_in": Matrix.from_param_with_fallback(matrix_in),
+            "transfer": Transfer.from_param_with_fallback(transfer),
+            "transfer_in": Transfer.from_param_with_fallback(transfer_in),
+            "primaries": Primaries.from_param_with_fallback(primaries),
+            "primaries_in": Primaries.from_param_with_fallback(primaries_in),
         } | self.get_params_args(False, clip, **kwargs)
 
 
@@ -1020,14 +1086,12 @@ class Bobber(BaseScaler):
     Abstract scaler class that applies bob deinterlacing.
     """
 
-    bob_function: Callable[..., ConstantFormatVideoNode]
+    bob_function: Callable[..., vs.VideoNode]
     """Bob function called internally when performing bobbing operations."""
 
     _implemented_funcs: ClassVar[tuple[str, ...]] = ("bob", "deinterlace")
 
-    def bob(
-        self, clip: vs.VideoNode, *, tff: FieldBasedLike | bool | None = None, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
+    def bob(self, clip: vs.VideoNode, *, tff: FieldBasedLike | bool | None = None, **kwargs: Any) -> vs.VideoNode:
         """
         Apply bob deinterlacing to a given clip using the selected resizer.
 
@@ -1044,13 +1108,11 @@ class Bobber(BaseScaler):
         """
         clip_fieldbased = FieldBased.from_param_or_video(tff, clip, True, self.__class__)
 
-        assert check_variable(clip, self.__class__)
-
         return self.bob_function(clip, **self.get_bob_args(clip, tff=clip_fieldbased.is_tff, **kwargs))
 
     def deinterlace(
         self, clip: vs.VideoNode, *, tff: FieldBasedLike | bool | None = None, double_rate: bool = True, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Apply deinterlacing to a given clip using the selected resizer.
 
@@ -1090,52 +1152,57 @@ class Bobber(BaseScaler):
         return self.kwargs | kwargs
 
 
-ScalerLike = Union[str, type[Scaler], Scaler]
+type ScalerLike = str | type[Scaler] | Scaler
 """
 Type alias for anything that can resolve to a Scaler.
 
 This includes:
- - A string identifier.
- - A class type subclassing `Scaler`.
- - An instance of a `Scaler`.
+
+- A string identifier.
+- A class type subclassing [Scaler][vskernels.Scaler].
+- An instance of a [Scaler][vskernels.Scaler].
 """
 
-DescalerLike = Union[str, type[Descaler], Descaler]
+type DescalerLike = str | type[Descaler] | Descaler
 """
 Type alias for anything that can resolve to a Descaler.
 
 This includes:
- - A string identifier.
- - A class type subclassing `Descaler`.
- - An instance of a `Descaler`.
+
+- A string identifier.
+- A class type subclassing [Descaler][vskernels.Descaler].
+- An instance of a [Descaler][vskernels.Descaler].
 """
 
-ResamplerLike = Union[str, type[Resampler], Resampler]
+type ResamplerLike = str | type[Resampler] | Resampler
 """
 Type alias for anything that can resolve to a Resampler.
 
 This includes:
- - A string identifier.
- - A class type subclassing `Resampler`.
- - An instance of a `Resampler`.
+
+- A string identifier.
+- A class type subclassing [Resampler][vskernels.Resampler].
+- An instance of a [Resampler][vskernels.Resampler].
 """
 
-KernelLike = Union[str, type[Kernel], Kernel]
+type KernelLike = str | type[Kernel] | Kernel
 """
 Type alias for anything that can resolve to a Kernel.
 
 This includes:
- - A string identifier.
- - A class type subclassing `Kernel`.
- - An instance of a `Kernel`.
+
+- A string identifier.
+- A class type subclassing [Kernel][vskernels.Kernel].
+- An instance of a [Kernel][vskernels.Kernel].
 """
 
-BobberLike = Union[str, type[Bobber], Bobber]
+type BobberLike = str | type[Bobber] | Bobber
 """
 Type alias for anything that can resolve to a Bobber.
 
 This includes:
- - A string identifier.
- - A class type subclassing `Bobber`.
- - An instance of a `Bobber`.
+
+- A string identifier.
+- A class type subclassing [Bobber][vskernels.Bobber].
+- An instance of a [Bobber][vskernels.Bobber].
 """
