@@ -4,28 +4,29 @@ This module defines the abstract classes for scaling, descaling and resampling o
 
 from __future__ import annotations
 
-import sys
+from fractions import Fraction
 from functools import partial
-from typing import TYPE_CHECKING, Any, Literal, Union, overload
+from typing import TYPE_CHECKING, Any, Literal, overload
 
-from jetpytools import CustomIndexError, CustomNotImplementedError, CustomValueError, FuncExcept, fallback
+from jetpytools import (
+    CustomIndexError,
+    CustomNotImplementedError,
+    CustomValueError,
+    FuncExcept,
+    fallback,
+    normalize_seq,
+)
 
 from vstools import (
     ChromaLocation,
-    ConstantFormatVideoNode,
     Dar,
     FieldBased,
     FieldBasedLike,
-    KwargsT,
     Resolution,
     Sar,
-    VideoNodeT,
-    check_correct_subsampling,
-    check_variable_format,
     depth,
     expect_bits,
     get_video_format,
-    normalize_seq,
     split,
     vs,
 )
@@ -35,9 +36,9 @@ from ..types import (
     BotFieldLeftShift,
     BotFieldTopShift,
     Center,
+    FieldShift,
     LeftShift,
     SampleGridModel,
-    ShiftT,
     Slope,
     TopFieldLeftShift,
     TopFieldTopShift,
@@ -91,27 +92,26 @@ def _check_dynamic_keeparscaler_params(
         )
 
     if exceptions:
-        if sys.version_info >= (3, 11):
-            raise ExceptionGroup("Multiple exceptions occurred!", exceptions)  # noqa: F821
-
-        raise Exception(exceptions)
+        raise ExceptionGroup("Multiple exceptions occurred!", exceptions)
 
     return True
 
 
 @overload
 def _descale_shift_norm(
-    shift: ShiftT, assume_progressive: Literal[True], func: FuncExcept | None = None
+    shift: tuple[TopShift, LeftShift] | FieldShift, assume_progressive: Literal[True], func: FuncExcept | None = None
 ) -> tuple[TopShift, LeftShift]: ...
 
 
 @overload
 def _descale_shift_norm(
-    shift: ShiftT, assume_progressive: Literal[False], func: FuncExcept | None = None
+    shift: tuple[TopShift, LeftShift] | FieldShift, assume_progressive: Literal[False], func: FuncExcept | None = None
 ) -> tuple[tuple[TopFieldTopShift, BotFieldTopShift], tuple[TopFieldLeftShift, BotFieldLeftShift]]: ...
 
 
-def _descale_shift_norm(shift: ShiftT, assume_progressive: bool = True, func: FuncExcept | None = None) -> Any:
+def _descale_shift_norm(
+    shift: tuple[TopShift, LeftShift] | FieldShift, assume_progressive: bool = True, func: FuncExcept | None = None
+) -> Any:
     if assume_progressive:
         if any(isinstance(sh, tuple) for sh in shift):
             raise CustomValueError("You can't descale per-field when the input is progressive!", func, shift)
@@ -127,10 +127,10 @@ def _linearize(
     clip: vs.VideoNode,
     linear: bool | None,
     sigmoid: bool | tuple[Slope, Center],
-    op_partial: partial[VideoNodeT],
+    op_partial: partial[vs.VideoNode],
     func: FuncExcept,
     **kwargs: Any,
-) -> VideoNodeT:
+) -> vs.VideoNode:
     if linear is False and sigmoid is not False:
         raise CustomValueError("If sigmoid is not False, linear can't be False as well!", func, (linear, sigmoid))
 
@@ -155,7 +155,7 @@ def _linearize(
         with LinearLight(**llargs) as ll:
             ll.linear = op_partial(ll.linear, **kwargs)
 
-        return ll.out  # type: ignore[return-value]
+        return ll.out
 
     return op_partial(clip, **kwargs)
 
@@ -199,13 +199,14 @@ class LinearScaler(Scaler):
         linear: bool | None = None,
         sigmoid: bool | tuple[Slope, Center] = False,
         **kwargs: Any,
-    ) -> vs.VideoNode | ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Scale a clip to the given resolution with optional linearization.
 
-        This method behaves like the base `Scaler.descale()` but adds support for
-        linear or sigmoid-based preprocessing and postprocessing. When enabled, the clip
-        is linearized before the scaling operation and de-linearized afterward.
+        This method behaves like the base [Scaler.scale()][vskernels.Scaler.scale] but adds support for linear
+        or sigmoid-based preprocessing and postprocessing.
+
+        When enabled, the clip is linearized before the scaling operation and de-linearized afterward.
 
         Keyword arguments passed during initialization are automatically injected here,
         unless explicitly overridden by the arguments provided at call time.
@@ -238,7 +239,7 @@ class LinearScaler(Scaler):
 
 class LinearDescaler(Descaler):
     """
-    Abctract descaler class that applies linearization before descaling.
+    Abstract descaler class that applies linearization before descaling.
 
     Only affects descaling results when `linear` or `sigmoid` parameters are specified.
 
@@ -255,7 +256,7 @@ class LinearDescaler(Descaler):
             height: int | None,
             shift: tuple[TopShift, LeftShift],
             **kwargs: Any,
-        ) -> ConstantFormatVideoNode:
+        ) -> vs.VideoNode:
             """
             An optional function to be implemented by subclasses.
 
@@ -271,7 +272,7 @@ class LinearDescaler(Descaler):
             height: int | None,
             shift: tuple[TopShift, LeftShift],
             **kwargs: Any,
-        ) -> ConstantFormatVideoNode:
+        ) -> vs.VideoNode:
             """
             An optional function to be implemented by subclasses.
 
@@ -291,13 +292,14 @@ class LinearDescaler(Descaler):
         linear: bool | None = None,
         sigmoid: bool | tuple[Slope, Center] = False,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Descale a clip to the specified resolution, optionally using linear light processing.
 
-        This method behaves like the base `Descaler.descale()` but adds support for
-        linear or sigmoid-based preprocessing and postprocessing. When enabled, the clip
-        is linearized before the descaling operation and de-linearized afterward.
+        This method behaves like the base [Descaler.descale()][vskernels.Descaler.descale] but adds support for linear
+        or sigmoid-based preprocessing and postprocessing.
+
+        When enabled, the clip is linearized before the descaling operation and de-linearized afterward.
 
         Keyword arguments passed during initialization are automatically injected here,
         unless explicitly overridden by the arguments provided at call time.
@@ -337,14 +339,15 @@ class LinearDescaler(Descaler):
         linear: bool | None = None,
         sigmoid: bool | tuple[Slope, Center] = False,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Rescale a clip to the given resolution from a previously descaled clip,
         optionally using linear light processing.
 
-        This method behaves like the base `Descaler.rescale()` but adds support for
-        linear or sigmoid-based preprocessing and postprocessing. When enabled, the clip
-        is linearized before the rescaling operation and de-linearized afterward.
+        This method behaves like the base [Descaler.rescale()][vskernels.Descaler.rescale] but adds support for linear
+        or sigmoid-based preprocessing and postprocessing.
+
+        When enabled, the clip is linearized before the rescaling operation and de-linearized afterward.
 
         Keyword arguments passed during initialization are automatically injected here,
         unless explicitly overridden by the arguments provided at call time.
@@ -375,6 +378,16 @@ class LinearDescaler(Descaler):
 
 
 class KeepArScaler(Scaler):
+    """
+    Abstract scaler class that preserves or adjusts aspect ratio during scaling.
+
+    Extends the base [Scaler][vskernels.Scaler] with logic for handling sample aspect ratio (SAR)
+    and display aspect ratio (DAR) transformations.
+
+    Provides automatic normalization and correction of scaling parameters to maintain proper image proportions
+    when `keep_ar` is enabled.
+    """
+
     def _ar_params_norm(
         self,
         clip: vs.VideoNode,
@@ -390,15 +403,23 @@ class KeepArScaler(Scaler):
                 'If "keep_ar" is not None, then at least one of "sar", "dar", or "dar_in" must be None.'
             )
 
-        # Basically what it does:
-        # - If `xar` is Xar or float -> Converted to Xar
-        # - If `xar` is None         -> It fallbacks to bool(keep_ar). Becomes True or False
-        # - If `xar` is True         -> Value after the `or`
-        # - If `xar` is False        -> Fallback value: Sar(1, 1), Dar(0) or out_dar
-        src_sar = Sar.from_param(sar if sar is not None else bool(keep_ar), Sar(1, 1)) or Sar.from_clip(clip)
+        def _norm_xar[XarT: (Dar, Sar)](
+            xar: Fraction | float | bool | None, if_true: XarT, if_false: XarT, xar_t: type[XarT]
+        ) -> XarT:
+            if xar is None:
+                xar = bool(keep_ar)
 
-        out_dar = Dar.from_param(dar if dar is not None else bool(keep_ar), Dar(0)) or Dar.from_res(width, height)
-        src_dar = Dar.from_param(dar_in if dar_in is not None else bool(keep_ar), out_dar) or Dar.from_clip(clip, False)
+            if xar is True:
+                return if_true
+
+            if xar is False:
+                return if_false
+
+            return xar_t(xar)
+
+        src_sar = _norm_xar(sar, Sar.from_clip(clip), Sar(1, 1), Sar)
+        out_dar = _norm_xar(dar, Dar.from_res(width, height), Dar(0), Dar)
+        src_dar = _norm_xar(dar_in, Dar.from_clip(clip, False), out_dar, Dar)
 
         return float(src_sar), float(src_dar), float(out_dar)
 
@@ -413,7 +434,7 @@ class KeepArScaler(Scaler):
         dar_in: Dar | bool | float | None,
         keep_ar: bool | None,
         **kwargs: Any,
-    ) -> tuple[KwargsT, tuple[TopShift, LeftShift], Sar | Literal[False]]:
+    ) -> tuple[dict[str, Any], tuple[TopShift, LeftShift], Sar | Literal[False]]:
         kwargs.setdefault("src_top", kwargs.pop("sy", shift[0]))
         kwargs.setdefault("src_left", kwargs.pop("sx", shift[1]))
         kwargs.setdefault("src_width", kwargs.pop("sw", clip.width))
@@ -463,7 +484,7 @@ class KeepArScaler(Scaler):
         dar_in: Dar | bool | float | None = None,
         keep_ar: bool | None = None,
         **kwargs: Any,
-    ) -> vs.VideoNode | ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Scale a clip to the given resolution with aspect ratio support.
 
@@ -488,8 +509,6 @@ class KeepArScaler(Scaler):
         """
         width, height = self._wh_norm(clip, width, height)
 
-        check_correct_subsampling(clip, width, height)
-
         if 0 in (clip.width, clip.height):
             _check_dynamic_keeparscaler_params(
                 border_handling, sample_grid_model, sar, dar, dar_in, keep_ar, self.scale
@@ -519,8 +538,9 @@ class ComplexScaler(KeepArScaler, LinearScaler):
     Abstract composite scaler class with support for aspect ratio preservation, linear light processing,
     and per-plane subpixel shifting.
 
-    Combines `KeepArScaler` for handling sample/display aspect ratios
-    and `LinearScaler` for linear and sigmoid processing.
+    Combines [KeepArScaler][vskernels.KeepArScaler] for handling sample/display aspect ratios
+    and [LinearScaler][vskernels.LinearScaler] for linear and sigmoid processing.
+
     Additionally, it introduces support for specifying per-plane subpixel shifts.
     """
 
@@ -545,7 +565,7 @@ class ComplexScaler(KeepArScaler, LinearScaler):
         # ComplexScaler adds blur
         blur: float | None = None,
         **kwargs: Any,
-    ) -> vs.VideoNode | ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Scale a clip to the given resolution, with aspect ratio and linear light support.
 
@@ -591,8 +611,6 @@ class ComplexScaler(KeepArScaler, LinearScaler):
         if isinstance(shift_top, (int, float)) and isinstance(shift_left, (int, float)):
             return super().scale(clip, width, height, (shift_top, shift_left), **kwargs)
 
-        assert check_variable_format(clip, self.scale)
-
         n_planes = clip.format.num_planes
 
         shift_top = normalize_seq(shift_top, n_planes)
@@ -636,7 +654,7 @@ class ComplexScaler(KeepArScaler, LinearScaler):
 
         for i, (plane, top, left) in enumerate(zip(split(clip), shift_top, shift_left)):
             if i:
-                w = round(width * 1 / 2**format_out.subsampling_h)
+                w = round(width * 1 / 2**format_out.subsampling_w)
                 h = round(height * 1 / 2**format_out.subsampling_h)
             else:
                 w, h = width, height
@@ -664,7 +682,7 @@ class ComplexDescaler(LinearDescaler):
     """
     Abstract descaler class with support for border handling and sampling grid alignment.
 
-    Extends `LinearDescaler` by introducing mechanisms to control how image borders
+    Extends [LinearDescaler][vskernels.LinearDescaler] by introducing mechanisms to control how image borders
     are handled and how the sampling grid is aligned during descaling.
     """
 
@@ -673,7 +691,7 @@ class ComplexDescaler(LinearDescaler):
         clip: vs.VideoNode,
         width: int | None = None,
         height: int | None = None,
-        shift: ShiftT = (0, 0),
+        shift: tuple[TopShift, LeftShift] | FieldShift = (0, 0),
         *,
         # `linear` and `sigmoid` parameters from LinearDescaler
         linear: bool | None = None,
@@ -685,7 +703,7 @@ class ComplexDescaler(LinearDescaler):
         ignore_mask: vs.VideoNode | None = None,
         blur: float | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Descale a clip to the given resolution, with image borders handling and sampling grid alignment,
         optionally using linear light processing.
@@ -717,7 +735,6 @@ class ComplexDescaler(LinearDescaler):
             The descaled video node, optionally processed in linear light.
         """
         width, height = self._wh_norm(clip, width, height)
-        check_correct_subsampling(clip, width, height)
 
         field_based = FieldBased.from_param_or_video(field_based, clip)
 
@@ -776,7 +793,7 @@ class ComplexDescaler(LinearDescaler):
         clip: vs.VideoNode,
         width: int | None = None,
         height: int | None = None,
-        shift: ShiftT = (0, 0),
+        shift: tuple[TopShift, LeftShift] | FieldShift = (0, 0),
         *,
         # `linear` and `sigmoid` parameters from LinearDescaler
         linear: bool | None = None,
@@ -788,7 +805,7 @@ class ComplexDescaler(LinearDescaler):
         ignore_mask: vs.VideoNode | None = None,
         blur: float | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Rescale a clip to the given resolution from a previously descaled clip,
         with image borders handling and sampling grid alignment, optionally using linear light processing.
@@ -817,7 +834,6 @@ class ComplexDescaler(LinearDescaler):
             The scaled clip.
         """
         width, height = self._wh_norm(clip, width, height)
-        check_correct_subsampling(clip, width, height)
 
         field_based = FieldBased.from_param_or_video(field_based, clip)
 
@@ -846,39 +862,42 @@ class ComplexDescaler(LinearDescaler):
 
 class ComplexKernel(Kernel, ComplexDescaler, ComplexScaler):
     """
-    Comprehensive abstract kernel class combining scaling, descaling,
-    and resampling with linear light and aspect ratio support.
+    Comprehensive abstract kernel class combining scaling, descaling, and resampling with linear light and aspect ratio support.
 
-    This class merges the full capabilities of `Kernel`, `ComplexDescaler`, and `ComplexScaler`.
-    """
+    This class merges the full capabilities of [Kernel][vskernels.Kernel], [ComplexDescaler][vskernels.ComplexDescaler],
+    and [ComplexScaler][vskernels.ComplexScaler].
+    """  # noqa: E501
 
 
-ComplexScalerLike = Union[str, type[ComplexScaler], ComplexScaler]
+type ComplexScalerLike = str | type[ComplexScaler] | ComplexScaler
 """
 Type alias for anything that can resolve to a ComplexScaler.
 
 This includes:
+
 - A string identifier.
-- A class type subclassing `ComplexScaler`.
-- An instance of a `ComplexScaler`.
+- A class type subclassing [ComplexScaler][vskernels.ComplexScaler].
+- An instance of a [ComplexScaler][vskernels.ComplexScaler].
 """
 
-ComplexDescalerLike = Union[str, type[ComplexDescaler], ComplexDescaler]
+type ComplexDescalerLike = str | type[ComplexDescaler] | ComplexDescaler
 """
 Type alias for anything that can resolve to a ComplexDescaler.
 
 This includes:
+
 - A string identifier.
-- A class type subclassing `ComplexDescaler`.
-- An instance of a `ComplexDescaler`.
+- A class type subclassing [ComplexDescaler][vskernels.ComplexDescaler].
+- An instance of a [ComplexDescaler][vskernels.ComplexDescaler].
 """
 
-ComplexKernelLike = Union[str, type[ComplexKernel], ComplexKernel]
+type ComplexKernelLike = str | type[ComplexKernel] | ComplexKernel
 """
 Type alias for anything that can resolve to a ComplexKernel.
 
 This includes:
+
 - A string identifier.
-- A class type subclassing `ComplexKernel`.
-- An instance of a `ComplexKernel`.
+- A class type subclassing [ComplexKernel][vskernels.ComplexKernel].
+- An instance of a [ComplexKernel][vskernels.ComplexKernel].
 """
