@@ -7,11 +7,11 @@ from __future__ import annotations
 
 from abc import abstractmethod
 from math import ceil
-from typing import Any, Union
+from typing import Any
 
 from jetpytools import CustomError, CustomValueError
 
-from vstools import ConstantFormatVideoNode, core, vs
+from vstools import core, vs
 
 from .base import Kernel
 from .complex import ComplexKernel
@@ -28,10 +28,11 @@ class CustomKernel(Kernel):
     """
     Abstract base class for defining custom kernel-based scaling and descaling operations.
 
-    This class allows users to implement their own kernel function by overriding the
-    `kernel()` method.
+    This class allows users to implement their own kernel function by overriding
+    the [kernel()][vskernels.CustomKernel.kernel()] method.
 
-    Subclasses must implement the `kernel()` method to specify the mathematical shape of the kernel.
+    Subclasses must implement the [kernel()][vskernels.CustomKernel.kernel()] method to specify
+    the mathematical shape of the kernel.
     """
 
     @abstractmethod
@@ -60,12 +61,10 @@ class CustomKernel(Kernel):
 
     def resample_function(
         self, clip: vs.VideoNode, width: int | None = None, height: int | None = None, *args: Any, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
-        return self.scale_function(clip, width, height, *args, **kwargs)  # type: ignore[return-value]
+    ) -> vs.VideoNode:
+        return self.scale_function(clip, width, height, *args, **kwargs)
 
-    def descale_function(
-        self, clip: vs.VideoNode, width: int, height: int, *args: Any, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
+    def descale_function(self, clip: vs.VideoNode, width: int, height: int, *args: Any, **kwargs: Any) -> vs.VideoNode:
         try:
             return core.descale.Decustom(
                 clip, width, height, self.kernel, ceil(kwargs.pop("taps", self.kernel_radius)), *args, **kwargs
@@ -80,9 +79,7 @@ class CustomKernel(Kernel):
 
             raise CustomError(e, self.__class__) from e
 
-    def rescale_function(
-        self, clip: vs.VideoNode, width: int, height: int, *args: Any, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
+    def rescale_function(self, clip: vs.VideoNode, width: int, height: int, *args: Any, **kwargs: Any) -> vs.VideoNode:
         try:
             return core.descale.ScaleCustom(
                 clip, width, height, self.kernel, ceil(kwargs.pop("taps", self.kernel_radius)), *args, **kwargs
@@ -95,35 +92,38 @@ class CustomComplexKernel(CustomKernel, ComplexKernel):
     """
     Abstract kernel class that combines custom kernel behavior with advanced scaling and descaling capabilities.
 
-    This class extends both `CustomKernel` and `ComplexKernel`, enabling the definition
-    of custom mathematical kernels with the advanced rescaling logic provided by
-    linear and aspect-ratio-aware components.
+    This class extends both [CustomKernel][vskernels.CustomKernel] and [ComplexKernel][vskernels.ComplexKernel],
+    enabling the definition of custom mathematical kernels with the advanced rescaling logic provided by linear
+    and aspect-ratio-aware components.
     """
 
 
-CustomComplexKernelLike = Union[str, type[CustomComplexKernel], CustomComplexKernel]
+type CustomComplexKernelLike = str | type[CustomComplexKernel] | CustomComplexKernel
 """
 Type alias for anything that can resolve to a CustomComplexKernel.
 
 This includes:
+
 - A string identifier.
-- A class type subclassing `CustomComplexKernel`.
-- An instance of a `CustomComplexKernel`.
+- A class type subclassing [CustomComplexKernel][vskernels.CustomComplexKernel].
+- An instance of a [CustomComplexKernel][vskernels.CustomComplexKernel].
 """
 
 
 class CustomComplexTapsKernel(CustomComplexKernel):
     """
-    Extension of `CustomComplexKernel` that introduces configurable kernel taps.
+    Extension of [CustomComplexKernel][vskernels.CustomComplexKernel] that introduces configurable kernel taps.
     """
 
     def __init__(self, taps: float, **kwargs: Any) -> None:
         """
         Initialize the kernel with a specific number of taps and optional keyword arguments.
 
-        These keyword arguments are automatically forwarded to the `implemented_funcs` methods
+        These keyword arguments are automatically forwarded to the
+        [implemented_funcs][vskernels.BaseScaler.implemented_funcs] methods
         but only if the method explicitly accepts them as named parameters.
-        If the same keyword is passed to both `__init__` and one of the `implemented_funcs`,
+        If the same keyword is passed to both `__init__` and one of the
+        [implemented_funcs][vskernels.BaseScaler.implemented_funcs],
         the one passed to `func` takes precedence.
 
         Args:
