@@ -3,7 +3,9 @@ from __future__ import annotations
 from math import sqrt
 from typing import Any, Callable, overload
 
-from vstools import ConstantFormatVideoNode, CustomValueError, core, vs
+from jetpytools import CustomValueError
+
+from vstools import core, vs
 
 from ...types import LeftShift, TopShift
 from .abstract import ZimgComplexKernel
@@ -32,9 +34,9 @@ class Bicubic(ZimgComplexKernel):
     """
 
     scale_function: Callable[..., vs.VideoNode] = core.lazy.resize2.Bicubic
-    resample_function: Callable[..., ConstantFormatVideoNode] = core.lazy.resize2.Bicubic
-    descale_function: Callable[..., ConstantFormatVideoNode] = core.lazy.descale.Debicubic
-    rescale_function: Callable[..., ConstantFormatVideoNode] = core.lazy.descale.Bicubic
+    resample_function: Callable[..., vs.VideoNode] = core.lazy.resize2.Bicubic
+    descale_function: Callable[..., vs.VideoNode] = core.lazy.descale.Debicubic
+    rescale_function: Callable[..., vs.VideoNode] = core.lazy.descale.Bicubic
 
     def __init__(self, b: float = 0, c: float = 0.5, **kwargs: Any) -> None:
         """
@@ -53,9 +55,11 @@ class Bicubic(ZimgComplexKernel):
         self, is_descale: bool, clip: vs.VideoNode, width: int | None = None, height: int | None = None, **kwargs: Any
     ) -> dict[str, Any]:
         args = super().get_params_args(is_descale, clip, width, height, **kwargs)
+
         if is_descale:
-            return args | {"b": self.b, "c": self.c}
-        return args | {"filter_param_a": self.b, "filter_param_b": self.c}
+            return {"b": self.b, "c": self.c} | args
+
+        return {"filter_param_a": self.b, "filter_param_b": self.c} | args
 
     def get_bob_args(
         self,
@@ -64,7 +68,7 @@ class Bicubic(ZimgComplexKernel):
         **kwargs: Any,
     ) -> dict[str, Any]:
         return super().get_bob_args(
-            clip, shift, filter="bicubic", filter_param_a=self.b, filter_param_b=self.c, **kwargs
+            clip, shift, filter="bicubic", **{"filter_param_a": self.b, "filter_param_b": self.c} | kwargs
         )
 
     @ZimgComplexKernel.cachedproperty
