@@ -2,40 +2,26 @@ from __future__ import annotations
 
 from contextlib import AbstractContextManager, suppress
 from dataclasses import dataclass
-from functools import cache, partial, wraps
+from functools import partial, wraps
 from math import exp
 from types import GenericAlias
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    ClassVar,
-    Concatenate,
-    Generic,
-    Union,
-    overload,
-)
+from typing import TYPE_CHECKING, Any, Callable, ClassVar, Concatenate, Self, TypeAlias, get_origin, overload
 
-from jetpytools import P, classproperty
-from typing_extensions import Self, TypeIs, TypeVar, TypeVarTuple, Unpack
+from jetpytools import CustomRuntimeError, CustomValueError, cachedproperty, classproperty
 
-from vsexprtools import norm_expr
+from vsjetpack import TypeIs, TypeVar
 from vstools import (
-    ConstantFormatVideoNode,
-    CustomRuntimeError,
-    CustomValueError,
     HoldsVideoFormat,
     Matrix,
     MatrixLike,
     Transfer,
     VideoFormatLike,
     VSFunctionNoArgs,
-    cachedproperty,
-    check_variable_format,
+    VSObject,
+    VSObjectABC,
     depth,
     get_video_format,
     vs,
-    vs_object,
 )
 
 from .abstract import (
@@ -58,7 +44,7 @@ from .abstract import (
     Scaler,
     ScalerLike,
 )
-from .abstract.base import BaseScaler, BaseScalerMeta, _BaseScalerT
+from .abstract.base import BaseScaler, BaseScalerMeta, abstract_kernels, partial_abstract_kernels
 from .kernels import Catrom, Point
 from .types import Center, LeftShift, Slope, TopShift
 
@@ -69,6 +55,7 @@ __all__ = [
     "MixedScalerProcess",
     "NoScale",
     "NoScaleLike",
+    "ScalerSpecializer",
     "is_complex_descaler_like",
     "is_complex_kernel_like",
     "is_complex_scaler_like",
@@ -82,21 +69,15 @@ __all__ = [
 ]
 
 
-@cache
-def _add_specializer_in_str(cls: BaseScalerSpecializerMeta) -> None:
-    if cls.__specializer__:
-        cls.__name__ += f"[{cls.__specializer__.__name__}]"
-
-
 class BaseScalerSpecializerMeta(BaseScalerMeta):
     """
     Meta class for BaseScalerSpecializer to handle specialization logic.
     """
 
-    __specializer__: type[BaseScaler] | None
+    __specializer__: type[BaseScaler]
 
-    def __new__(  # noqa: PYI019
-        mcls: type[_BaseScalerSpecializerMetaT],
+    def __new__[MetaSelf: BaseScalerSpecializerMeta](  # noqa: PYI019
+        mcls: type[MetaSelf],
         name: str,
         bases: tuple[type, ...],
         namespace: dict[str, Any],
@@ -104,42 +85,40 @@ class BaseScalerSpecializerMeta(BaseScalerMeta):
         *,
         specializer: type[BaseScaler] | None = None,
         **kwargs: Any,
-    ) -> _BaseScalerSpecializerMetaT:
-        if specializer is not None:
-            bases = (specializer, *bases)
-            namespace["__orig_bases__"] = (specializer, *namespace["__orig_bases__"])
+    ) -> MetaSelf:
+        if specializer:
+            bases = (*bases, specializer)
+            namespace["__orig_bases__"] = (*namespace["__orig_bases__"], specializer)
+
+            name += f"[{specializer.__name__}]"
 
             with suppress(KeyError):
                 del namespace["kernel_radius"]
 
         obj = super().__new__(mcls, name, bases, namespace, **kwargs)
-        obj.__specializer__ = specializer
+
+        if specializer:
+            obj.__specializer__ = specializer
 
         return obj
 
     @property
     def __isspecialized__(self) -> bool:
-        return bool(self.__specializer__)
+        return hasattr(self, "__specializer__")
 
 
-_BaseScalerTs = TypeVarTuple("_BaseScalerTs")
-_BaseScalerSpecializerMetaT = TypeVar("_BaseScalerSpecializerMetaT", bound=BaseScalerSpecializerMeta)
-_ScalerT = TypeVar("_ScalerT", bound=Scaler)
-_ScalerWithCatromDefaultT = TypeVar("_ScalerWithCatromDefaultT", bound=Scaler, default=Catrom)
-
-
-class BaseScalerSpecializer(BaseScaler, Generic[_BaseScalerT], metaclass=BaseScalerSpecializerMeta, abstract=True):
+class BaseScalerSpecializer[DefaultScalerT: BaseScaler](BaseScaler, metaclass=BaseScalerSpecializerMeta, abstract=True):
     """
-    An abstract base class to provide specialization logic for Scaler-like classes.
+    An abstract base class to provide specialization logic for BaseScaler-like classes.
     """
 
-    _default_scaler: ClassVar[type[BaseScaler]]
+    default_scaler: ClassVar[type[BaseScaler]]
 
     if not TYPE_CHECKING:
 
         def __new__(cls, *args: Any, **kwargs: Any) -> Self:
             if not cls.__isspecialized__:
-                return cls.__class_getitem__(cls._default_scaler)(*args, **kwargs)
+                cls = get_origin(cls[cls.default_scaler])
 
             return super().__new__(cls, *args, **kwargs)
 
@@ -152,10 +131,6 @@ class BaseScalerSpecializer(BaseScaler, Generic[_BaseScalerT], metaclass=BaseSca
 
         self.kwargs = kwargs
 
-    def __str__(self) -> str:
-        _add_specializer_in_str(self.__class__)
-        return super().__str__()
-
     def __class_getitem__(cls, base_scaler: Any) -> GenericAlias:
         """
         Specialize this class with a given scaler kernel.
@@ -170,20 +145,44 @@ class BaseScalerSpecializer(BaseScaler, Generic[_BaseScalerT], metaclass=BaseSca
             return GenericAlias(cls, (base_scaler,))
 
         specialized_scaler = BaseScalerSpecializerMeta(
-            cls.__name__, (cls,), cls.__dict__.copy(), specializer=base_scaler, partial_abstract=True
+            cls.__name__,
+            (cls,),
+            cls.__dict__.copy(),
+            specializer=base_scaler,
+            partial_abstract=base_scaler in partial_abstract_kernels,
+            abstract=base_scaler in abstract_kernels,
         )
 
         return GenericAlias(specialized_scaler, (base_scaler,))
 
+    @property
+    def specializer(self) -> DefaultScalerT:
+        """
+        Returns the effective specializer.
 
-class NoScale(BaseScalerSpecializer[_ScalerWithCatromDefaultT], Scaler, partial_abstract=True):
+        Returns:
+            The effective specializer.
+        """
+        return self.__class__.__specializer__  # type: ignore[return-value]
+
+
+class ScalerSpecializer[DefaultScalerT: Scaler](BaseScalerSpecializer[DefaultScalerT], Scaler, abstract=True):
+    """
+    An abstract base class to provide specialization logic for Scaler-like classes.
+    """
+
+
+_ScalerWithCatromDefaultT = TypeVar("_ScalerWithCatromDefaultT", bound=Scaler, default=Catrom)
+
+
+class NoScale(ScalerSpecializer[_ScalerWithCatromDefaultT]):
     """
     A utility scaler class that performs no scaling on the input clip.
 
     If used without a specified scaler, it defaults to inheriting from `Catrom`.
     """
 
-    _default_scaler = Catrom
+    default_scaler = Catrom
 
     def scale(
         self,
@@ -192,7 +191,7 @@ class NoScale(BaseScalerSpecializer[_ScalerWithCatromDefaultT], Scaler, partial_
         height: int | None = None,
         shift: tuple[TopShift, LeftShift] = (0, 0),
         **kwargs: Any,
-    ) -> vs.VideoNode | ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Return the input clip unscaled, validating that the dimensions are consistent.
 
@@ -234,33 +233,37 @@ class NoScale(BaseScalerSpecializer[_ScalerWithCatromDefaultT], Scaler, partial_
         return NoScale[Scaler.from_param(scaler)]  # type: ignore[return-value,misc]
 
 
-NoScaleLike = str | type[NoScale[_ScalerT]] | NoScale[_ScalerT]
+_ScalerWithScalerDefaultT = TypeVar("_ScalerWithScalerDefaultT", bound=Scaler, default=Scaler)
+
+# TODO: type NoScaleLike[_ScalerT: Scaler = Scaler] = str | type[NoScale[_ScalerT]] | NoScale[_ScalerT]
+NoScaleLike: TypeAlias = str | type[NoScale[_ScalerWithScalerDefaultT]] | NoScale[_ScalerWithScalerDefaultT]
 """
 Type alias for anything that can resolve to a NoScale scaler.
 
 This includes:
- - A string identifier.
- - A class type subclassing `NoScale`.
- - An instance of `NoScale`.
+
+- A string identifier.
+- A class type subclassing [NoScale][vskernels.NoScale].
+- An instance of [NoScale][vskernels.NoScale].
 """
 
 
-class BaseMixedScalerMeta(BaseScalerSpecializerMeta, Generic[Unpack[_BaseScalerTs]]):
+class BaseMixedScalerMeta[*_BaseScalerTs](BaseScalerSpecializerMeta):
     """
     Meta class for BaseMixedScaler to handle mixed scaling logic.
     """
 
-    __others__: tuple[Unpack[_BaseScalerTs]]
+    __others__: tuple[*_BaseScalerTs]
 
     def __new__(
         mcls,
         name: str,
         bases: tuple[type, ...],
         namespace: dict[str, Any],
-        *others: Unpack[_BaseScalerTs],
+        *others: *_BaseScalerTs,
         specializer: type[BaseScaler] | None = None,
         **kwargs: Any,
-    ) -> BaseMixedScalerMeta[Unpack[_BaseScalerTs]]:
+    ) -> BaseMixedScalerMeta[*_BaseScalerTs]:
         obj = super().__new__(mcls, name, bases, namespace, specializer=specializer, **kwargs)
 
         if others:
@@ -278,19 +281,16 @@ class BaseMixedScalerMeta(BaseScalerSpecializerMeta, Generic[Unpack[_BaseScalerT
         return obj
 
 
-class BaseMixedScaler(
-    BaseScalerSpecializer[_BaseScalerT],
-    Generic[_BaseScalerT, Unpack[_BaseScalerTs]],
-    metaclass=BaseMixedScalerMeta,
-    abstract=True,
+class BaseMixedScaler[DefaultScalerT: BaseScaler, *_BaseScalerTs](
+    BaseScalerSpecializer[DefaultScalerT], metaclass=BaseMixedScalerMeta, abstract=True
 ):
     """
-    An abstract base class to provide mixed or chained scaling for Scaler-like classes.
+    An abstract base class to provide mixed or chained scaling for BaseScaler-like classes.
     """
 
     @classproperty
     @classmethod
-    def _others(cls) -> tuple[Unpack[_BaseScalerTs]]:
+    def _others(cls) -> tuple[*_BaseScalerTs]:
         # Workaround as we can't specify the bound values of a TypeVarTuple yet
         return tuple(o() for o in cls.__others__)  # type: ignore[operator]
 
@@ -308,31 +308,33 @@ class BaseMixedScaler(
             specializer, *others = scalers
         else:
             specializer = scalers
-            others = []
+            others = ()
 
         if isinstance(specializer, TypeVar):
             cls.__others__ = tuple(others) or cls.__others__
             return GenericAlias(cls, (specializer, *cls.__others__))
 
-        mixed_spe = BaseMixedScalerMeta.__new__(
-            BaseMixedScalerMeta,
+        mixed_spe = BaseMixedScalerMeta(
             cls.__name__,
             (cls,),
             cls.__dict__.copy(),
             *others,
             specializer=specializer,
-            partial_abstract=True,
+            partial_abstract=specializer in partial_abstract_kernels,
+            abstract=specializer in abstract_kernels,
         )
 
         return GenericAlias(mixed_spe, (specializer, *others))
 
 
-class MixedScalerProcess(BaseMixedScaler[_ScalerT, Unpack[_BaseScalerTs]], Scaler, abstract=True):
+class MixedScalerProcess[DefaultScalerT: Scaler, *_BaseScalerTs](
+    BaseMixedScaler[DefaultScalerT, *_BaseScalerTs], Scaler, abstract=True
+):
     """
     An abstract class for chained scaling with an additional processing step.
     """
 
-    def __init__(self, *, function: VSFunctionNoArgs[vs.VideoNode, vs.VideoNode], **kwargs: Any) -> None:
+    def __init__(self, *, function: VSFunctionNoArgs, **kwargs: Any) -> None:
         """
         Initialize the MixedScalerProcess.
 
@@ -346,22 +348,29 @@ class MixedScalerProcess(BaseMixedScaler[_ScalerT, Unpack[_BaseScalerTs]], Scale
 
 
 @dataclass
-class LinearLightProcessing(vs_object):
+class LinearLightProcessing(VSObject):
     ll: LinearLight
 
     def get_linear(self) -> vs.VideoNode:
         """
         Getter for `linear` cached property.
         """
+        if (
+            self.ll._wclip.format.sample_type,
+            self.ll._wclip.format.subsampling_w,
+            self.ll._wclip.format.subsampling_h,
+        ) != (vs.FLOAT, 0, 0):
+            fmt = vs.RGBS if self.ll._wclip.format.color_family in (vs.YUV, vs.RGB) else vs.GRAYS
+        else:
+            fmt = self.ll._wclip
+
         wclip = self.ll._resampler.resample(
-            self.ll._wclip,
-            vs.RGBS if self.ll._wclip.format.color_family in (vs.YUV, vs.RGB) else vs.GRAYS,
-            matrix_in=self.ll._matrix,
-            transfer_in=self.ll._curve,
-            transfer=Transfer.LINEAR,
+            self.ll._wclip, fmt, matrix_in=self.ll._matrix, transfer_in=self.ll._curve, transfer=Transfer.LINEAR
         )
 
         if self.ll.sigmoid:
+            from vsexprtools import norm_expr
+
             wclip = norm_expr(
                 wclip,
                 "{center} 1 {slope} / 1 x 0 max 1 min {scale} * {offset} + / 1 - log * -",
@@ -400,6 +409,8 @@ class LinearLightProcessing(vs_object):
             raise CustomValueError("You need to set .linear before getting .out!", self.__class__)
 
         if self.ll.sigmoid:
+            from vsexprtools import norm_expr
+
             processed = norm_expr(
                 self._linear,
                 "1 1 {slope} {center} x 0 max 1 min - * exp + / {offset} - {scale} /",
@@ -416,13 +427,9 @@ class LinearLightProcessing(vs_object):
 
         return resample_to(processed, self.ll._fmt, self.ll._matrix, self.ll._resampler)
 
-    def __vs_del__(self, core_id: int) -> None:
-        del self._linear
-        cachedproperty.clear_cache(self)
-
 
 @dataclass
-class LinearLight(AbstractContextManager[LinearLightProcessing], vs_object):
+class LinearLight(VSObjectABC, AbstractContextManager[LinearLightProcessing]):
     """
     Utility class for processing a clip in linear format.
 
@@ -453,7 +460,7 @@ class LinearLight(AbstractContextManager[LinearLightProcessing], vs_object):
 
     @overload
     @classmethod
-    def from_func(
+    def from_func[**P](
         cls,
         func: Callable[Concatenate[vs.VideoNode, P], vs.VideoNode],
         /,
@@ -472,7 +479,7 @@ class LinearLight(AbstractContextManager[LinearLightProcessing], vs_object):
 
     @overload
     @classmethod
-    def from_func(
+    def from_func[**P](
         cls,
         /,
         *,
@@ -492,19 +499,19 @@ class LinearLight(AbstractContextManager[LinearLightProcessing], vs_object):
         """
 
     @classmethod
-    def from_func(
+    def from_func[**P](
         cls,
         func: Callable[Concatenate[vs.VideoNode, P], vs.VideoNode] | None = None,
         /,
         sigmoid: bool | tuple[Slope, Center] = False,
         resampler: ResamplerLike = Catrom,
         out_fmt: int | VideoFormatLike | HoldsVideoFormat | None = None,
-    ) -> Union[
-        Callable[Concatenate[vs.VideoNode, P], vs.VideoNode],
-        Callable[
+    ) -> (
+        Callable[Concatenate[vs.VideoNode, P], vs.VideoNode]
+        | Callable[
             [Callable[Concatenate[vs.VideoNode, P], vs.VideoNode]], Callable[Concatenate[vs.VideoNode, P], vs.VideoNode]
-        ],
-    ]:
+        ]
+    ):
         """
         Decorator version of LinearLight.
         """
@@ -521,8 +528,6 @@ class LinearLight(AbstractContextManager[LinearLightProcessing], vs_object):
         return _wrapped
 
     def __enter__(self) -> LinearLightProcessing:
-        assert check_variable_format(self.clip, self.__class__)
-
         if self.sigmoid is not False:
             if self.sigmoid is True:
                 self.sigmoid = (6.5, 0.75)
@@ -552,11 +557,6 @@ class LinearLight(AbstractContextManager[LinearLightProcessing], vs_object):
     def __exit__(self, *args: object, **kwargs: Any) -> None:
         self._exited = True
 
-    def __vs_del__(self, core_id: int) -> None:
-        for name in ("clip", "out_fmt", "_fmt", "_wclip"):
-            with suppress(AttributeError):
-                delattr(self, name)
-
 
 def resample_to(
     clip: vs.VideoNode,
@@ -588,59 +588,78 @@ def _is_base_scaler_like(obj: Any, base_scaler: type[BaseScaler]) -> bool:
     if isinstance(obj, str):
         try:
             base_scaler.from_param(obj)
-        except base_scaler._err_class:
-            return False
-        else:
             return True
+        except base_scaler._err_class:
+            pass
 
     return False
 
 
 def is_scaler_like(obj: Any) -> TypeIs[ScalerLike]:
-    """Returns true if obj is a ScalerLike"""
+    """Returns true if obj is a ScalerLike."""
     return _is_base_scaler_like(obj, Scaler)
 
 
 def is_descaler_like(obj: Any) -> TypeIs[DescalerLike]:
-    """Returns true if obj is a DescalerLike"""
+    """Returns true if obj is a DescalerLike."""
     return _is_base_scaler_like(obj, Descaler)
 
 
 def is_resampler_like(obj: Any) -> TypeIs[ResamplerLike]:
-    """Returns true if obj is a ResamplerLike"""
+    """Returns true if obj is a ResamplerLike."""
     return _is_base_scaler_like(obj, Resampler)
 
 
 def is_kernel_like(obj: Any) -> TypeIs[KernelLike]:
-    """Returns true if obj is a KernelLike"""
+    """Returns true if obj is a KernelLike."""
     return _is_base_scaler_like(obj, Kernel)
 
 
 def is_bobber_like(obj: Any) -> TypeIs[BobberLike]:
-    """Returns true if obj is a BobberLike"""
+    """Returns true if obj is a BobberLike."""
     return _is_base_scaler_like(obj, Bobber)
 
 
 def is_complex_scaler_like(obj: Any) -> TypeIs[ComplexScalerLike]:
-    """Returns true if obj is a ComplexScalerLike"""
+    """Returns true if obj is a ComplexScalerLike."""
     return _is_base_scaler_like(obj, ComplexScaler)
 
 
 def is_complex_descaler_like(obj: Any) -> TypeIs[ComplexDescalerLike]:
-    """Returns true if obj is a ComplexDescalerLike"""
+    """Returns true if obj is a ComplexDescalerLike."""
     return _is_base_scaler_like(obj, ComplexDescaler)
 
 
 def is_complex_kernel_like(obj: Any) -> TypeIs[ComplexKernelLike]:
-    """Returns true if obj is a ComplexKernelLike"""
+    """Returns true if obj is a ComplexKernelLike."""
     return _is_base_scaler_like(obj, ComplexKernel)
 
 
 def is_custom_complex_kernel_like(obj: Any) -> TypeIs[CustomComplexKernelLike]:
-    """Returns true if obj is a CustomComplexKernelLike"""
+    """Returns true if obj is a CustomComplexKernelLike."""
     return _is_base_scaler_like(obj, CustomComplexKernel)
 
 
-def is_noscale_like(obj: Any) -> TypeIs[NoScaleLike[Scaler]]:
-    """Returns true if obj is a NoScaleLike"""
-    return _is_base_scaler_like(obj, NoScale)
+def is_noscale_like[_ScalerT: Scaler](obj: Any, specializer: type[_ScalerT] = Scaler) -> TypeIs[NoScaleLike[_ScalerT]]:  # type: ignore[assignment]
+    """
+    Returns true if obj is a NoScaleLike.
+    """
+    if isinstance(obj, NoScale):
+        return isinstance(obj.specializer, specializer)
+
+    if isinstance(obj, str):
+        try:
+            NoScale.from_param(obj)
+            return True
+        except NoScale._err_class:
+            return False
+
+    if isinstance(obj, GenericAlias):
+        obj = get_origin(obj)
+
+    if isinstance(obj, BaseScalerSpecializerMeta) and issubclass(obj, NoScale):
+        return (obj.__isspecialized__ and issubclass(obj.__specializer__, specializer)) or issubclass(
+            obj.default_scaler, specializer
+        )
+
+    return False
