@@ -3,22 +3,19 @@ from __future__ import annotations
 from abc import ABC, abstractmethod
 from typing import Any, Sequence, overload
 
+from jetpytools import CustomTypeError, inject_self, normalize_seq
+
 from vsexprtools import ExprOp
 from vsrgtools import box_blur
 from vstools import (
     ColorRange,
-    ConstantFormatVideoNode,
-    CustomTypeError,
     FrameRangeN,
     FrameRangesN,
     FramesLengthError,
     Position,
     Size,
-    check_variable,
     depth,
-    inject_self,
     limiter,
-    normalize_seq,
     replace_ranges,
     vs,
 )
@@ -32,12 +29,12 @@ class GeneralMask(ABC):
     """
 
     @abstractmethod
-    def get_mask(self, clip: vs.VideoNode, /, *args: Any, **kwargs: Any) -> ConstantFormatVideoNode: ...
+    def get_mask(self, clip: vs.VideoNode, /, *args: Any, **kwargs: Any) -> vs.VideoNode: ...
 
     @inject_self.init_kwargs.clean
     def apply_mask(
         self, _clipa: vs.VideoNode, _clipb: vs.VideoNode, _ref: vs.VideoNode | None = None, /, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         return _clipa.std.MaskedMerge(_clipb, self.get_mask(_ref or _clipa, **kwargs))
 
 
@@ -64,7 +61,7 @@ class BoundingBox(GeneralMask):
 
         self.size, self.pos, self.invert = Size(size), Position(pos), invert
 
-    def get_mask(self, ref: vs.VideoNode, /, *args: Any, **kwargs: Any) -> ConstantFormatVideoNode:
+    def get_mask(self, ref: vs.VideoNode, /, *args: Any, **kwargs: Any) -> vs.VideoNode:
         from .utils import squaremask
 
         return squaremask(ref, self.size.x, self.size.y, self.pos.x, self.pos.y, self.invert, False, func=self.get_mask)
@@ -110,9 +107,9 @@ class DeferredMask(GeneralMask):
             raise FramesLengthError(self.__class__, "", "Received reference frame and range list size mismatch!")
 
     @limiter(mask=True)
-    def get_mask(self, clip: vs.VideoNode, /, ref: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def get_mask(self, clip: vs.VideoNode, /, ref: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         """
-        Get the constructed mask
+        Get the constructed mask.
 
         Args:
             clip: Source clip.
@@ -120,11 +117,8 @@ class DeferredMask(GeneralMask):
             **kwargs: Keyword arguments passed to the internal `_mask` method.
 
         Returns:
-            Constructed mask
+            Constructed mask.
         """
-        assert check_variable(clip, self.get_mask)
-        assert check_variable(ref, self.get_mask)
-
         if self.refframes:
             hm = vs.core.std.BlankClip(
                 ref, format=ref.format.replace(color_family=vs.GRAY, subsampling_h=0, subsampling_w=0).id, keep=True
@@ -156,4 +150,4 @@ class DeferredMask(GeneralMask):
         return hm
 
     @abstractmethod
-    def _mask(self, clip: vs.VideoNode, ref: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode: ...
+    def _mask(self, clip: vs.VideoNode, ref: vs.VideoNode, **kwargs: Any) -> vs.VideoNode: ...
