@@ -7,26 +7,19 @@ from __future__ import annotations
 from abc import ABC, abstractmethod
 from enum import IntFlag, auto
 from inspect import isabstract
-from typing import TYPE_CHECKING, Any, ClassVar, Sequence, TypeAlias, TypeVar
+from typing import TYPE_CHECKING, Any, ClassVar, Self, Sequence
 
-from jetpytools import inject_kwargs_params, to_arr
-from typing_extensions import Self
+from jetpytools import FuncExcept, get_subclasses, inject_kwargs_params, inject_self, to_arr
 
 from vsexprtools import ExprList, ExprOp, norm_expr
 from vstools import (
     ColorRange,
-    ConstantFormatVideoNode,
     ConvMode,
-    FuncExcept,
     Planes,
-    T,
-    check_variable,
     core,
     depth,
     get_peak_value,
-    get_subclasses,
     get_video_format,
-    inject_self,
     join,
     normalize_planes,
     scale_mask,
@@ -93,7 +86,7 @@ class MagDirection(IntFlag):
     SOUTH = S | SW | SE
     """All southern directions (South, Southwest, Southeast)."""
 
-    def select_matrices(self, matrices: Sequence[T]) -> Sequence[T]:
+    def select_matrices[T](self, matrices: Sequence[T]) -> Sequence[T]:
         """
         Return matrices matching the active directions in `self`.
 
@@ -111,12 +104,12 @@ class MagDirection(IntFlag):
         return [matrix for flag, matrix in zip(primary_flags, matrices) if self & flag]
 
 
-def _base_from_param(
-    cls: type[EdgeDetectTypeVar],
-    value: str | type[EdgeDetectTypeVar] | EdgeDetectTypeVar | None,
+def _base_from_param[_EdgeDetectT: EdgeDetect](
+    cls: type[_EdgeDetectT],
+    value: str | type[_EdgeDetectT] | _EdgeDetectT | None,
     exception_cls: type[_UnknownMaskDetectError],
     func_except: FuncExcept | None = None,
-) -> type[EdgeDetectTypeVar]:
+) -> type[_EdgeDetectT]:
     # If value is an instance returns the class
     if isinstance(value, cls):
         return value.__class__
@@ -140,11 +133,11 @@ def _base_from_param(
     raise exception_cls(func_except or cls.from_param, str(value))
 
 
-def _base_ensure_obj(
-    cls: type[EdgeDetectTypeVar],
-    value: str | type[EdgeDetectTypeVar] | EdgeDetectTypeVar | None,
+def _base_ensure_obj[_EdgeDetectT: EdgeDetect](
+    cls: type[_EdgeDetectT],
+    value: str | type[_EdgeDetectT] | _EdgeDetectT | None,
     func_except: FuncExcept | None = None,
-) -> EdgeDetectTypeVar:
+) -> _EdgeDetectT:
     if isinstance(value, cls):
         return value
 
@@ -182,7 +175,7 @@ class EdgeDetect(ABC):
         clamp: bool | tuple[float, float] | list[tuple[float, float]] = False,
         planes: Planes = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Makes edge mask based on convolution kernel.
 
@@ -199,8 +192,6 @@ class EdgeDetect(ABC):
         Returns:
             Mask clip
         """
-        assert check_variable(clip, self.__class__)
-
         clip_p = self._preprocess(clip, **kwargs)
 
         mask = self._compute_edge_mask(clip_p, multi=multi, planes=planes, **kwargs)
@@ -210,9 +201,7 @@ class EdgeDetect(ABC):
         return self._finalize_mask(mask, lthr, hthr, clamp, planes)
 
     @classmethod
-    def _depth_scale(cls, clip: vs.VideoNode, bitdepth: vs.VideoNode) -> ConstantFormatVideoNode:
-        assert check_variable(clip, cls)
-
+    def _depth_scale(cls, clip: vs.VideoNode, bitdepth: vs.VideoNode) -> vs.VideoNode:
         fmt = get_video_format(bitdepth)
 
         if fmt.sample_type == vs.INTEGER:
@@ -251,12 +240,12 @@ class EdgeDetect(ABC):
 
     def _finalize_mask(
         self,
-        mask: ConstantFormatVideoNode,
+        mask: vs.VideoNode,
         lthr: float | Sequence[float] | None,
         hthr: float | Sequence[float] | None,
         clamp: bool | tuple[float, float] | list[tuple[float, float]],
         planes: Planes,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         if not any([lthr, hthr, clamp]):
             planes = normalize_planes(mask, planes)
 
@@ -308,17 +297,17 @@ class EdgeDetect(ABC):
     @abstractmethod
     def _compute_edge_mask(
         self,
-        clip: ConstantFormatVideoNode,
+        clip: vs.VideoNode,
         *,
         multi: float | Sequence[float] = 1.0,
         planes: Planes = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode: ...
+    ) -> vs.VideoNode: ...
 
-    def _preprocess(self, clip: ConstantFormatVideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def _preprocess(self, clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         return ColorRange.FULL.apply(clip)
 
-    def _postprocess(self, clip: ConstantFormatVideoNode, input_bits: vs.VideoNode) -> ConstantFormatVideoNode:
+    def _postprocess(self, clip: vs.VideoNode, input_bits: vs.VideoNode) -> vs.VideoNode:
         return clip
 
 
@@ -332,12 +321,12 @@ class TCannyEdgeDetect(EdgeDetect):
 
     def _compute_edge_mask(
         self,
-        clip: ConstantFormatVideoNode,
+        clip: vs.VideoNode,
         *,
         multi: float | Sequence[float] = 1.0,
         planes: Planes = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         tcanny_args = {"op": self._op, "planes": planes} | {"sigma": 0, "mode": 1} | kwargs
 
         if not isinstance(multi, Sequence):
@@ -368,12 +357,12 @@ class MatrixEdgeDetect(EdgeDetect):
 
     def _compute_edge_mask(
         self,
-        clip: ConstantFormatVideoNode,
+        clip: vs.VideoNode,
         *,
         multi: float | Sequence[float] = 1.0,
         planes: Planes = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         exprs = [
             ExprOp.convolution("x", mat, divisor=div, saturate=False, mode=ConvMode(mode))[0]
             for mat, div, mode in zip(self._get_matrices(), self._get_divisors(), self._get_mode_types())
@@ -389,7 +378,7 @@ class MatrixEdgeDetect(EdgeDetect):
         multi: float | Sequence[float] = 1.0,
         planes: Planes = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode: ...
+    ) -> vs.VideoNode: ...
 
     def _get_matrices(self) -> Sequence[Sequence[float]]:
         return self.matrices
@@ -417,25 +406,25 @@ class EdgeMasksEdgeDetect(MatrixEdgeDetect):
         clamp: bool | tuple[float, float] | list[tuple[float, float]] = False,
         planes: Planes = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         if not hasattr(core, "edgemasks"):
             kwargs.setdefault("use_expr", True)
 
         return super().edgemask(clip, lthr, hthr, multi, clamp, planes, **kwargs)
 
-    def _preprocess(self, clip: ConstantFormatVideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def _preprocess(self, clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         if kwargs.get("use_expr"):
             return super()._preprocess(clip)
         return clip
 
     def _compute_edge_mask(
         self,
-        clip: ConstantFormatVideoNode,
+        clip: vs.VideoNode,
         *,
         multi: float | Sequence[float] = 1.0,
         planes: Planes = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         if kwargs.pop("use_expr", False):
             return super()._compute_edge_mask(clip, multi=multi, planes=planes, **kwargs)
 
@@ -447,10 +436,10 @@ class NormalizeProcessor(MatrixEdgeDetect):
     Edge detection processor with normalized precision.
     """
 
-    def _preprocess(self, clip: ConstantFormatVideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def _preprocess(self, clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         return super()._preprocess(depth(clip, 32))
 
-    def _postprocess(self, clip: ConstantFormatVideoNode, input_bits: vs.VideoNode) -> ConstantFormatVideoNode:
+    def _postprocess(self, clip: vs.VideoNode, input_bits: vs.VideoNode) -> vs.VideoNode:
         return super()._postprocess(self._depth_scale(clip, input_bits), input_bits)
 
 
@@ -511,7 +500,7 @@ class MagnitudeEdgeMasks(EdgeMasksEdgeDetect, MagnitudeMatrix):
         clamp: bool | tuple[float, float] | list[tuple[float, float]] = False,
         planes: Planes = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         if self.mag_directions != MagDirection.ALL:
             kwargs["use_expr"] = True
 
@@ -531,7 +520,7 @@ class SingleMatrix(MatrixEdgeDetect, ABC):
         multi: float | Sequence[float] = 1.0,
         planes: Planes = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         return norm_expr(
             clip,
             [exprs[0], "{multi}"],
@@ -555,7 +544,7 @@ class EuclideanDistance(MatrixEdgeDetect, ABC):
         multi: float | Sequence[float] = 1.0,
         planes: Planes = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         return norm_expr(
             clip,
             [exprs[0], "dup *", exprs[1], "dup * + sqrt 0 max", "{multi}"],
@@ -581,7 +570,7 @@ class Max(MatrixEdgeDetect, ABC):
         multi: float | Sequence[float] = 1.0,
         planes: Planes = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         return norm_expr(
             clip,
             [*exprs, ExprOp.MAX * (len(exprs) - 1), "{multi}"],
@@ -627,9 +616,6 @@ class RidgeDetect(MatrixEdgeDetect):
         Returns:
             Mask clip
         """
-
-        assert check_variable(clip, self.__class__)
-
         clip_p = self._preprocess_ridge(clip)
 
         mask = self._compute_ridge_mask(clip_p, multi=multi, planes=planes, **kwargs)
@@ -640,28 +626,28 @@ class RidgeDetect(MatrixEdgeDetect):
 
     def _compute_ridge_mask(
         self,
-        clip: ConstantFormatVideoNode,
+        clip: vs.VideoNode,
         *,
         multi: float | Sequence[float] = 1.0,
         planes: Planes = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
-        def _x(c: ConstantFormatVideoNode) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
+        def _x(c: vs.VideoNode) -> vs.VideoNode:
             return c.std.Convolution(matrix=self._get_matrices()[0], divisor=self._get_divisors()[0], planes=planes)
 
-        def _y(c: ConstantFormatVideoNode) -> ConstantFormatVideoNode:
+        def _y(c: vs.VideoNode) -> vs.VideoNode:
             return c.std.Convolution(matrix=self._get_matrices()[1], divisor=self._get_divisors()[1], planes=planes)
 
         return self._merge_ridge([_x(clip), _y(clip)], multi=multi, planes=planes, **kwargs)
 
     def _merge_ridge(
         self,
-        clips: Sequence[ConstantFormatVideoNode],
+        clips: Sequence[vs.VideoNode],
         *,
         multi: float | Sequence[float] = 1.0,
         planes: Planes = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         (xx,) = ExprOp.convolution("x", self._get_matrices()[0], divisor=self._get_divisors()[0], mode=ConvMode.SQUARE)
         (yy,) = ExprOp.convolution("y", self._get_matrices()[1], divisor=self._get_divisors()[1], mode=ConvMode.SQUARE)
         (xy,) = ExprOp.convolution("x", self._get_matrices()[1], divisor=self._get_divisors()[1], mode=ConvMode.SQUARE)
@@ -686,14 +672,14 @@ class RidgeDetect(MatrixEdgeDetect):
             **kwargs,
         )
 
-    def _preprocess_ridge(self, clip: ConstantFormatVideoNode) -> ConstantFormatVideoNode:
+    def _preprocess_ridge(self, clip: vs.VideoNode) -> vs.VideoNode:
         return depth(super()._preprocess(clip), 32)
 
-    def _postprocess_ridge(self, clip: ConstantFormatVideoNode, input_bits: vs.VideoNode) -> ConstantFormatVideoNode:
+    def _postprocess_ridge(self, clip: vs.VideoNode, input_bits: vs.VideoNode) -> vs.VideoNode:
         return self._depth_scale(super()._postprocess(clip, input_bits), input_bits)
 
 
-EdgeDetectLike: TypeAlias = type[EdgeDetect] | EdgeDetect | str
+type EdgeDetectLike = type[EdgeDetect] | EdgeDetect | str
 """
 Type alias for anything that can resolve to a EdgeDetect.
 
@@ -703,7 +689,7 @@ This includes:
  - An instance of a `EdgeDetect`.
 """
 
-RidgeDetectLike: TypeAlias = type[RidgeDetect] | RidgeDetect | str
+type RidgeDetectLike = type[RidgeDetect] | RidgeDetect | str
 """
 Type alias for anything that can resolve to a RidgeDetect.
 
@@ -719,9 +705,6 @@ EdgeDetectT = EdgeDetectLike
 RidgeDetectT = RidgeDetectLike
 """Deprecated alias of RidgeDetectLike."""
 
-EdgeDetectTypeVar = TypeVar("EdgeDetectTypeVar", bound=EdgeDetect)
-RidgeDetectTypeVar = TypeVar("RidgeDetectTypeVar", bound=RidgeDetect)
-
 
 def get_all_edge_detects(
     clip: vs.VideoNode,
@@ -731,7 +714,7 @@ def get_all_edge_detects(
     clamp: bool | tuple[float, float] | list[tuple[float, float]] = False,
     planes: Planes = None,
     **kwargs: Any,
-) -> list[ConstantFormatVideoNode]:
+) -> list[vs.VideoNode]:
     """
     Returns all the EdgeDetect subclasses
 
@@ -753,7 +736,7 @@ def get_all_edge_detects(
         if not isabstract(s) and s.__module__.split(".")[-1] != "_abstract"
     }
 
-    out = list[ConstantFormatVideoNode]()
+    out = list[vs.VideoNode]()
 
     for edge_detect in sorted(all_subclasses, key=lambda x: x.__name__):
         try:
@@ -777,7 +760,7 @@ def get_all_ridge_detect(
     clamp: bool | tuple[float, float] | list[tuple[float, float]] = False,
     planes: Planes = None,
     **kwargs: Any,
-) -> list[ConstantFormatVideoNode]:
+) -> list[vs.VideoNode]:
     """
     Returns all the RidgeDetect subclasses
 
@@ -799,7 +782,7 @@ def get_all_ridge_detect(
         if not isabstract(s) and s.__module__.split(".")[-1] != "_abstract"
     }
 
-    out = list[ConstantFormatVideoNode]()
+    out = list[vs.VideoNode]()
 
     for edge_detect in sorted(all_subclasses, key=lambda x: x.__name__):
         try:
