@@ -1,41 +1,39 @@
 from __future__ import annotations
 
 from abc import abstractmethod
-from dataclasses import dataclass, field
+from dataclasses import InitVar, dataclass, field
 from pathlib import Path
-from typing import Any, Mapping, Type
+from typing import TYPE_CHECKING, Any, Mapping, Type
+
+from jetpytools import CustomOverflowError, FileNotExistsError, FilePathType, fallback, iterate
 
 from vsexprtools import ExprOp, ExprToken, expr_func, norm_expr
 from vskernels import Point
 from vsrgtools import box_blur, median_blur
-from vssource import IMWRI, Indexer
+
+if TYPE_CHECKING:
+    from vssource import Indexer
+
 from vstools import (
     ColorRange,
-    ConstantFormatVideoNode,
-    CustomOverflowError,
-    FileNotExistsError,
-    FilePathType,
     FrameRangeN,
     FrameRangesN,
     Matrix,
     VSFunctionNoArgs,
-    check_variable,
+    VSObjectABC,
     core,
     depth,
-    fallback,
     get_lowest_value,
     get_neutral_value,
     get_neutral_values,
     get_peak_value,
     get_y,
-    iterate,
     limiter,
     normalize_ranges,
     replace_ranges,
     scale_delta,
     scale_value,
     vs,
-    vs_object,
 )
 
 from .abstract import BoundingBox, DeferredMask, GeneralMask
@@ -60,31 +58,32 @@ __all__ = [
 
 
 @dataclass
-class CustomMaskFromClipsAndRanges(GeneralMask, vs_object):
+class CustomMaskFromClipsAndRanges(VSObjectABC, GeneralMask):
     """
     Abstract CustomMaskFromClipsAndRanges interface
     """
 
     clips: list[vs.VideoNode] = field(init=False)
 
-    processing: VSFunctionNoArgs[vs.VideoNode, ConstantFormatVideoNode] = field(
-        default=core.lazy.std.BinarizeMask, kw_only=True
-    )
-    idx: Indexer | Type[Indexer] = field(default=IMWRI, kw_only=True)
+    processing: VSFunctionNoArgs = field(default=core.lazy.std.BinarizeMask, kw_only=True)
+    idx: InitVar[Indexer | Type[Indexer] | None] = field(default=None, kw_only=True)
 
-    def get_mask(self, ref: vs.VideoNode, /, *args: Any, **kwargs: Any) -> ConstantFormatVideoNode:
+    def __post_init__(self, idx: Indexer | Type[Indexer] | None) -> None:
+        from vssource import IMWRI
+
+        self._idx = idx if idx else IMWRI
+
+    def get_mask(self, ref: vs.VideoNode, /, *args: Any, **kwargs: Any) -> vs.VideoNode:
         """
-        Get the constructed mask
+        Get the constructed mask.
 
         Args:
             ref: Reference clip.
-            **kwargs: Keyword arguments passed to `replace_ranges` function.
+            **kwargs: Keyword arguments passed to [replace_ranges][vstools.replace_ranges] function.
 
         Returns:
-            Constructed mask
+            Constructed mask.
         """
-        assert check_variable(ref, self.get_mask)
-
         mask = vs.core.std.BlankClip(
             ref,
             format=ref.format.replace(color_family=vs.GRAY, subsampling_h=0, subsampling_w=0).id,
@@ -108,9 +107,6 @@ class CustomMaskFromClipsAndRanges(GeneralMask, vs_object):
     @abstractmethod
     def frame_ranges(self, clip: vs.VideoNode) -> list[list[tuple[int, int]]]: ...
 
-    def __vs_del__(self, core_id: int) -> None:
-        self.clips.clear()
-
 
 @dataclass
 class CustomMaskFromFolder(CustomMaskFromClipsAndRanges):
@@ -120,13 +116,14 @@ class CustomMaskFromFolder(CustomMaskFromClipsAndRanges):
 
     folder_path: FilePathType
 
-    def __post_init__(self) -> None:
+    def __post_init__(self, idx: Indexer | Type[Indexer] | None) -> None:
+        super().__post_init__(idx)
         if not (folder_path := Path(str(self.folder_path))).is_dir():
             raise FileNotExistsError('"folder_path" must be an existing path directory!', self.get_mask)
 
         self.files = list(folder_path.glob("*"))
 
-        self.clips = [self.idx.source(file, bits=-1) for file in self.files]
+        self.clips = [self._idx.source(file, bits=-1) for file in self.files]
 
     def frame_ranges(self, clip: vs.VideoNode) -> list[list[tuple[int, int]]]:
         return [
@@ -144,8 +141,8 @@ class CustomMaskFromRanges(CustomMaskFromClipsAndRanges):
 
     ranges: Mapping[FilePathType, FrameRangeN | FrameRangesN]
 
-    def __post_init__(self) -> None:
-        self.clips = [self.idx.source(str(file), bits=-1) for file in self.ranges]
+    def __post_init__(self, idx: Indexer | Type[Indexer] | None) -> None:
+        self.clips = [self._idx.source(str(file), bits=-1) for file in self.ranges]
 
     def frame_ranges(self, clip: vs.VideoNode) -> list[list[tuple[int, int]]]:
         return [normalize_ranges(clip, ranges) for ranges in self.ranges.values()]
@@ -160,7 +157,7 @@ class HardsubMask(DeferredMask):
 
     def get_progressive_dehardsub(
         self, hardsub: vs.VideoNode, ref: vs.VideoNode, partials: list[vs.VideoNode]
-    ) -> tuple[list[ConstantFormatVideoNode], list[ConstantFormatVideoNode]]:
+    ) -> tuple[list[vs.VideoNode], list[vs.VideoNode]]:
         """
         Dehardsub using multiple superior hardsubbed sources and one inferior non-subbed source.
 
@@ -172,11 +169,9 @@ class HardsubMask(DeferredMask):
         Returns:
             Dehardsub stages and masks used for progressive dehardsub.
         """
-        assert check_variable(hardsub, self.get_progressive_dehardsub)
-
         masks = [self.get_mask(hardsub, ref)]
         partials_dehardsubbed = [hardsub]
-        dehardsub_masks = list[ConstantFormatVideoNode]()
+        dehardsub_masks = list[vs.VideoNode]()
         partials = [*partials, ref]
 
         thr = scale_value(self.bin_thr, 32, masks[-1])
@@ -194,7 +189,7 @@ class HardsubMask(DeferredMask):
 
     def apply_dehardsub(
         self, hardsub: vs.VideoNode, ref: vs.VideoNode, partials: list[vs.VideoNode] | None = None
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Dehardsub using multiple superior hardsubbed sources and one inferior non-subbed source.
 
@@ -218,8 +213,8 @@ class HardsubMask(DeferredMask):
 class HardsubSignFades(HardsubMask):
     """
     Helper for hardsub scene filtering, typically used for de-hardsubbing signs during fades or hard-to-catch signs.
-    Originally written by Kageru from Kagefunc:
-    `https://github.com/Irrational-Encoding-Wizardry/kagefunc`
+
+    Originally written by Kageru from [kagefunc](https://github.com/Irrational-Encoding-Wizardry/kagefunc)
     """
 
     highpass: float
@@ -260,7 +255,7 @@ class HardsubSignFades(HardsubMask):
 
         super().__init__(ranges, bound, blur=blur, refframes=refframes)
 
-    def _mask(self, clip: vs.VideoNode, ref: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def _mask(self, clip: vs.VideoNode, ref: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         clipedge, refedge = (box_blur(normalize_mask(self.edgemask, x, **kwargs)) for x in (clip, ref))
 
         highpass = scale_delta(self.highpass, 32, clip)
@@ -274,7 +269,7 @@ class HardsubSignFades(HardsubMask):
 
 class HardsubSign(HardsubMask):
     """
-    Hardsub scenefiltering helper using `Zastin <https://github.com/kgrabs>`_'s hardsub mask.
+    Hardsub scenefiltering helper using [Zastin](https://github.com/kgrabs)'s hardsub mask.
     """
 
     thr: float
@@ -318,9 +313,7 @@ class HardsubSign(HardsubMask):
         super().__init__(ranges, bound, blur=blur, refframes=refframes)
 
     @limiter(mask=True)
-    def _mask(self, clip: vs.VideoNode, ref: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
-        assert check_variable(clip, self._mask)
-
+    def _mask(self, clip: vs.VideoNode, ref: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         hsmf = norm_expr([clip, ref], "x y - abs", func=self.__class__)
         hsmf = core.resize.Bilinear(hsmf, format=clip.format.replace(subsampling_w=0, subsampling_h=0).id)
 
@@ -337,8 +330,8 @@ class HardsubSign(HardsubMask):
 class HardsubLine(HardsubMask):
     """
     Helper for de-hardsubbing white text with black border subtitles.
-    Originally written by Kageru from Kagefunc:
-    `https://github.com/Irrational-Encoding-Wizardry/kagefunc`
+
+    Originally written by Kageru from [kagefunc](https://github.com/Irrational-Encoding-Wizardry/kagefunc)
     """
 
     expand: int | None
@@ -366,9 +359,7 @@ class HardsubLine(HardsubMask):
 
         super().__init__(ranges, bound, blur=blur, refframes=refframes)
 
-    def _mask(self, clip: vs.VideoNode, ref: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
-        assert check_variable(clip, self.__class__)
-
+    def _mask(self, clip: vs.VideoNode, ref: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         expand_n = fallback(self.expand, clip.width // 200)
 
         y_range = get_peak_value(clip) - get_lowest_value(clip)
@@ -444,7 +435,7 @@ class HardsubLineFade(HardsubLine):
 
         super().__init__(ranges, bound, expand, blur=blur, refframes=None)
 
-    def get_mask(self, clip: vs.VideoNode, /, ref: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def get_mask(self, clip: vs.VideoNode, /, ref: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         self.refframes = [r[0] + round((r[1] - r[0]) * self.ref_float) for r in normalize_ranges(ref, self.ranges)]
 
         return super().get_mask(clip, ref)
@@ -483,7 +474,7 @@ class HardsubASS(HardsubMask):
         super().__init__(ranges, bound, blur=blur, refframes=refframes)
 
     @limiter(mask=True)
-    def _mask(self, clip: vs.VideoNode, ref: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def _mask(self, clip: vs.VideoNode, ref: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         mask = core.sub.TextFile(ref, self.filename, fontdir=self.fontdir, blend=False).std.PropToClip("_Alpha")
 
         mask = mask.std.BinarizeMask(1)
@@ -496,28 +487,21 @@ class HardsubASS(HardsubMask):
 
 def bounded_dehardsub(
     hrdsb: vs.VideoNode, ref: vs.VideoNode, signs: list[HardsubMask], partials: list[vs.VideoNode] | None = None
-) -> ConstantFormatVideoNode:
-    assert check_variable(hrdsb, bounded_dehardsub)
+) -> vs.VideoNode:
     for sign in signs:
         hrdsb = sign.apply_dehardsub(hrdsb, ref, partials)
 
     return hrdsb
 
 
-def diff_hardsub_mask(a: vs.VideoNode, b: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
-    assert check_variable(a, diff_hardsub_mask)
-    assert check_variable(b, diff_hardsub_mask)
-
+def diff_hardsub_mask(a: vs.VideoNode, b: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
     return a.std.BlankClip(color=get_neutral_values(a), keep=True).std.MaskedMerge(
         a.std.MakeDiff(b), HardsubLine(**kwargs).get_mask(a, b)
     )
 
 
 @limiter(mask=True)
-def get_all_sign_masks(hrdsb: vs.VideoNode, ref: vs.VideoNode, signs: list[HardsubMask]) -> ConstantFormatVideoNode:
-    assert check_variable(hrdsb, get_all_sign_masks)
-    assert check_variable(ref, get_all_sign_masks)
-
+def get_all_sign_masks(hrdsb: vs.VideoNode, ref: vs.VideoNode, signs: list[HardsubMask]) -> vs.VideoNode:
     mask = core.std.BlankClip(
         ref, format=ref.format.replace(color_family=vs.GRAY, subsampling_w=0, subsampling_h=0).id, keep=True
     )
