@@ -1,29 +1,33 @@
 from __future__ import annotations
 
+from functools import partial
 from itertools import zip_longest
 from math import sqrt
-from typing import Any, Literal, Sequence, cast
+from typing import Any, Literal, Sequence
 
-from vsexprtools import ExprList, ExprOp, TupleExprList, norm_expr
-from vsrgtools import BlurMatrix
-from vstools import (
-    ConstantFormatVideoNode,
-    ConvMode,
+from jetpytools import (
     CustomValueError,
     FuncExcept,
-    GenericVSFunction,
-    Planes,
-    SpatialConvMode,
-    check_variable_format,
     copy_signature,
-    core,
     fallback,
     inject_self,
     interleave_arr,
     iterate,
+    to_arr,
+)
+
+from vsexprtools import ExprList, ExprOp, TupleExprList, norm_expr
+from vsrgtools import BlurMatrix
+from vstools import (
+    ConvMode,
+    GenericVSFunction,
+    Planes,
+    SpatialConvMode,
+    VSFunctionAllArgs,
+    core,
+    scale_delta,
     scale_mask,
     scale_value,
-    to_arr,
     vs,
 )
 
@@ -36,8 +40,9 @@ RadiusLike = int | tuple[int, SpatialConvMode]
 Type alias for specifying a convolution radius.
 
 This can be one of the following:
- - An `int`: Interpreted as a square convolution with size `(2 * radius + 1) x (2 * radius + 1)`.
- - A `tuple[int, SpatialConvMode]`: A pair specifying the radius and the convolution mode.
+
+- An `int`: Interpreted as a square convolution with size `(2 * radius + 1) x (2 * radius + 1)`.
+- A `tuple[int, SpatialConvMode]`: A pair specifying the radius and the convolution mode.
 """
 
 
@@ -60,9 +65,10 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Replaces each pixel with the largest value in its 3x3 neighbourhood.
+
         This operation is also known as dilation with a radius of 1.
 
         Args:
@@ -90,9 +96,10 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Replaces each pixel with the smallest value in its 3x3 neighbourhood.
+
         This operation is also known as erosion with a radius of 1.
 
         Args:
@@ -121,11 +128,11 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Replaces each pixel with the average of the (radius * 2 + 1) ** 2 - 1 pixels
-        in its (radius * 2 + 1)x(radius * 2 + 1) neighbourhood, but only if that average
-        is greater than the center pixel.
+        in its (radius * 2 + 1)x(radius * 2 + 1) neighbourhood, but only if that average is greater
+        than the center pixel.
 
         A radius of 1 will replace each pixel with the average of the 8 pixels in its 3x3 neighbourhood.
         A radius of 2 will replace each pixel with the average of the 24 pixels in its 5x5 neighbourhood.
@@ -160,7 +167,7 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Replaces each pixel with the average of the (radius * 2 + 1) ** 2 - 1 pixels
         in its (radius * 2 + 1)x(radius * 2 + 1) neighbourhood, but only if that average
@@ -199,7 +206,7 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Replaces each pixel with the largest value in its (radius * 2 + 1)x(radius * 2 + 1) neighbourhood.
 
@@ -243,7 +250,7 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Replaces each pixel with the smallest value in its (radius * 2 + 1)x(radius * 2 + 1) neighbourhood.
 
@@ -286,7 +293,7 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Replaces multiple times each pixel with the largest value in its 3x3 neighbourhood.
         Specifying a mode will allow custom growing mode.
@@ -316,7 +323,7 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Replaces multiple times each pixel with the smallest value in its 3x3 neighbourhood.
         Specifying a mode will allow custom growing mode.
@@ -347,7 +354,7 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         A closing is an dilation followed by an erosion.
 
@@ -384,7 +391,7 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         An opening is an erosion followed by an dilation.
 
@@ -421,7 +428,7 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         A morphological gradient is the difference between a dilation and erosion.
 
@@ -440,8 +447,6 @@ class Morpho:
         """
         func = func or self.gradient
 
-        assert check_variable_format(clip, func)
-
         if isinstance(radius, tuple):
             r, conv_mode = radius
         else:
@@ -480,7 +485,7 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         A top hat or a white hat is the difference of the original clip and the opening.
 
@@ -499,16 +504,14 @@ class Morpho:
         """
         func = func or self.top_hat
 
-        assert check_variable_format(clip, func)
-
         opened = self.opening(clip, radius, thr, iterations, coords, multiply, planes, func=func, **kwargs)
 
         return norm_expr([clip, opened], "x y -", planes, func=func)
 
     @copy_signature(top_hat)
     @inject_self
-    def white_hate(self, *args: Any, **kwargs: Any) -> ConstantFormatVideoNode:
-        """Alias for [top_hat][vsmasktools.Morpho.top_hat]"""
+    def white_hate(self, *args: Any, **kwargs: Any) -> vs.VideoNode:
+        """Alias for [top_hat][vsmasktools.Morpho.top_hat]."""
         return self.top_hat(*args, **{"func": self.white_hate} | kwargs)
 
     @inject_self
@@ -524,7 +527,7 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         A bottom hat or a black hat is the difference of the closing and the original clip.
 
@@ -543,16 +546,14 @@ class Morpho:
         """
         func = func or self.bottom_hat
 
-        assert check_variable_format(clip, func)
-
         closed = self.closing(clip, radius, thr, iterations, coords, multiply, planes, func=func, **kwargs)
 
         return norm_expr([closed, clip], "x y -", planes, func=func)
 
     @copy_signature(bottom_hat)
     @inject_self
-    def black_hat(self, *args: Any, **kwargs: Any) -> ConstantFormatVideoNode:
-        """Alias for [bottom_hat][vsmasktools.Morpho.bottom_hat]"""
+    def black_hat(self, *args: Any, **kwargs: Any) -> vs.VideoNode:
+        """Alias for [bottom_hat][vsmasktools.Morpho.bottom_hat]."""
         return self.top_hat(*args, **{"func": self.black_hat} | kwargs)
 
     @inject_self
@@ -568,7 +569,7 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         An outer hat is the difference of the dilation and the original clip.
 
@@ -587,8 +588,6 @@ class Morpho:
         """
         func = func or self.outer_hat
 
-        assert check_variable_format(clip, func)
-
         if isinstance(radius, tuple):
             r, conv_mode = radius
         else:
@@ -622,7 +621,7 @@ class Morpho:
         *,
         func: FuncExcept | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         An inner hat is the difference of the original clip and the erosion.
 
@@ -641,8 +640,6 @@ class Morpho:
         """
         func = func or self.inner_hat
 
-        assert check_variable_format(clip, func)
-
         if isinstance(radius, tuple):
             r, conv_mode = radius
         else:
@@ -671,7 +668,7 @@ class Morpho:
         lowval: float | Sequence[float] | None = None,
         highval: float | Sequence[float] | None = None,
         planes: Planes = None,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Turns every pixel in the image into either `lowval`, if it's below `midthr`, or `highval` otherwise.
 
@@ -699,7 +696,7 @@ class Morpho:
         lowval: float | Sequence[float] | None = None,
         highval: float | Sequence[float] | None = None,
         planes: Planes = None,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Turns every pixel in the mask clip into either `lowval` if it's below `midthr`, or `highval` otherwise.
 
@@ -716,7 +713,7 @@ class Morpho:
             planes: Specifies which planes to process. Unprocessed planes are copied unchanged.
         """
         midthr, lowval, highval = (
-            thr and [scale_value(t, 32, clip) for t in to_arr(thr)] for thr in (midthr, lowval, highval)
+            thr and [scale_mask(t, 32, clip) for t in to_arr(thr)] for thr in (midthr, lowval, highval)
         )
 
         return core.std.BinarizeMask(clip, midthr, lowval, highval, planes)
@@ -745,7 +742,7 @@ class Morpho:
             e = ExprList(interleave_arr(e, op * e.mlength, 2))
 
             if thr is not None:
-                e.append("x", scale_value(thr, 32, clip))
+                e.append("x", scale_delta(thr, 32, clip))
                 limit = (ExprOp.SUB, ExprOp.MAX) if op == ExprOp.MIN else (ExprOp.ADD, ExprOp.MIN)
                 e.append(*limit)
 
@@ -770,10 +767,10 @@ class Morpho:
         planes: Planes = None,
         *,
         func: FuncExcept,
-        mm_func: GenericVSFunction[ConstantFormatVideoNode],
+        mm_func: GenericVSFunction,
         op: Literal[ExprOp.MIN, ExprOp.MAX],
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         if isinstance(radius, tuple):
             radius, conv_mode = radius
         else:
@@ -799,14 +796,14 @@ class Morpho:
                         coords = Coordinates.RECTANGLE
 
             if thr is not None:
-                kwargs.update(threshold=scale_mask(thr, 32, clip))
+                kwargs.update(threshold=scale_delta(thr, 32, clip))
 
             kwargs.update(coordinates=coords, planes=planes)
 
             if multiply is not None:
                 mm_func = self._multiply_mm_func(mm_func, multiply)
 
-        return iterate(clip, mm_func, iterations, **kwargs)  # type: ignore[return-value]
+        return iterate(clip, mm_func, iterations, **kwargs)
 
     def _xxpand_transform(
         self,
@@ -820,7 +817,7 @@ class Morpho:
         op: Literal[ExprOp.MIN, ExprOp.MAX],
         func: FuncExcept,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         sh = fallback(sh, sw)
 
         function = self.maximum if op is ExprOp.MAX else self.minimum
@@ -837,7 +834,7 @@ class Morpho:
 
             clip = function(clip, thr, 1, coords, planes=planes, func=func, **kwargs)
 
-        return clip  # type: ignore[return-value]
+        return clip
 
     def _xxflate(
         self,
@@ -852,20 +849,20 @@ class Morpho:
         func: FuncExcept,
         inflate: bool,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         if isinstance(radius, tuple):
             radius, conv_mode = radius
         else:
             conv_mode = ConvMode.SQUARE
 
-        Func = GenericVSFunction[ConstantFormatVideoNode]  # noqa: N806
+        xxflate_func: VSFunctionAllArgs
 
         if radius == 1 and conv_mode == ConvMode.SQUARE and not coords:
-            xxflate_func: Func = core.lazy.std.Inflate if inflate else core.lazy.std.Deflate
+            xxflate_func = core.lazy.std.Inflate if inflate else core.lazy.std.Deflate
             kwargs.update(planes=planes)
 
             if thr is not None:
-                kwargs.update(threshold=scale_mask(thr, 32, clip))
+                kwargs.update(threshold=scale_delta(thr, 32, clip))
 
             if multiply is not None:
                 xxflate_func = self._multiply_mm_func(xxflate_func, multiply)
@@ -879,23 +876,21 @@ class Morpho:
                 e.append(ExprOp.ADD * e.mlength, len(e), ExprOp.DIV, "x", ExprOp.MAX if inflate else ExprOp.MIN)
 
                 if thr is not None:
-                    thr = scale_value(thr, 32, clip)
+                    thr = scale_delta(thr, 32, clip)
                     limit = ["x", thr, ExprOp.ADD, ExprOp.MIN] if inflate else ["x", thr, ExprOp.SUB, ExprOp.MAX]
                     e.append(limit)
 
                 if multiply is not None:
                     e.append(multiply, ExprOp.MUL)
 
-            kwargs.update(expr=expr, planes=planes, func=func)
+            kwargs.update(planes=planes, func=func)
 
-            xxflate_func = cast(Func, norm_expr)
+            xxflate_func = partial(norm_expr, expr=expr)
 
-        return iterate(clip, xxflate_func, iterations, **kwargs)  # pyright: ignore[reportReturnType]
+        return iterate(clip, xxflate_func, iterations, **kwargs)
 
-    def _multiply_mm_func(
-        self, func: GenericVSFunction[ConstantFormatVideoNode], multiply: float
-    ) -> GenericVSFunction[ConstantFormatVideoNode]:
-        def mm_func(clip: ConstantFormatVideoNode, *args: Any, **kwargs: Any) -> ConstantFormatVideoNode:
+    def _multiply_mm_func(self, func: GenericVSFunction, multiply: float) -> GenericVSFunction:
+        def mm_func(clip: vs.VideoNode, *args: Any, **kwargs: Any) -> vs.VideoNode:
             return norm_expr(func(clip, *args, **kwargs), "x {multiply} *", multiply=multiply, func=self.__class__)
 
         return mm_func
@@ -936,7 +931,7 @@ def grow_mask(
     *,
     func: FuncExcept | None = None,
     **kwargs: Any,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     func = func or grow_mask
 
     morpho = Morpho()
