@@ -2,24 +2,20 @@ from __future__ import annotations
 
 from typing import Sequence, overload
 
+from jetpytools import FuncExcept, StrList, fallback, to_arr
+
 from vsexprtools import ExprOp, ExprVars, norm_expr
 from vsrgtools import box_blur, gauss_blur
 from vstools import (
     ColorRange,
-    ConstantFormatVideoNode,
     DitherType,
-    FuncExcept,
-    StrList,
-    check_variable,
     depth,
-    fallback,
     get_peak_value,
     get_sample_type,
     get_y,
     limiter,
     plane,
     scale_value,
-    to_arr,
     vs,
 )
 
@@ -32,13 +28,13 @@ __all__ = ["adg_mask", "flat_mask", "retinex", "texture_mask"]
 @overload
 def adg_mask(
     clip: vs.VideoNode, luma_scaling: float = 8.0, relative: bool = False, func: FuncExcept | None = None
-) -> ConstantFormatVideoNode: ...
+) -> vs.VideoNode: ...
 
 
 @overload
 def adg_mask(
-    clip: vs.VideoNode, luma_scaling: Sequence[float] = ..., relative: bool = False, func: FuncExcept | None = None
-) -> list[ConstantFormatVideoNode]: ...
+    clip: vs.VideoNode, luma_scaling: Sequence[float], relative: bool = False, func: FuncExcept | None = None
+) -> list[vs.VideoNode]: ...
 
 
 def adg_mask(
@@ -46,7 +42,7 @@ def adg_mask(
     luma_scaling: float | Sequence[float] = 8.0,
     relative: bool = False,
     func: FuncExcept | None = None,
-) -> ConstantFormatVideoNode | list[ConstantFormatVideoNode]:
+) -> vs.VideoNode | list[vs.VideoNode]:
     """
     Generates an adaptive grain mask based on each frame's average luma and pixel value.
 
@@ -65,8 +61,6 @@ def adg_mask(
     """
     func = func or adg_mask
 
-    assert check_variable(clip, func)
-
     luma = plane(clip, 0)
 
     if clip.format.bits_per_sample > 16 or relative:
@@ -83,7 +77,7 @@ def adg_mask(
 
         x_string += "0 0.999 clamp X!"
 
-        def _adgfunc(luma: ConstantFormatVideoNode, ls: float) -> ConstantFormatVideoNode:
+        def _adgfunc(luma: vs.VideoNode, ls: float) -> vs.VideoNode:
             return norm_expr(
                 luma,
                 f"{x_string} 1 X@ X@ X@ X@ X@ "
@@ -94,7 +88,7 @@ def adg_mask(
     else:
         y, y_inv = luma.std.PlaneStats(), luma.std.Invert().std.PlaneStats()
 
-        def _adgfunc(luma: ConstantFormatVideoNode, ls: float) -> ConstantFormatVideoNode:
+        def _adgfunc(luma: vs.VideoNode, ls: float) -> vs.VideoNode:
             return luma.adg.Mask(ls)
 
     scaled_clips = [_adgfunc(y_inv if ls < 0 else y, abs(ls)) for ls in to_arr(luma_scaling)]
@@ -113,7 +107,7 @@ def retinex(
     upper_thr: float = 0.001,
     fast: bool = True,
     func: FuncExcept | None = None,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Multi-Scale Retinex (MSR) implementation for dynamic range and contrast enhancement.
 
@@ -132,8 +126,6 @@ def retinex(
     """
     func = func or retinex
 
-    assert check_variable(clip, func)
-
     sigma = sorted(sigma)
 
     y = get_y(clip)
@@ -171,7 +163,7 @@ def retinex(
     return norm_expr(msr_stats, expr_balance, None, y, func=func)
 
 
-def flat_mask(src: vs.VideoNode, radius: int = 5, thr: float = 0.011, gauss: bool = False) -> ConstantFormatVideoNode:
+def flat_mask(src: vs.VideoNode, radius: int = 5, thr: float = 0.011, gauss: bool = False) -> vs.VideoNode:
     luma = get_y(src)
 
     blur = gauss_blur(luma, radius * 0.361083333) if gauss else box_blur(luma, radius)
@@ -187,13 +179,13 @@ def texture_mask(
     clip: vs.VideoNode,
     rady: int = 2,
     radc: int | None = None,
-    blur: int | float = 8,
+    blur: float = 8,
     thr: float = 0.2,
     stages: list[tuple[int, int]] = [(60, 2), (40, 4), (20, 2)],
     points: list[tuple[bool, float]] = [(False, 1.75), (True, 2.5), (True, 5), (False, 10)],
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     levels = [x for x, _ in points]
-    _points = [scale_value(x, 8, clip) for _, x in points]
+    points_ = [scale_value(x, 8, clip) for _, x in points]
     thr = scale_value(thr, 8, 32, ColorRange.FULL)
 
     qm, peak = len(points), get_peak_value(clip)
@@ -206,20 +198,20 @@ def texture_mask(
         rmask, (Morpho.minimum(Morpho.binarize(emask, thr, 1.0, 0), iterations=it) for thr, it in stages)
     )
 
-    expr = [f"x {_points[0]} < x {_points[-1]} > or 0"]
+    expr = [f"x {points_[0]} < x {points_[-1]} > or 0"]
 
-    for x in range(len(_points) - 1):
-        if _points[x + 1] < _points[-1]:
-            expr.append(f"x {_points[x + 1]} <=")
+    for x in range(len(points_) - 1):
+        if points_[x + 1] < points_[-1]:
+            expr.append(f"x {points_[x + 1]} <=")
 
         if levels[x] == levels[x + 1]:
             expr.append(f"{peak if levels[x] else 0}")
         else:
-            mean = peak * (levels[x + 1] - levels[x]) / (_points[x + 1] - _points[x])
-            expr.append(f"x {_points[x]} - {mean} * {peak * levels[x]} +")
+            mean = peak * (levels[x + 1] - levels[x]) / (points_[x + 1] - points_[x])
+            expr.append(f"x {points_[x]} - {mean} * {peak * levels[x]} +")
 
     weighted = norm_expr(rm_txt, [expr, ExprOp.TERN * (qm - 1)], func=texture_mask)
 
-    weighted = gauss_blur(weighted, blur) if isinstance(blur, float) else box_blur(weighted, blur)
+    weighted = box_blur(weighted, blur) if isinstance(blur, int) else gauss_blur(weighted, blur)
 
     return norm_expr(weighted, f"x {peak * thr} - {1 / (1 - thr)} *", func=texture_mask)
