@@ -1,23 +1,18 @@
 from __future__ import annotations
 
-from typing import Any, Callable, Concatenate, Generic, Iterable, overload
+from typing import Any, Callable, Concatenate, Iterable, overload
 
-from jetpytools import P, R, SupportsString, cachedproperty
+from jetpytools import CustomValueError, FuncExcept, SupportsString, cachedproperty
 
 from vsexprtools import ExprOp, norm_expr
 from vskernels import Bilinear, Kernel, KernelLike
 from vsrgtools import box_blur, gauss_blur
 from vstools import (
     ColorRange,
-    ConstantFormatVideoNode,
-    CustomValueError,
     FrameRangeN,
     FrameRangesN,
-    FuncExcept,
     Planes,
     check_ref_clip,
-    check_variable,
-    check_variable_format,
     core,
     depth,
     flatten_vnodes,
@@ -46,13 +41,9 @@ __all__ = [
 ]
 
 
-def max_planes(
-    *_clips: vs.VideoNode | Iterable[vs.VideoNode], resizer: KernelLike = Bilinear
-) -> ConstantFormatVideoNode:
+def max_planes(*_clips: vs.VideoNode | Iterable[vs.VideoNode], resizer: KernelLike = Bilinear) -> vs.VideoNode:
     clips = flatten_vnodes(_clips)
 
-    assert check_variable_format(clips, max_planes)
-
     resizer = Kernel.ensure_obj(resizer, max_planes)
 
     width, height, fmt = clips[0].width, clips[0].height, clips[0].format.replace(subsampling_w=0, subsampling_h=0)
@@ -60,7 +51,7 @@ def max_planes(
     return ExprOp.MAX.combine(split(resizer.scale(clip, width, height, format=fmt)) for clip in clips)
 
 
-class RegionMask(Generic[P, R]):
+class RegionMask[**P, R]:
     """
     Class decorator that wraps [region_rel_mask][vsmasktools.region_rel_mask]
     and [region_abs_mask][vsmasktools.region_abs_mask] function and extends their functionality.
@@ -96,7 +87,7 @@ def region_rel_mask(
     replace_out: SupportsString | None = None,
     planes: Planes = None,
     func: FuncExcept | None = None,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Generates a mask that defines a rectangular region within the clip, replacing pixels inside or outside the region,
     using relative coordinates.
@@ -119,8 +110,6 @@ def region_rel_mask(
     """
     func = func or region_rel_mask
 
-    assert check_variable_format(clip, func)
-
     if replace_in is None:
         replace_in = "x"
     if replace_out is None:
@@ -163,7 +152,7 @@ def region_abs_mask(
     replace_out: SupportsString | None = None,
     planes: Planes = None,
     func: FuncExcept | None = None,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Generates a mask that defines a rectangular region within the clip, replacing pixels inside or outside the region,
     using absolute coordinates.
@@ -207,7 +196,7 @@ def squaremask(
     force_gray: bool = True,
     planes: Planes = None,
     func: FuncExcept | None = None,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Create a square used for simple masking.
 
@@ -229,8 +218,6 @@ def squaremask(
     """
     func = func or squaremask
 
-    assert check_variable(clip, func)
-
     mask_format = (
         clip.format.replace(color_family=vs.GRAY, subsampling_w=0, subsampling_h=0) if force_gray else clip.format
     )
@@ -256,11 +243,11 @@ def replace_squaremask(
     clipb: vs.VideoNode,
     mask_params: tuple[int, int, int, int],
     ranges: FrameRangeN | FrameRangesN | None = None,
-    blur_sigma: int | float | None = None,
+    blur_sigma: float | None = None,
     invert: bool = False,
     func: FuncExcept | None = None,
     show_mask: bool = False,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Replace an area of the frame with another clip using a simple square mask.
 
@@ -286,14 +273,13 @@ def replace_squaremask(
     """
     func = func or replace_squaremask
 
-    assert check_variable(clipa, func) and check_variable(clipb, func)
-
     mask = squaremask(clipb[0], *mask_params, invert, func=func)
 
-    if isinstance(blur_sigma, int):
-        mask = box_blur(mask, blur_sigma)
-    elif isinstance(blur_sigma, float):
-        mask = gauss_blur(mask, blur_sigma)
+    if blur_sigma:
+        if isinstance(blur_sigma, int):
+            mask = box_blur(mask, blur_sigma)
+        elif isinstance(blur_sigma, float):
+            mask = gauss_blur(mask, blur_sigma)
 
     mask = core.std.Loop(mask, clipa.num_frames)
 
@@ -316,7 +302,7 @@ def freeze_replace_squaremask(
     mask_params: tuple[int, int, int, int],
     frame: int,
     frame_range: tuple[int, int],
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     start, end = frame_range
 
     masked_insert = replace_squaremask(mask[frame], insert[frame], mask_params)
@@ -325,9 +311,7 @@ def freeze_replace_squaremask(
 
 
 @overload
-def normalize_mask(
-    mask: vs.VideoNode, clip: vs.VideoNode, *, func: FuncExcept | None = None
-) -> ConstantFormatVideoNode: ...
+def normalize_mask(mask: vs.VideoNode, clip: vs.VideoNode, *, func: FuncExcept | None = None) -> vs.VideoNode: ...
 
 
 @overload
@@ -337,7 +321,7 @@ def normalize_mask(
     ref: vs.VideoNode,
     *,
     func: FuncExcept | None = None,
-) -> ConstantFormatVideoNode: ...
+) -> vs.VideoNode: ...
 
 
 @overload
@@ -348,13 +332,13 @@ def normalize_mask(
     ridge: bool = ...,
     func: FuncExcept | None = None,
     **kwargs: Any,
-) -> ConstantFormatVideoNode: ...
+) -> vs.VideoNode: ...
 
 
 @overload
 def normalize_mask(
     mask: GeneralMask, clip: vs.VideoNode, ref: vs.VideoNode, *, func: FuncExcept | None = None
-) -> ConstantFormatVideoNode: ...
+) -> vs.VideoNode: ...
 
 
 @overload
@@ -366,7 +350,7 @@ def normalize_mask(
     ridge: bool = ...,
     func: FuncExcept | None = None,
     **kwargs: Any,
-) -> ConstantFormatVideoNode: ...
+) -> vs.VideoNode: ...
 
 
 def normalize_mask(
@@ -377,7 +361,7 @@ def normalize_mask(
     ridge: bool = False,
     func: FuncExcept | None = None,
     **kwargs: Any,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Normalize any mask type to match the format and range of the input clip.
 
@@ -424,7 +408,7 @@ def normalize_mask(
     return depth(cmask, clip, range_in=ColorRange.FULL, range_out=ColorRange.FULL)
 
 
-class RektPartial(Generic[P, R]):
+class RektPartial[**P, R]:
     """
     Class decorator that wraps the [rekt_partial][vsmasktools.utils.rekt_partial] function
     and extends its functionality.
@@ -451,7 +435,7 @@ class RektPartial(Generic[P, R]):
         offset_y: int = 0,
         *args: Any,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Creates a rectangular mask to apply fixes only within the masked area,
         significantly speeding up filters like anti-aliasing and scaling.
@@ -481,7 +465,7 @@ def rekt_partial(
     bottom: int = 0,
     *args: Any,
     **kwargs: Any,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Creates a rectangular mask to apply fixes only within the masked area,
     significantly speeding up filters like anti-aliasing and scaling.
@@ -498,10 +482,8 @@ def rekt_partial(
         A new clip with the applied mask.
     """
 
-    assert check_variable(clip, rekt_partial._func)
-
-    def _filtered_func(clip: vs.VideoNode, *args: Any, **kwargs: Any) -> ConstantFormatVideoNode:
-        return func(clip, *args, **kwargs)  # type: ignore[return-value]
+    def _filtered_func(clip: vs.VideoNode, *args: Any, **kwargs: Any) -> vs.VideoNode:
+        return func(clip, *args, **kwargs)
 
     if left == top == right == bottom == 0:
         return _filtered_func(clip, *args, **kwargs)
@@ -514,7 +496,7 @@ def rekt_partial(
 
     filtered = core.std.AddBorders(filtered, left, right, top, bottom)
 
-    ratio_w, ratio_h = 1 << clip.format.subsampling_w, 1 << clip.format.subsampling_h
+    ratio_w, ratio_h = 2**clip.format.subsampling_w, 2**clip.format.subsampling_h
 
     vals = list(
         filter(
