@@ -2,35 +2,27 @@ from __future__ import annotations
 
 from functools import partial, reduce
 from math import sqrt
-from typing import TYPE_CHECKING, Any, Callable, Generic, Literal, Sequence, Union, overload
+from typing import TYPE_CHECKING, Any, Callable, Literal, Sequence, Union, overload
 
-from jetpytools import CustomIntEnum, CustomStrEnum, FuncExcept, P, R, cround, to_arr
+from jetpytools import CustomIntEnum, CustomStrEnum, CustomValueError, FuncExcept, cround, normalize_seq, to_arr
 
 from vsexprtools import norm_expr
 from vskernels import Bilinear, Gaussian, Point, Scaler, ScalerLike
 from vstools import (
     ColorRange,
-    ConstantFormatVideoNode,
     ConvMode,
-    CustomValueError,
-    KwargsT,
     OneDimConvMode,
     Planes,
     SpatialConvMode,
     TempConvMode,
     VSFunctionNoArgs,
     check_ref_clip,
-    check_variable,
-    check_variable_format,
     core,
     depth,
     expect_bits,
     get_plane_sizes,
-    get_prop,
     join,
     normalize_planes,
-    normalize_seq,
-    shift_clip_multi,
     split,
     vs,
 )
@@ -60,7 +52,7 @@ def box_blur(
     mode: OneDimConvMode | TempConvMode = ConvMode.HV,
     planes: Planes = None,
     **kwargs: Any,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Applies a box blur to the input clip.
 
@@ -77,8 +69,6 @@ def box_blur(
     Returns:
         Blurred clip.
     """
-    assert check_variable(clip, box_blur)
-
     if isinstance(radius, Sequence):
         return normalize_radius(clip, box_blur, radius, planes, passes=passes, mode=mode, **kwargs)
 
@@ -102,9 +92,7 @@ def box_blur(
     return clip.vszip.BoxBlur(*box_args)
 
 
-def side_box_blur(clip: vs.VideoNode, radius: int = 1, planes: Planes = None) -> ConstantFormatVideoNode:
-    assert check_variable_format(clip, side_box_blur)
-
+def side_box_blur(clip: vs.VideoNode, radius: int = 1, planes: Planes = None) -> vs.VideoNode:
     half_kernel = [(1 if i <= 0 else 0) for i in range(-radius, radius + 1)]
 
     vrt_filters = [
@@ -132,7 +120,7 @@ def side_box_blur(clip: vs.VideoNode, radius: int = 1, planes: Planes = None) ->
     )
 
 
-class GaussBlur(Generic[P, R]):
+class GaussBlur[**P, R]:
     """
     Class decorator that wraps the [gauss_blur][vsrgtools.blur.gauss_blur] function
     and extends its functionality.
@@ -168,7 +156,7 @@ class GaussBlur(Generic[P, R]):
         mode: OneDimConvMode | TempConvMode = ConvMode.HV,
         planes: Planes = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Applies Gaussian blur to a clip from an intuitive radius.
 
@@ -207,7 +195,7 @@ def gauss_blur(
     mode: OneDimConvMode | TempConvMode = ConvMode.HV,
     planes: Planes = None,
     **kwargs: Any,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Applies Gaussian blur to a clip, supporting spatial and temporal modes, and per-plane control.
 
@@ -226,8 +214,6 @@ def gauss_blur(
     Returns:
         Blurred clip.
     """
-    assert check_variable(clip, gauss_blur)
-
     planes = normalize_planes(clip, planes)
 
     if not TYPE_CHECKING and mode == ConvMode.SQUARE:
@@ -245,7 +231,7 @@ def gauss_blur(
 
     if not mode.is_temporal:
 
-        def _resize2_blur(plane: ConstantFormatVideoNode, sigma: float, taps: int) -> ConstantFormatVideoNode:
+        def _resize2_blur(plane: vs.VideoNode, sigma: float, taps: int) -> vs.VideoNode:
             resize_kwargs = dict[str, Any]()
 
             # Downscale approximation can be used by specifying _fast=True
@@ -266,7 +252,7 @@ def gauss_blur(
             else:
                 resize_kwargs.update({f"force_{k}": k in mode for k in "hv"})
 
-            return Gaussian(sigma, taps).scale(plane, **resize_kwargs | kwargs)  # type: ignore[return-value]
+            return Gaussian(sigma, taps).scale(plane, **resize_kwargs | kwargs)
 
         if not {*range(clip.format.num_planes)} - {*planes}:
             return _resize2_blur(clip, sigma, radius)
@@ -284,7 +270,7 @@ def min_blur(
     mode: tuple[ConvMode, ConvMode] = (ConvMode.HV, ConvMode.SQUARE),
     planes: Planes = None,
     **kwargs: Any,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Combines binomial (Gaussian-like) blur and median filtering for a balanced smoothing effect.
 
@@ -292,7 +278,7 @@ def min_blur(
     a "best of both worlds" result — combining the edge-preserving nature of median filtering
     with the smoothness of Gaussian blur. The effect is somewhat reminiscent of a bilateral filter.
 
-    Original concept: http://avisynth.nl/index.php/MinBlur
+    Original concept: <http://avisynth.nl/index.php/MinBlur>
 
     Args:
         clip: Source clip.
@@ -307,8 +293,6 @@ def min_blur(
     Returns:
         Clip with MinBlur applied.
     """
-    assert check_variable(clip, min_blur)
-
     planes = normalize_planes(clip, planes)
 
     if isinstance(radius, Sequence):
@@ -325,7 +309,7 @@ def min_blur(
 _SbrBlurT = Union[
     BlurMatrix,
     Sequence[float],
-    VSFunctionNoArgs[vs.VideoNode, vs.VideoNode],
+    VSFunctionNoArgs,
 ]
 
 
@@ -339,10 +323,11 @@ def sbr(
     *,
     func: FuncExcept | None = None,
     **kwargs: Any,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     A helper function for high-pass filtering a blur difference, inspired by an AviSynth script by Didée.
-    `https://forum.doom9.org/showthread.php?p=1584186#post1584186`
+
+    <https://forum.doom9.org/showthread.php?p=1584186#post1584186>
 
     Args:
         clip: Source clip.
@@ -362,7 +347,7 @@ def sbr(
     if isinstance(radius, Sequence):
         return normalize_radius(clip, min_blur, list(radius), planes)
 
-    def _apply_blur(clip: ConstantFormatVideoNode, blur: _SbrBlurT | vs.VideoNode) -> ConstantFormatVideoNode:
+    def _apply_blur(clip: vs.VideoNode, blur: _SbrBlurT | vs.VideoNode) -> vs.VideoNode:
         if isinstance(blur, Sequence):
             return BlurMatrix.custom(blur, mode)(clip, planes, **kwargs)
 
@@ -371,12 +356,8 @@ def sbr(
 
         blurred = blur(clip) if callable(blur) else blur
 
-        assert check_variable_format(blurred, func)
-
         return blurred
 
-    assert check_variable(clip, func)
-
     planes = normalize_planes(clip, planes)
 
     blurred = _apply_blur(clip, blur)
@@ -386,7 +367,7 @@ def sbr(
 
     return norm_expr(
         [clip, diff, blurred_diff],
-        "y neutral - D1! y z - D2! D1@ D2@ xor x x D1@ abs D2@ abs < D1@ D2@ ? - ?",
+        "y z - D1! y neutral - D2! D1@ D2@ xor x x D1@ abs D2@ abs < D1@ D2@ ? - ?",
         planes=planes,
         func=func,
     )
@@ -400,7 +381,7 @@ def median_blur(
     planes: Planes = None,
     *,
     func: FuncExcept | None = None,
-) -> ConstantFormatVideoNode: ...
+) -> vs.VideoNode: ...
 
 
 @overload
@@ -414,7 +395,7 @@ def median_blur(
     scalep: bool = True,
     *,
     func: FuncExcept | None = None,
-) -> ConstantFormatVideoNode: ...
+) -> vs.VideoNode: ...
 
 
 @overload
@@ -426,7 +407,7 @@ def median_blur(
     *,
     scenechange: bool = False,
     func: FuncExcept | None = None,
-) -> ConstantFormatVideoNode: ...
+) -> vs.VideoNode: ...
 
 
 @overload
@@ -440,7 +421,7 @@ def median_blur(
     scalep: bool = True,
     scenechange: bool = False,
     func: FuncExcept | None = None,
-) -> ConstantFormatVideoNode: ...
+) -> vs.VideoNode: ...
 
 
 def median_blur(
@@ -453,7 +434,7 @@ def median_blur(
     scalep: bool = True,
     scenechange: bool = False,
     func: FuncExcept | None = None,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Applies a median blur to the clip using spatial or temporal neighborhood.
 
@@ -484,46 +465,11 @@ def median_blur(
     """
     func = func or median_blur
 
-    assert check_variable(clip, func)
-
     if mode == ConvMode.TEMPORAL:
         if not isinstance(radius, int):
             raise CustomValueError("A list of radius isn't supported for ConvMode.TEMPORAL!", func, radius)
 
-        tmedian = core.zsmooth.TemporalMedian(clip, radius, planes)
-
-        if not scenechange:
-            return tmedian
-
-        clips = shift_clip_multi(clip, (-radius, radius))
-        r = radius
-
-        def median_scenechange(
-            n: int, f: list[vs.VideoFrame], tmedian: vs.VideoNode, clips: Sequence[vs.VideoNode]
-        ) -> vs.VideoNode:
-            backwards_prop = [get_prop(frame, "_SceneChangeNext", int) for frame in f[:r]]
-            forward_prop = [get_prop(frame, "_SceneChangePrev", int) for frame in f[r + 1 :]]
-
-            if not any([*backwards_prop, *forward_prop]):
-                return tmedian
-
-            clips_back = clips[:r]
-
-            for i, prop in enumerate(reversed(backwards_prop)):
-                if prop:
-                    clips_back = clips_back[r - i :]
-                    break
-
-            clips_forw = clips[r + 1 :]
-
-            for i, prop in enumerate(forward_prop):
-                if prop:
-                    clips_forw = clips_forw[: i - r]
-                    break
-
-            return MeanMode.MEDIAN(clips[r], clips_back, clips_forw, planes=planes)
-
-        return core.std.FrameEval(clip, lambda n, f: median_scenechange(n, f, tmedian, clips), clips, clips)
+        return core.zsmooth.TemporalMedian(clip, radius, planes, scenechange)
 
     radius = normalize_seq(radius, clip.format.num_planes)
 
@@ -542,7 +488,7 @@ def median_blur(
     return MeanMode.MEDIAN.single(clip, radius, mode, planes=planes, func=func)
 
 
-class Bilateral(Generic[P, R]):
+class Bilateral[**P, R]:
     """
     Class decorator that wraps the [bilateral][vsrgtools.blur.bilateral] function
     and extends its functionality.
@@ -576,7 +522,7 @@ class Bilateral(Generic[P, R]):
         Uses `bilateralgpu_rtc.Bilateral` — a CUDA-based GPU implementation with runtime shader compilation.
         """
 
-        def Bilateral(self, clip: vs.VideoNode, *args: Any, **kwargs: Any) -> ConstantFormatVideoNode:  # noqa: N802
+        def Bilateral(self, clip: vs.VideoNode, *args: Any, **kwargs: Any) -> vs.VideoNode:  # noqa: N802
             """
             Applies the bilateral filter using the plugin associated with the selected backend.
 
@@ -599,7 +545,7 @@ def bilateral(
     sigmaR: float | Sequence[float] | None = None,  # noqa: N803
     backend: Bilateral.Backend = Bilateral.Backend.CPU,
     **kwargs: Any,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Applies a bilateral filter for edge-preserving and noise-reducing smoothing.
 
@@ -629,12 +575,10 @@ def bilateral(
     Returns:
         Bilaterally filtered clip.
     """
-    assert check_variable_format(clip, bilateral)
-
     if backend == Bilateral.Backend.CPU:
-        bilateral_args = KwargsT(ref=ref, sigmaS=sigmaS, sigmaR=sigmaR, planes=normalize_planes(clip))
+        bilateral_args = {"ref": ref, "sigmaS": sigmaS, "sigmaR": sigmaR, "planes": normalize_planes(clip)}
     else:
-        bilateral_args = KwargsT(ref=ref, sigma_spatial=sigmaS, sigma_color=sigmaR)
+        bilateral_args = {"ref": ref, "sigma_spatial": sigmaS, "sigma_color": sigmaR}
 
     return backend.Bilateral(clip, **bilateral_args | kwargs)
 
@@ -645,7 +589,7 @@ def flux_smooth(
     spatial_threshold: float | Sequence[float] | None = None,
     planes: Planes = None,
     scalep: bool = True,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     FluxSmoothT examines each pixel and compares it to the corresponding pixel in the previous and next frames.
     Smoothing occurs if both the previous frame's value and the next frame's value are greater,
@@ -662,7 +606,7 @@ def flux_smooth(
     Args:
         clip: Clip to process.
         temporal_threshold: Temporal neighbour pixels within this threshold from the current pixel are included in the
-            average. Can be specified as an array, with values corresonding to each plane of the input clip. A negative
+            average. Can be specified as an array, with values corresponding to each plane of the input clip. A negative
             value (such as -1) indicates that the plane should not be processed and will be copied from the input clip.
         spatial_threshold: Spatial neighbour pixels within this threshold from the current pixel are included in the
             average. A negative value (such as -1) indicates that the plane should not be processed and will be copied
@@ -674,16 +618,13 @@ def flux_smooth(
     Returns:
         Smoothed clip.
     """
-
-    assert check_variable_format(clip, flux_smooth)
-
     if spatial_threshold:
         return core.zsmooth.FluxSmoothST(clip, temporal_threshold, spatial_threshold, planes, scalep)
 
     return core.zsmooth.FluxSmoothT(clip, temporal_threshold, planes, scalep)
 
 
-class GuidedFilter(Generic[P, R]):
+class GuidedFilter[**P, R]:
     """
     Class decorator that wraps the [guided_filter][vsrgtools.blur.guided_filter] function
     and extends its functionality.
@@ -730,8 +671,6 @@ def guided_filter(
     downscaler: ScalerLike = Point,
     upscaler: ScalerLike = Bilinear,
 ) -> vs.VideoNode:
-    assert check_variable(clip, guided_filter)
-
     planes = normalize_planes(clip, planes)
 
     downscaler = Scaler.ensure_obj(downscaler, guided_filter)
