@@ -1,18 +1,18 @@
 from __future__ import annotations
 
-from typing import Callable, Generic, Sequence
+from typing import Callable, Sequence
 
-from jetpytools import CustomIntEnum, CustomStrEnum, P, R
+from jetpytools import CustomIntEnum, CustomStrEnum, KwargsNotNone
 
 from vsexprtools import norm_expr
-from vstools import ConstantFormatVideoNode, KwargsNotNone, Planes, check_variable, normalize_param_planes, vs
+from vstools import Planes, normalize_param_planes, vs
 
 from .aka_expr import removegrain_aka_exprs, repair_aka_exprs
 
 __all__ = ["clense", "remove_grain", "removegrain", "repair", "vertical_cleaner"]
 
 
-class Repair(Generic[P, R]):
+class Repair[**P, R]:
     """
     Class decorator that wraps the [repair][vsrgtools.rgtools.repair] function
     and extends its functionality.
@@ -178,9 +178,7 @@ class Repair(Generic[P, R]):
         Use RemoveGrain mode 28's result to constrain the pixel.
         """
 
-        def __call__(
-            self, clip: vs.VideoNode, repairclip: vs.VideoNode, planes: Planes = None
-        ) -> ConstantFormatVideoNode:
+        def __call__(self, clip: vs.VideoNode, repairclip: vs.VideoNode, planes: Planes = None) -> vs.VideoNode:
             """
             Apply the selected repair mode to a `clip` using a `repairclip`.
 
@@ -201,7 +199,7 @@ def repair(
     repairclip: vs.VideoNode,
     mode: int | Repair.Mode | Sequence[int | Repair.Mode],
     planes: Planes = None,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Constrains the input `clip` using a `repairclip` by clamping pixel values
     based on a chosen [mode][vsrgtools.rgtools.Repair.Mode].
@@ -234,9 +232,6 @@ def repair(
     Returns:
         Clip with repaired pixels, bounded by the reference.
     """
-    assert check_variable(clip, repair)
-    assert check_variable(repairclip, repair)
-
     mode = normalize_param_planes(clip, mode, planes, Repair.Mode.NONE)
 
     if not sum(mode):
@@ -245,10 +240,10 @@ def repair(
     if all(m in range(24 + 1) for m in mode):
         return clip.zsmooth.Repair(repairclip, mode)
 
-    return norm_expr([clip, repairclip], tuple([repair_aka_exprs[m]() for m in mode]), func=repair)
+    return norm_expr([clip, repairclip], tuple(repair_aka_exprs[m]() for m in mode), func=repair)
 
 
-class RemoveGrain(Generic[P, R]):
+class RemoveGrain[**P, R]:
     """
     Class decorator that wraps the [remove_grain][vsrgtools.rgtools.remove_grain] function
     and extends its functionality.
@@ -294,7 +289,7 @@ class RemoveGrain(Generic[P, R]):
 
         MINMAX_MEDIAN = 4
         """
-        Deprecated. Similar to mode 1, but clamps to fourth-lowest/highest.
+        Similar to mode 1, but clamps to fourth-lowest/highest.
         """
 
         EDGE_CLIP_STRONG = 5
@@ -329,7 +324,7 @@ class RemoveGrain(Generic[P, R]):
 
         BINOMIAL_BLUR = 11
         """
-        Deprecated. Use `BlurMatrix.BINOMIAL`. Applies weighted 3x3 blur.
+        Applies a 3x3 binomial blur.
         """
 
         BOB_TOP_CLOSE = 13
@@ -364,12 +359,12 @@ class RemoveGrain(Generic[P, R]):
 
         MEAN_NO_CENTER = 19
         """
-        Deprecated. Use `BlurMatrix.MEAN_NO_CENTER`. Mean of neighborhood (excluding center).
+        Mean of neighborhood (excluding center).
         """
 
         MEAN = 20
         """
-        Deprecated. Use `BlurMatrix.MEAN`. Arithmetic mean of 3x3 neighborhood.
+        Arithmetic mean of 3x3 neighborhood.
         """
 
         BOX_BLUR_NO_CENTER = MEAN_NO_CENTER
@@ -422,7 +417,7 @@ class RemoveGrain(Generic[P, R]):
         Variant of 27 with different pairs. Usually visually similar.
         """
 
-        def __call__(self, clip: vs.VideoNode, planes: Planes = None) -> ConstantFormatVideoNode:
+        def __call__(self, clip: vs.VideoNode, planes: Planes = None) -> vs.VideoNode:
             """
             Apply the selected remove grain mode to a `clip`.
 
@@ -439,7 +434,7 @@ class RemoveGrain(Generic[P, R]):
 @RemoveGrain
 def remove_grain(
     clip: vs.VideoNode, mode: int | RemoveGrain.Mode | Sequence[int | RemoveGrain.Mode], planes: Planes = None
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Apply spatial denoising using the RemoveGrain algorithm.
 
@@ -468,8 +463,6 @@ def remove_grain(
     Returns:
         Processed (denoised) clip.
     """
-    assert check_variable(clip, remove_grain)
-
     mode = normalize_param_planes(clip, mode, planes, RemoveGrain.Mode.NONE)
 
     if not sum(mode):
@@ -478,10 +471,10 @@ def remove_grain(
     if all(m in range(24 + 1) for m in mode):
         return clip.zsmooth.RemoveGrain(mode)
 
-    return norm_expr(clip, tuple([removegrain_aka_exprs[m]() for m in mode]), func=remove_grain)
+    return norm_expr(clip, tuple(removegrain_aka_exprs[m]() for m in mode), func=remove_grain)
 
 
-class Clense(Generic[P, R]):
+class Clense[**P, R]:
     """
     Class decorator that wraps the [clense][vsrgtools.rgtools.clense] function
     and extends its functionality.
@@ -532,7 +525,7 @@ class Clense(Generic[P, R]):
             previous_clip: vs.VideoNode | None = None,
             next_clip: vs.VideoNode | None = None,
             planes: Planes = None,
-        ) -> ConstantFormatVideoNode:
+        ) -> vs.VideoNode:
             """
             Apply the selected clense mode to a clip using
             the [zsmooth](https://github.com/adworacz/zsmooth?tab=readme-ov-file#clense--forwardclense--backwardclense)
@@ -557,7 +550,7 @@ def clense(
     next_clip: vs.VideoNode | None = None,
     mode: Clense.Mode | str = Clense.Mode.NONE,
     planes: Planes = None,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Apply a clense (temporal median) filter based on the specified mode.
 
@@ -581,8 +574,6 @@ def clense(
     Returns:
         Clensed clip with temporal median filtering.
     """
-    assert check_variable(clip, clense)
-
     kwargs = KwargsNotNone(previous=previous_clip, next=next_clip)
 
     if mode == Clense.Mode.NONE:
@@ -591,7 +582,7 @@ def clense(
     return getattr(clip.zsmooth, mode)(planes=planes, **kwargs)
 
 
-class VerticalCleaner(Generic[P, R]):
+class VerticalCleaner[**P, R]:
     """
     Class decorator that wraps the [vertical_cleaner][vsrgtools.rgtools.vertical_cleaner] function
     and extends its functionality.
@@ -626,7 +617,7 @@ class VerticalCleaner(Generic[P, R]):
         Applies a detail-preserving vertical median filter (less aggressive).
         """
 
-        def __call__(self, clip: vs.VideoNode, planes: Planes = None) -> ConstantFormatVideoNode:
+        def __call__(self, clip: vs.VideoNode, planes: Planes = None) -> vs.VideoNode:
             """
             Applies the vertical cleaning mode to the given clip.
 
@@ -645,7 +636,7 @@ def vertical_cleaner(
     clip: vs.VideoNode,
     mode: int | VerticalCleaner.Mode | Sequence[int | VerticalCleaner.Mode],
     planes: Planes = None,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Applies a fast vertical median or relaxed median filter to the clip
     using the [zsmooth](https://github.com/adworacz/zsmooth?tab=readme-ov-file#verticalcleaner) plugin.
@@ -673,8 +664,6 @@ def vertical_cleaner(
     Returns:
         Filtered clip.
     """
-    assert check_variable(clip, vertical_cleaner)
-
     mode = normalize_param_planes(clip, mode, planes, VerticalCleaner.Mode.NONE)
 
     if not sum(mode):
