@@ -2,31 +2,21 @@ from __future__ import annotations
 
 from typing import Any, Sequence
 
-from vstools import (
-    ConstantFormatVideoNode,
-    GenericVSFunction,
-    Planes,
-    check_variable_format,
-    join,
-    normalize_planes,
-    normalize_seq,
-    split,
-    vs,
-)
+from jetpytools import normalize_seq
+
+from vstools import GenericVSFunction, Planes, join, normalize_planes, split, vs
 
 __all__ = ["normalize_radius"]
 
 
 def normalize_radius(
     __clip: vs.VideoNode,
-    __function: GenericVSFunction[ConstantFormatVideoNode],
+    __function: GenericVSFunction,
     __radius: Sequence[float | int] | dict[str, Sequence[float | int]],
     /,
     planes: Planes,
     **kwargs: Any,
-) -> ConstantFormatVideoNode:
-    assert check_variable_format(__clip, normalize_radius)
-
+) -> vs.VideoNode:
     if isinstance(__radius, dict):
         name, radius = __radius.popitem()
     else:
