@@ -4,14 +4,12 @@ from dataclasses import dataclass
 from functools import partial
 from math import ceil, floor
 from types import NoneType
-from typing import Any, Callable, NamedTuple, TypeAlias, overload
+from typing import Any, Callable, NamedTuple, Self, overload
 
-from jetpytools import CustomTypeError, FuncExcept, mod_x
-from typing_extensions import Self, Unpack
+from jetpytools import CustomTypeError, FuncExcept, mod2, mod_x
 
-from vskernels import Lanczos, MixedScalerProcess, Scaler, ScalerLike, is_scaler_like
-from vskernels.util import _BaseScalerTs, _ScalerT
-from vstools import FunctionUtil, KwargsT, Planes, Resolution, VSFunctionNoArgs, get_w, mod2, vs
+from vskernels import MixedScalerProcess, Scaler, ScalerLike, is_scaler_like
+from vstools import FunctionUtil, Planes, Resolution, VSFunctionNoArgs, get_w, vs
 
 from .various import ComplexSuperSamplerProcess
 
@@ -41,14 +39,14 @@ def scale_var_clip(
     Returns:
         Scaled clip.
     """
-    _cached_clips = dict[str, vs.VideoNode]()
+    cached_clips = dict[str, vs.VideoNode]()
 
     no_accepts_var = list[Scaler]()
 
     def _eval_scale(f: vs.VideoFrame, n: int) -> vs.VideoNode:
         key = f"{f.width}_{f.height}"
 
-        if key not in _cached_clips:
+        if key not in cached_clips:
             res = Resolution(f.width, f.height)
 
             norm_scaler = scaler(res) if callable(scaler) else scaler
@@ -78,19 +76,19 @@ def scale_var_clip(
             if debug:
                 scaled = scaled.std.SetFrameProps(var_width=res.width, var_height=res.height)
 
-            _cached_clips[key] = scaled
+            cached_clips[key] = scaled
 
-        return _cached_clips[key]
+        return cached_clips[key]
 
     out_clip = clip if callable(width) or callable(height) else clip.std.BlankClip(width, height)
 
     return out_clip.std.FrameEval(_eval_scale, clip, clip)
 
 
-LeftCrop: TypeAlias = int
-RightCrop: TypeAlias = int
-TopCrop: TypeAlias = int
-BottomCrop: TypeAlias = int
+type LeftCrop = int
+type RightCrop = int
+type TopCrop = int
+type BottomCrop = int
 
 
 class CropRel(NamedTuple):
@@ -133,7 +131,7 @@ class ScalingArgs:
 
         return ((clip.height / self.height) if do_h else 1.0, (clip.width / self.width) if do_w else 1.0)
 
-    def kwargs(self, clip_or_rate: vs.VideoNode | float | None = None, /) -> KwargsT:
+    def kwargs(self, clip_or_rate: vs.VideoNode | float | None = None, /) -> dict[str, Any]:
         kwargs = dict[str, Any]()
 
         do_h, do_w = self._do()
@@ -183,8 +181,8 @@ class ScalingArgs:
     def from_args(
         cls,
         base_clip: vs.VideoNode,
-        height: int | float,
-        width: int | float | None = None,
+        height: int | float,  # noqa: PYI041
+        width: int | float | None = None,  # noqa: PYI041
         base_height: int | None = None,
         base_width: int | None = None,
         src_top: float = 0,
@@ -197,7 +195,7 @@ class ScalingArgs:
 
         Args:
             base_clip: Source clip.
-            height:  Target (de)scaling height. Casting to float will ensure fractional calculations.
+            height: Target (de)scaling height. Casting to float will ensure fractional calculations.
             width: Target (de)scaling width. Casting to float will ensure fractional calculations. If None, it will be
                 calculated from the height and the aspect ratio of the base_clip.
             base_height: The height from which to contain the clip. If None, it will be calculated from the height.
@@ -209,7 +207,7 @@ class ScalingArgs:
 
                    - "w" means only the width is calculated.
                    - "h" means only the height is calculated.
-                   - "hw or "wh" mean both width and height are calculated.
+                   - "hw" or "wh" mean both width and height are calculated.
 
         Returns:
             ScalingArgs object suitable for scaling functions.
@@ -284,9 +282,9 @@ class ScalingArgs:
 @overload
 def pre_ss(
     clip: vs.VideoNode,
-    function: VSFunctionNoArgs[vs.VideoNode, vs.VideoNode],
+    function: VSFunctionNoArgs,
     rfactor: float = 2.0,
-    sp: type[MixedScalerProcess[_ScalerT, Unpack[_BaseScalerTs]]] = ComplexSuperSamplerProcess[Lanczos],  # type: ignore[assignment]
+    sp: type[MixedScalerProcess[Any, Any]] = ComplexSuperSamplerProcess,
     *,
     mod: int = 4,
     planes: Planes = None,
@@ -299,7 +297,7 @@ def pre_ss(
     clip: vs.VideoNode,
     *,
     rfactor: float = 2.0,
-    sp: MixedScalerProcess[_ScalerT, Unpack[_BaseScalerTs]],
+    sp: MixedScalerProcess[Any, Any],
     mod: int = 4,
     planes: Planes = None,
     func: FuncExcept | None = None,
@@ -309,7 +307,7 @@ def pre_ss(
 @overload
 def pre_ss(
     clip: vs.VideoNode,
-    function: VSFunctionNoArgs[vs.VideoNode, vs.VideoNode],
+    function: VSFunctionNoArgs,
     rfactor: float = 2.0,
     *,
     supersampler: ScalerLike | Callable[[vs.VideoNode, int, int], vs.VideoNode],
@@ -320,12 +318,11 @@ def pre_ss(
 ) -> vs.VideoNode: ...
 
 
-def pre_ss(  # pyright: ignore[reportInconsistentOverload]
+def pre_ss(
     clip: vs.VideoNode,
-    function: VSFunctionNoArgs[vs.VideoNode, vs.VideoNode] | None = None,
+    function: VSFunctionNoArgs | None = None,
     rfactor: float = 2.0,
-    sp: type[MixedScalerProcess[_ScalerT, Unpack[_BaseScalerTs]]]
-    | MixedScalerProcess[_ScalerT, Unpack[_BaseScalerTs]] = ComplexSuperSamplerProcess[Lanczos],  # type: ignore[assignment]
+    sp: type[MixedScalerProcess[Any, Any]] | MixedScalerProcess[Any, Any] = ComplexSuperSamplerProcess,
     supersampler: ScalerLike | Callable[[vs.VideoNode, int, int], vs.VideoNode] | None = None,
     downscaler: ScalerLike | Callable[[vs.VideoNode, int, int], vs.VideoNode] | None = None,
     mod: int = 4,
@@ -333,33 +330,32 @@ def pre_ss(  # pyright: ignore[reportInconsistentOverload]
     func: FuncExcept | None = None,
 ) -> vs.VideoNode:
     """
-    Supersamples the input clip, applies a given function to the higher-resolution version,
-    and then downscales it back to the original resolution.
+    Supersamples the input clip, applies a given function to the higher-resolution version, and then downscales it back to the original resolution.
 
     This function generalizes the behavior of
     [SuperSamplerProcess][vsaa.deinterlacers.SuperSamplerProcess] and
     [ComplexSuperSamplerProcess][vsscale.various.ComplexSuperSamplerProcess].
 
     - Examples:
-        ```
+        ```py
         out = pre_ss(clip, lambda clip: cool_function(clip, ...), planes=0)
         ```
 
         - Passing NNEDI3 as a supersampler:
-        ```
+        ```py
         from vsaa import SuperSamplerProcess
 
         out = pre_ss(clip, lambda clip: cool_function(clip, ...), SuperSamplerProcess)
         ```
         - This works too:
-        ```
+        ```py
         from vsaa import SuperSamplerProcess
 
         out = pre_ss(clip, sp=SuperSamplerProcess(function=lambda clip: cool_function(clip, ...)))
         ```
 
         - Specifying `supersampler` and `downscaler`:
-        ```
+        ```py
         from vskernels import Point
 
         out = pre_ss(clip, lambda clip: cool_function(clip, ...), supersampler=Point, downscaler=Point, planes=0)
@@ -369,9 +365,9 @@ def pre_ss(  # pyright: ignore[reportInconsistentOverload]
         clip: Source clip.
         function: A function to apply on the supersampled clip.
         rfactor: Scaling factor for supersampling. Defaults to 2.
-        sp: A `MixedScalerProcess` instance or class.
-            Default is `ComplexSuperSamplerProcess[Lanczos]`.
-            It upscales with Lanczos and downscales with Point.
+        sp: A [MixedScalerProcess][vskernels.MixedScalerProcess] instance or class.
+            Default is [ComplexSuperSamplerProcess[Lanczos]][vsscale.ComplexSuperSamplerProcess].
+            It upscales with [Lanczos][vskernels.Lanczos] and downscales with [Point][vskernels.Point].
         supersampler: Scaler used to upscale the input clip if `sp` is not specified.
         downscaler: Downscaler used for undoing the upscaling done by the supersampler if `sp` is not specified.
         mod: Ensures the supersampled resolution is a multiple of this value. Defaults to 4.
@@ -380,7 +376,7 @@ def pre_ss(  # pyright: ignore[reportInconsistentOverload]
 
     Returns:
         A clip with the given function applied at higher resolution, then downscaled back.
-    """
+    """  # noqa: E501
     func_util = FunctionUtil(clip, func or pre_ss, planes)
 
     args = (
