@@ -6,40 +6,27 @@ from __future__ import annotations
 
 import re
 from abc import ABC
+from dataclasses import MISSING as DATACLASSES_MISSING
 from dataclasses import Field, asdict, fields, replace
 from functools import cache
-from logging import DEBUG, debug, getLogger, warning
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    ClassVar,
-    Literal,
-    Protocol,
-    SupportsFloat,
-    TypeAlias,
-    TypeVar,
-    get_args,
-    runtime_checkable,
-)
+from logging import DEBUG, getLogger
+from typing import TYPE_CHECKING, Any, ClassVar, Literal, Protocol, Self, SupportsFloat, get_args, runtime_checkable
 
-from jetpytools import CustomImportError
-from packaging.version import Version
-from typing_extensions import Self, deprecated
+from jetpytools import CustomImportError, CustomValueError, SPath, SPathLike
+
+if TYPE_CHECKING:
+    from packaging.version import Version
 
 from vsexprtools import norm_expr
+from vsjetpack import deprecated
 from vskernels import Bilinear, Catrom, Kernel, KernelLike, ScalerLike
 from vstools import (
     ColorRange,
-    ConstantFormatVideoNode,
-    CustomValueError,
     DitherType,
     Matrix,
     MatrixLike,
     OutdatedPluginError,
     ProcessVariableResClip,
-    SPath,
-    SPathLike,
-    check_variable_format,
     check_variable_resolution,
     core,
     depth,
@@ -66,7 +53,7 @@ __all__ = [
 ]
 
 
-_IntT = TypeVar("_IntT", bound=int)
+_log = getLogger(__name__)
 
 
 @runtime_checkable
@@ -91,6 +78,7 @@ if TYPE_CHECKING:
     Type alias for anything that can resolve to a Backend from vs-mlrt.
 
     This includes:
+
     - A string identifier.
     - A class type subclassing `Backend`.
     - An instance of a `Backend`.
@@ -100,6 +88,8 @@ else:
 
 
 def _normalize_git_version(raw: str) -> Version:
+    from packaging.version import Version
+
     raw = raw.strip().lstrip("v")
 
     matched = re.match(r"(?P<tag>[0-9A-Za-z.\-_]+?)(?:-(?P<count>\d+)-g(?P<hash>[0-9a-f]+))?$", raw)
@@ -141,7 +131,9 @@ def _check_vsmlrt_script_version(cls: type[BaseOnnxScaler]) -> None:
     except ImportError:
         raise CustomImportError(cls, "vsmlrt") from None
 
-    if (current_version := Version(vsmlrt.__version__)) < cls._REQUIRED_VSMLRT_SCRIPT_VERSION:
+    from packaging.version import Version
+
+    if (current_version := Version(vsmlrt.__version__)) < Version(cls._REQUIRED_VSMLRT_SCRIPT_VERSION):
         raise CustomImportError(
             cls,
             "vsmlrt",
@@ -157,7 +149,9 @@ def _check_vsmlrt_plugin_version(backend_name: str, cls: type[BaseOnnxScaler]) -
 
     current_version = _normalize_git_version(getattr(core, plugin_name).Version()["version"].decode())
 
-    if current_version < cls._REQUIRED_VSMLRT_PLUGIN_VERSION:
+    from packaging.version import Version
+
+    if current_version < Version(cls._REQUIRED_VSMLRT_PLUGIN_VERSION):
         raise OutdatedPluginError(
             cls,
             plugin_name,
@@ -218,8 +212,8 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
     Abstract generic scaler class for an ONNX model.
     """
 
-    _REQUIRED_VSMLRT_SCRIPT_VERSION = Version("3.22.35")
-    _REQUIRED_VSMLRT_PLUGIN_VERSION = Version("15.13")
+    _REQUIRED_VSMLRT_SCRIPT_VERSION = "3.22.36"
+    _REQUIRED_VSMLRT_PLUGIN_VERSION = "15.14"
 
     if not TYPE_CHECKING:
 
@@ -302,18 +296,19 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
 
         self.max_instances = max_instances
 
-        if getLogger().level <= DEBUG:
-            debug(f"{self}: Using {self.backend.__class__.__name__} backend")
+        _log.debug("%s: Using '%s' backend", self, self.backend.__class__.__name__)
 
+        if _log.isEnabledFor(DEBUG):
             valid_fields = _get_backend_fields(self.backend)
 
             for k, v in asdict(self.backend).items():
-                debug(f"{self}: {k}={v}, default is {valid_fields[k].default}")
+                if v != (v_default := valid_fields[k].default) != DATACLASSES_MISSING:
+                    _log.debug(f"{self}: {k}={v!r}, default is {v_default!r}")
 
-            debug(f"{self}: User tiles: {self.tiles}")
-            debug(f"{self}: User tilesize: {self.tilesize}")
-            debug(f"{self}: User overlap: {(self.overlap_w, self.overlap_h)}")
-            debug(f"{self}: User multiple: {self.multiple}")
+        _log.debug("%s: User tiles: %s", self, self.tiles)
+        _log.debug("%s: User tilesize: %s", self, self.tilesize)
+        _log.debug("%s: User overlap: %s", self, (self.overlap_w, self.overlap_h))
+        _log.debug("%s: User multiple: %s", self, self.multiple)
 
     def scale(
         self,
@@ -322,7 +317,7 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
         height: int | None = None,
         shift: tuple[float, float] = (0, 0),
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Scale the given clip using the ONNX model.
 
@@ -340,8 +335,6 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
         """
         from vsmlrt import Backend
 
-        assert check_variable_format(clip, self.__class__)
-
         width, height = self._wh_norm(clip, width, height)
 
         preprocess_kwargs = dict[str, Any]()
@@ -356,37 +349,39 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
                     ckwargs[k.removeprefix(prefix)] = kwargs.pop(k)
                     break
 
-        debug(f"{self}: Preprocess kwargs: {preprocess_kwargs}")
-        debug(f"{self}: Postprocess kwargs: {postprocess_kwargs}")
-        debug(f"{self}: Inference kwargs: {inference_kwargs}")
+        _log.debug("%s: Preprocess kwargs: %s", self.scale, preprocess_kwargs)
+        _log.debug("%s: Postprocess kwargs: %s", self.scale, postprocess_kwargs)
+        _log.debug("%s: Inference kwargs: %s", self.scale, inference_kwargs)
 
         wclip = self.preprocess_clip(clip, **preprocess_kwargs)
 
         if 0 not in {clip.width, clip.height}:
             scaled = self.inference(wclip, **inference_kwargs)
         else:
-            debug(f"{self}: Variable resolution clip detected!")
+            _log.debug("%s: Variable resolution clip detected...", self.scale)
 
             if not isinstance(self.backend, (Backend.TRT, Backend.TRT_RTX)):
                 raise CustomValueError(
                     "Variable resolution clips can only be processed with TRT Backend!", self.__class__, self.backend
                 )
 
-            warning(f"{self.__class__.__name__}: Variable resolution clip detected!")
+            _log.warning("%s: Variable resolution clip detected!", self.scale)
 
             if self.backend.static_shape:
-                warning("static_shape is True, setting it to False...")
+                _log.warning("%s: static_shape is True, setting it to False...", self.scale)
                 self.backend.static_shape = False
 
             if not self.backend.max_shapes:
-                warning("max_shapes is None, setting it to (1936, 1088). You may want to adjust it...")
+                _log.warning(
+                    "%s: max_shapes is None, setting it to (1936, 1088). You may want to adjust it...", self.scale
+                )
                 self.backend.max_shapes = (1936, 1088)
 
             if not self.backend.opt_shapes:
-                warning("opt_shapes is None, setting it to (64, 64). You may want to adjust it...")
+                _log.warning("%s: opt_shapes is None, setting it to (64, 64). You may want to adjust it...", self.scale)
                 self.backend.opt_shapes = (64, 64)
 
-            scaled = ProcessVariableResClip[ConstantFormatVideoNode].from_func(
+            scaled = ProcessVariableResClip.from_func(
                 wclip, lambda c: self.inference(c, **inference_kwargs), False, wclip.format, self.max_instances
             )
 
@@ -413,23 +408,24 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
 
         return calc_tilesize(**kwargs)
 
-    def preprocess_clip(self, clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def preprocess_clip(self, clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         """
         Performs preprocessing on the clip prior to inference.
         """
-        debug(f"{self}.pre: Before pp; Clip format is {clip.format!r}")
+        _log.debug("%s: Before pp; Clip format is %r", self.preprocess_clip, clip.format)
 
         clip = depth(clip, self._pick_precision(16, 32), vs.FLOAT, **kwargs)
 
-        debug(f"{self}.pre: After pp; Clip format is {clip.format!r}")
+        _log.debug("%s: After pp; Clip format is %r", self.preprocess_clip, clip.format)
 
         return limiter(clip, func=self.__class__)
 
-    def postprocess_clip(self, clip: vs.VideoNode, input_clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def postprocess_clip(self, clip: vs.VideoNode, input_clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         """
         Handles postprocessing of the model's output after inference.
         """
-        debug(f"{self}.post: Before pp; Clip format is {clip.format!r}")
+        _log.debug("%s: Before pp; Clip format is %r", self.preprocess_clip, clip.format)
+        _log.debug("%s: Before pp; Clip format is %r", self.postprocess_clip, clip.format)
 
         clip = depth(
             clip,
@@ -438,11 +434,11 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
             **kwargs,
         )
 
-        debug(f"{self}.post: After pp; Clip format is {clip.format!r}")
+        _log.debug("%s: After pp; Clip format is %r", self.postprocess_clip, clip.format)
 
         return clip
 
-    def inference(self, clip: ConstantFormatVideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def inference(self, clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         """
         Runs inference on the given video clip using the configured model and backend.
         """
@@ -451,15 +447,15 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
 
         tilesize, overlaps = self.calc_tilesize(clip)
 
-        debug(f"{self}: Passing clip to inference: {clip.format!r}")
-        debug(f"{self}: Passing model: {self.model}")
-        debug(f"{self}: Passing tiles size: {tilesize}")
-        debug(f"{self}: Passing overlaps: {overlaps}")
-        debug(f"{self}: Passing extra kwargs: {kwargs}")
+        _log.debug("%s: Passing clip to inference: %r", self.inference, clip.format)
+        _log.debug("%s: Passing model: %s", self.inference, self.model)
+        _log.debug("%s: Passing tiles size: %s", self.inference, tilesize)
+        _log.debug("%s: Passing overlaps: %s", self.inference, overlaps)
+        _log.debug("%s: Passing extra kwargs: %s", self.inference, kwargs)
 
         return inference(clip, self.model, overlaps, tilesize, self.backend, **kwargs)
 
-    def _pick_precision(self, fp16: _IntT, fp32: _IntT) -> _IntT:
+    def _pick_precision[_IntT: int](self, fp16: _IntT, fp32: _IntT) -> _IntT:
         from vsmlrt import Backend
 
         precision = (
@@ -480,7 +476,11 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
             else fp32
         )
 
-        debug(f"{self}: Selecting precision: {get_video_format(precision) if precision > 32 else precision!r}")
+        _log.debug(
+            "%s: Selecting precision: %r",
+            self._pick_precision,
+            get_video_format(precision) if precision > 32 else precision,
+        )
 
         return precision
 
@@ -490,13 +490,11 @@ class BaseOnnxScalerRGB(BaseOnnxScaler):
     Abstract ONNX class for RGB models.
     """
 
-    def preprocess_clip(self, clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def preprocess_clip(self, clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         clip = self.kernel.resample(clip, self._pick_precision(vs.RGBH, vs.RGBS), Matrix.RGB, **kwargs)
         return limiter(clip, func=self.__class__)
 
-    def postprocess_clip(self, clip: vs.VideoNode, input_clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
-        assert check_variable_format(clip, self.__class__)
-
+    def postprocess_clip(self, clip: vs.VideoNode, input_clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         if get_video_format(clip) != get_video_format(input_clip):
             kwargs = (
                 dict[str, Any](
@@ -579,26 +577,26 @@ class BaseArtCNN(BaseOnnxScaler):
 
 
 class BaseArtCNNLuma(BaseArtCNN):
-    def preprocess_clip(self, clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def preprocess_clip(self, clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         return super().preprocess_clip(get_y(clip), **kwargs)
 
     def _finish_scale(
         self,
-        clip: ConstantFormatVideoNode,
-        input_clip: ConstantFormatVideoNode,
+        clip: vs.VideoNode,
+        input_clip: vs.VideoNode,
         width: int,
         height: int,
         shift: tuple[float, float] = (0, 0),
         matrix: MatrixLike | None = None,
         copy_props: bool = False,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         # Changes compared to BaseGenericScaler are:
         # - extract luma if input clip is luma only is removed  since this is a no op here
         # - Chroma planes are scaled accordingly with the artcnn'd luma,
         #   avoiding getting a luma plane when passing a YUV clip.
 
         if (clip.width, clip.height) != (width, height):
-            clip = self.scaler.scale(clip, width, height)  # type: ignore[assignment]
+            clip = self.scaler.scale(clip, width, height)
 
         if input_clip.format.color_family == vs.YUV:
             scaled_chroma = self.scaler.scale(input_clip, clip.width, clip.height)
@@ -617,8 +615,7 @@ class BaseArtCNNLuma(BaseArtCNN):
 
 
 class BaseArtCNNChroma(BaseArtCNN):
-    def preprocess_clip(self, clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
-        assert check_variable_format(clip, self.__class__)
+    def preprocess_clip(self, clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         assert clip.format.color_family == vs.YUV
 
         bits = self._pick_precision(16, 32)
@@ -627,50 +624,50 @@ class BaseArtCNNChroma(BaseArtCNN):
         if clip.format.subsampling_h != 0 or clip.format.subsampling_w != 0:
             chroma_scaler = Kernel.ensure_obj(kwargs.pop("chroma_scaler", Bilinear))
 
-            debug(f"{self}.pre: Before pp; Clip format is {clip.format!r}")
+            _log.debug("%s: Before pp; Clip format is %r", self.preprocess_clip, clip.format)
 
             clip = chroma_scaler.resample(clip, format, **kwargs)
 
-            debug(f"{self}.pre: Before pp; Clip format is {clip.format!r}")
+            _log.debug("%s: Before pp; Clip format is %r", self.preprocess_clip, clip.format)
 
             return norm_expr(clip, ("x 0 1 clamp", "x 0.5 + 0 1 clamp"), func=self.__class__)
 
         return norm_expr(clip, "x plane_min - plane_max plane_min - / 0 1 clamp", format=format, func=self.__class__)
 
-    def postprocess_clip(self, clip: vs.VideoNode, input_clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def postprocess_clip(self, clip: vs.VideoNode, input_clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         clip = norm_expr(clip, "x 0.5 -", [1, 2], func=self.__class__)
         return super().postprocess_clip(clip, input_clip, **kwargs)
 
-    def inference(self, clip: ConstantFormatVideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def inference(self, clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         from vsmlrt import flexible_inference
 
         tilesize, overlaps = self.calc_tilesize(clip)
 
-        debug(f"{self}: Passing clip to inference: {clip.format!r}")
-        debug(f"{self}: Passing model: {self.model}")
-        debug(f"{self}: Passing tiles size: {tilesize}")
-        debug(f"{self}: Passing overlaps: {overlaps}")
-        debug(f"{self}: Passing extra kwargs: {kwargs}")
+        _log.debug("%s: Passing clip to inference: %s", self.inference, clip.format)
+        _log.debug("%s: Passing model: %s", self.inference, self.model)
+        _log.debug("%s: Passing tiles size: %s", self.inference, tilesize)
+        _log.debug("%s: Passing overlaps: %s", self.inference, overlaps)
+        _log.debug("%s: Passing extra kwargs: %s", self.inference, kwargs)
 
         u, v = flexible_inference(clip, self.model, overlaps, tilesize, self.backend, **kwargs)
 
-        debug(f"{self}: Inferenced clip: {u.format!r}")
-        debug(f"{self}: Inferenced clip: {v.format!r}")
+        _log.debug("%s: Inferenced clip: %s", self.inference, u.format)
+        _log.debug("%s: Inferenced clip: %s", self.inference, v.format)
 
         return core.std.ShufflePlanes([clip, u, v], [0, 0, 0], vs.YUV, clip)
 
     def _finish_scale(
         self,
-        clip: ConstantFormatVideoNode,
-        input_clip: ConstantFormatVideoNode,
+        clip: vs.VideoNode,
+        input_clip: vs.VideoNode,
         width: int,
         height: int,
         shift: tuple[float, float] = (0, 0),
         matrix: MatrixLike | None = None,
         copy_props: bool = False,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         if (clip.width, clip.height) != (width, height):
-            clip = self.scaler.scale(clip, width, height)  # type: ignore[assignment]
+            clip = self.scaler.scale(clip, width, height)
 
         if shift != (0, 0):
             clip = self.shifter.shift(clip, shift)
@@ -692,7 +689,7 @@ class ArtCNN(BaseArtCNNLuma):
     You will have to grab the extended models pack or get it from the repo itself.
     (And create an "ArtCNN" folder in your models folder yourself)
 
-    https://github.com/Artoriuz/ArtCNN/releases/latest
+    <https://github.com/Artoriuz/ArtCNN/releases/latest>
 
     Defaults to R8F64.
 
@@ -1039,7 +1036,7 @@ class BaseWaifu2x(BaseOnnxScaler):
             **kwargs,
         )
 
-    def inference(self, clip: ConstantFormatVideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def inference(self, clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         from vsmlrt import Waifu2x as mlrt_Waifu2x
         from vsmlrt import Waifu2xModel
 
@@ -1069,7 +1066,7 @@ class _Waifu2xCunet(BaseWaifu2x, BaseOnnxScalerRGB):
             height: int | None = None,
             shift: tuple[float, float] = (0, 0),
             **kwargs: Any,
-        ) -> ConstantFormatVideoNode:
+        ) -> vs.VideoNode:
             """
             Scale the given clip using the ONNX model.
 
@@ -1097,31 +1094,26 @@ class _Waifu2xCunet(BaseWaifu2x, BaseOnnxScalerRGB):
             """
             ...
 
-    def inference(self, clip: ConstantFormatVideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def inference(self, clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         # Cunet model ruins image borders, so we need to pad it before upscale and crop it after.
         if kwargs.pop("no_pad", False):
             return super().inference(clip, **kwargs)
 
-        with padder.ctx(16, 4) as pad:
+        with padder.ctx(4, 4) as pad:
             padded = pad.MIRROR(clip)
             scaled = super().inference(padded, **kwargs)
             cropped = pad.CROP(scaled)
 
         return cropped
 
-    def postprocess_clip(self, clip: vs.VideoNode, input_clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def postprocess_clip(self, clip: vs.VideoNode, input_clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         # Cunet model also has a tint issue but it is not constant
         # It leaves flat areas alone but tints detailed areas.
         # Since most people will use Cunet to rescale details, the tint fix is enabled by default.
         if kwargs.pop("no_tint_fix", False):
             return super().postprocess_clip(clip, input_clip, **kwargs)
 
-        tint_fix = norm_expr(
-            clip,
-            "x 0.5 255 / + 0 1 clamp",
-            planes=0 if get_video_format(input_clip).color_family is vs.GRAY else None,
-            func="Waifu2x." + self.__class__.__name__,
-        )
+        tint_fix = norm_expr(clip, "x 0.5 255 / +", func="Waifu2x." + self.__class__.__name__)
         return super().postprocess_clip(tint_fix, input_clip, **kwargs)
 
 
@@ -1292,8 +1284,8 @@ class Waifu2x(_Waifu2xCunet):
         _model = 10
 
 
-_DPIRGrayModel: TypeAlias = int
-_DPIRColorModel: TypeAlias = int
+type _DPIRGrayModel = int
+type _DPIRColorModel = int
 
 
 class BaseDPIR(BaseOnnxScaler):
@@ -1360,12 +1352,12 @@ class BaseDPIR(BaseOnnxScaler):
         *,
         copy_props: bool = True,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         assert check_variable_resolution(clip, self.__class__)
 
         return super().scale(clip, width, height, shift, copy_props=copy_props, **kwargs)
 
-    def preprocess_clip(self, clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def preprocess_clip(self, clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         if get_color_family(clip) == vs.GRAY:
             return super().preprocess_clip(clip, **kwargs)
 
@@ -1373,9 +1365,7 @@ class BaseDPIR(BaseOnnxScaler):
 
         return limiter(clip, func=self.__class__)
 
-    def postprocess_clip(self, clip: vs.VideoNode, input_clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
-        assert check_variable_format(clip, self.__class__)
-
+    def postprocess_clip(self, clip: vs.VideoNode, input_clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         if get_video_format(clip) != get_video_format(input_clip):
             kwargs = (
                 dict[str, Any](
@@ -1390,7 +1380,7 @@ class BaseDPIR(BaseOnnxScaler):
 
         return clip
 
-    def inference(self, clip: ConstantFormatVideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def inference(self, clip: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
         from vsmlrt import DPIRModel, inference, models_path
 
         # Normalizing the strength clip
@@ -1401,7 +1391,7 @@ class BaseDPIR(BaseOnnxScaler):
         else:
             self.strength = clip.std.BlankClip(format=strength_fmt.id, color=float(self.strength) / 255, keep=True)
 
-        debug(f"{self}: Passing strength clip format: {self.strength.format!r}")
+        _log.debug("%s: Passing strength clip format: %s", self.inference, self.strength.format)
 
         # Get model name
         self.model = (
@@ -1411,10 +1401,10 @@ class BaseDPIR(BaseOnnxScaler):
         # Basic inference args
         tilesize, overlaps = self.calc_tilesize(clip)
 
-        debug(f"{self}: Passing model: {self.model}")
-        debug(f"{self}: Passing tiles size: {tilesize}")
-        debug(f"{self}: Passing overlaps: {overlaps}")
-        debug(f"{self}: Passing extra kwargs: {kwargs}")
+        _log.debug("%s: Passing model: %s", self.inference, self.model)
+        _log.debug("%s: Passing tiles size: %s", self.inference, tilesize)
+        _log.debug("%s: Passing overlaps: %s", self.inference, overlaps)
+        _log.debug("%s: Passing extra kwargs: %s", self.inference, kwargs)
 
         # Padding
         padding = padder.mod_padding(clip, self.multiple, 0)
@@ -1427,13 +1417,10 @@ class BaseDPIR(BaseOnnxScaler):
 
         return inference([clip, strength], self.model, overlaps, tilesize, self.backend, **kwargs).std.Crop(*padding)
 
-    def __vs_del__(self, core_id: int) -> None:
-        del self.strength
-
 
 class DPIR(BaseDPIR):
     """
-    Deep Plug-and-Play Image Restoration
+    Deep Plug-and-Play Image Restoration.
     """
 
     _model = (2, 3)
