@@ -1,25 +1,22 @@
 from __future__ import annotations
 
-from abc import ABC
 from functools import wraps
-from typing import Any, Callable, Iterable, TypeVar
+from typing import Any, Callable, Iterable
 
 from jetpytools import cachedproperty
 
 from vsexprtools import ExprToken, norm_expr
-from vskernels import Bilinear, BorderHandling, Hermite, Kernel, KernelLike, Scaler, ScalerLike
+from vskernels import Bilinear, BorderHandling, ComplexKernel, ComplexKernelLike, Hermite, Scaler, ScalerLike
 from vskernels.types import LeftShift, TopShift
 from vsmasktools import Kirsch, based_diff_mask, region_rel_mask, stabilize_mask
 from vstools import (
     ColorRange,
-    ConstantFormatVideoNode,
     DitherType,
     FieldBased,
     FieldBasedLike,
     FrameRangeN,
     FrameRangesN,
-    VideoNodeT,
-    check_variable,
+    VSObjectABC,
     core,
     depth,
     get_peak_value,
@@ -28,7 +25,6 @@ from vstools import (
     replace_ranges,
     split,
     vs,
-    vs_object,
 )
 
 from .helpers import BottomCrop, CropRel, LeftCrop, RightCrop, ScalingArgs, TopCrop
@@ -39,10 +35,8 @@ __all__ = [
     "RescaleBase",
 ]
 
-_RescaleT = TypeVar("_RescaleT", bound="RescaleBase")
 
-
-class RescaleBase(vs_object, ABC):
+class RescaleBase(VSObjectABC):
     """Base class for Rescale wrapper"""
 
     descale_args: ScalingArgs
@@ -54,35 +48,33 @@ class RescaleBase(vs_object, ABC):
         self,
         clip: vs.VideoNode,
         /,
-        kernel: KernelLike,
+        kernel: ComplexKernelLike,
         upscaler: ScalerLike = ArtCNN,
         downscaler: ScalerLike = Hermite(linear=True),
         field_based: FieldBasedLike | bool | None = None,
         border_handling: int | BorderHandling = BorderHandling.MIRROR,
         **kwargs: Any,
     ) -> None:
-        assert check_variable(clip, self.__class__)
-
         self._clipy, *chroma = split(clip)
         self._chroma = chroma
 
-        self._kernel = Kernel.ensure_obj(kernel)
+        self._kernel = ComplexKernel.ensure_obj(kernel)
         self._upscaler = Scaler.ensure_obj(upscaler)
 
         self._downscaler = Scaler.ensure_obj(downscaler)
 
-        self._field_based = FieldBased.from_param(field_based)
+        self._field_based = FieldBased.from_param_with_fallback(field_based)
 
-        self._border_handling = BorderHandling(int(border_handling))
+        self._border_handling = BorderHandling.from_param(border_handling)
 
         self.__add_props = kwargs.get("_add_props")
 
     @staticmethod
-    def _apply_field_based(
-        function: Callable[[_RescaleT, VideoNodeT], VideoNodeT],
-    ) -> Callable[[_RescaleT, VideoNodeT], VideoNodeT]:
+    def _apply_field_based[_RescaleT: RescaleBase](
+        function: Callable[[_RescaleT, vs.VideoNode], vs.VideoNode],
+    ) -> Callable[[_RescaleT, vs.VideoNode], vs.VideoNode]:
         @wraps(function)
-        def wrap(self: _RescaleT, clip: VideoNodeT) -> VideoNodeT:
+        def wrap(self: _RescaleT, clip: vs.VideoNode) -> vs.VideoNode:
             if self._field_based:
                 clip = self._field_based.apply(clip)
                 clip = function(self, clip)
@@ -93,11 +85,11 @@ class RescaleBase(vs_object, ABC):
         return wrap
 
     @staticmethod
-    def _add_props(
-        function: Callable[[_RescaleT, VideoNodeT], VideoNodeT],
-    ) -> Callable[[_RescaleT, VideoNodeT], VideoNodeT]:
+    def _add_props[_RescaleT: RescaleBase](
+        function: Callable[[_RescaleT, vs.VideoNode], vs.VideoNode],
+    ) -> Callable[[_RescaleT, vs.VideoNode], vs.VideoNode]:
         @wraps(function)
-        def wrap(self: _RescaleT, clip: VideoNodeT) -> VideoNodeT:
+        def wrap(self: _RescaleT, clip: vs.VideoNode) -> vs.VideoNode:
             if not self.__add_props:
                 return function(self, clip)
 
@@ -115,7 +107,7 @@ class RescaleBase(vs_object, ABC):
 
     @_add_props
     @_apply_field_based
-    def _generate_descale(self, clip: ConstantFormatVideoNode) -> ConstantFormatVideoNode:
+    def _generate_descale(self, clip: vs.VideoNode) -> vs.VideoNode:
         return self._kernel.descale(
             clip,
             self.descale_args.width,
@@ -126,8 +118,8 @@ class RescaleBase(vs_object, ABC):
 
     @_add_props
     @_apply_field_based
-    def _generate_rescale(self, clip: ConstantFormatVideoNode) -> ConstantFormatVideoNode:
-        return self._kernel.scale(  # type: ignore[return-value]
+    def _generate_rescale(self, clip: vs.VideoNode) -> vs.VideoNode:
+        return self._kernel.scale(
             clip,
             self._clipy.width,
             self._clipy.height,
@@ -136,16 +128,14 @@ class RescaleBase(vs_object, ABC):
         )
 
     @_add_props
-    def _generate_doubled(self, clip: ConstantFormatVideoNode) -> ConstantFormatVideoNode:
+    def _generate_doubled(self, clip: vs.VideoNode) -> vs.VideoNode:
         return self._upscaler.supersample(clip, 2)
 
     @_add_props
-    def _generate_upscale(self, clip: ConstantFormatVideoNode) -> ConstantFormatVideoNode:
-        return self._downscaler.scale(  # type: ignore[return-value]
-            clip, self._clipy.width, self._clipy.height, **self.descale_args.kwargs(clip)
-        )
+    def _generate_upscale(self, clip: vs.VideoNode) -> vs.VideoNode:
+        return self._downscaler.scale(clip, self._clipy.width, self._clipy.height, **self.descale_args.kwargs(clip))
 
-    descale = cachedproperty[ConstantFormatVideoNode, vs.VideoNode](
+    descale = cachedproperty[vs.VideoNode, vs.VideoNode](
         lambda self: self._generate_descale(self._clipy),
         lambda self, value: cachedproperty.update_cache(self, "descale", value),
         lambda self: cachedproperty.clear_cache(self, ["descale", "rescale", "doubled", "upscale"]),
@@ -154,7 +144,7 @@ class RescaleBase(vs_object, ABC):
     Gets the descaled clip.
     """
 
-    rescale = cachedproperty[ConstantFormatVideoNode, vs.VideoNode](
+    rescale = cachedproperty[vs.VideoNode, vs.VideoNode](
         lambda self: self._generate_rescale(self.descale),
         lambda self, value: cachedproperty.update_cache(self, "rescale", value),
         lambda self: cachedproperty.clear_cache(self, "rescale"),
@@ -163,7 +153,7 @@ class RescaleBase(vs_object, ABC):
     Gets the rescaled clip.
     """
 
-    doubled = cachedproperty[ConstantFormatVideoNode, vs.VideoNode](
+    doubled = cachedproperty[vs.VideoNode, vs.VideoNode](
         lambda self: self._generate_doubled(self.descale),
         lambda self, value: cachedproperty.update_cache(self, "doubled", value),
         lambda self: cachedproperty.clear_cache(self, ["doubled", "upscale"]),
@@ -172,7 +162,7 @@ class RescaleBase(vs_object, ABC):
     Gets the doubled clip.
     """
 
-    upscale = cachedproperty[ConstantFormatVideoNode, vs.VideoNode](
+    upscale = cachedproperty[vs.VideoNode, vs.VideoNode](
         lambda self: core.std.CopyFrameProps(
             join([self._generate_upscale(self.doubled), *self._chroma]), self._clipy, "_ChromaLocation"
         ),
@@ -183,11 +173,6 @@ class RescaleBase(vs_object, ABC):
     Returns the upscaled clip
     """
 
-    def __vs_del__(self, core_id: int) -> None:
-        del self._clipy
-        del self._chroma
-        cachedproperty.clear_cache(self)
-
 
 class Rescale(RescaleBase):
     """
@@ -279,11 +264,11 @@ class Rescale(RescaleBase):
         self,
         clip: vs.VideoNode,
         /,
-        height: int | float,
-        kernel: KernelLike,
+        height: int | float,  # noqa: PYI041
+        kernel: ComplexKernelLike,
         upscaler: ScalerLike = ArtCNN,
         downscaler: ScalerLike = Hermite(linear=True),
-        width: int | float | None = None,
+        width: int | float | None = None,  # noqa: PYI041
         base_height: int | None = None,
         base_width: int | None = None,
         crop: tuple[LeftCrop, RightCrop, TopCrop, BottomCrop] = CropRel(),
@@ -321,9 +306,9 @@ class Rescale(RescaleBase):
 
                 Defaults to ``0``.
         """
-        self._line_mask: ConstantFormatVideoNode | None = None
-        self._credit_mask: ConstantFormatVideoNode | None = None
-        self._ignore_mask: ConstantFormatVideoNode | None = None
+        self._line_mask: vs.VideoNode | None = None
+        self._credit_mask: vs.VideoNode | None = None
+        self._ignore_mask: vs.VideoNode | None = None
         self._crop = crop
         self._pre = clip
 
@@ -336,13 +321,13 @@ class Rescale(RescaleBase):
         if self._crop > (0, 0, 0, 0):
             self._clipy = self._clipy.std.Crop(*self._crop)
 
-    def _generate_descale(self, clip: ConstantFormatVideoNode) -> ConstantFormatVideoNode:
+    def _generate_descale(self, clip: vs.VideoNode) -> vs.VideoNode:
         if not self._ignore_mask:
             return super()._generate_descale(clip)
 
         @self._add_props
         @self._apply_field_based
-        def _generate_descale_ignore_mask(self: Rescale, clip: ConstantFormatVideoNode) -> ConstantFormatVideoNode:
+        def _generate_descale_ignore_mask(self: Rescale, clip: vs.VideoNode) -> vs.VideoNode:
             assert self._ignore_mask
 
             self.descale_args.mode = "h"
@@ -373,7 +358,7 @@ class Rescale(RescaleBase):
 
         return _generate_descale_ignore_mask(self, clip)
 
-    def _generate_upscale(self, clip: ConstantFormatVideoNode) -> ConstantFormatVideoNode:
+    def _generate_upscale(self, clip: vs.VideoNode) -> vs.VideoNode:
         upscale = super()._generate_upscale(clip)
 
         merged_mask = norm_expr([self.line_mask, self.credit_mask], "x y - 0 mask_max clamp", func=self.__class__)
@@ -396,7 +381,7 @@ class Rescale(RescaleBase):
         return upscale
 
     @property
-    def line_mask(self) -> ConstantFormatVideoNode:
+    def line_mask(self) -> vs.VideoNode:
         """Gets the lineart mask to be applied on the upscaled clip."""
         lm = self._line_mask or core.std.BlankClip(
             self._clipy, color=get_peak_value(self._clipy, False, ColorRange.FULL), keep=True
@@ -431,7 +416,7 @@ class Rescale(RescaleBase):
         self._line_mask = None
 
     @property
-    def credit_mask(self) -> ConstantFormatVideoNode:
+    def credit_mask(self) -> vs.VideoNode:
         """Gets the credit mask to be applied on the upscaled clip."""
         if self._credit_mask:
             return self._credit_mask
@@ -454,7 +439,7 @@ class Rescale(RescaleBase):
         self._credit_mask = None
 
     @property
-    def ignore_mask(self) -> ConstantFormatVideoNode:
+    def ignore_mask(self) -> vs.VideoNode:
         """Gets the ignore mask to be applied on the descaled clip."""
         if self._ignore_mask:
             return self._ignore_mask
@@ -476,7 +461,7 @@ class Rescale(RescaleBase):
 
     def default_line_mask(
         self, clip: vs.VideoNode | None = None, scaler: ScalerLike = Bilinear, **kwargs: Any
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Load a default Kirsch line mask in the class instance. Additionally, it is returned.
 
@@ -511,7 +496,7 @@ class Rescale(RescaleBase):
         stabilize: bool = True,
         scenechanges: Iterable[int] | None = None,
         **kwargs: Any,
-    ) -> ConstantFormatVideoNode:
+    ) -> vs.VideoNode:
         """
         Load a credit mask by making a difference mask between src and rescaled clips
 
@@ -547,11 +532,3 @@ class Rescale(RescaleBase):
         self.credit_mask = credit_mask
 
         return self.credit_mask
-
-    def __vs_del__(self, core_id: int) -> None:
-        del self._line_mask
-        del self._credit_mask
-        del self._ignore_mask
-        del self._pre
-
-        super().__vs_del__(core_id)
