@@ -6,7 +6,9 @@ from fractions import Fraction
 from itertools import count
 from typing import cast
 
-from vstools import CustomValueError, DependencyNotFoundError, Region, SPath, core, get_prop, vs
+from jetpytools import CustomValueError, DependencyNotFoundError, SPath
+
+from vstools import core, get_prop, vs
 
 from ...dataclasses import AllNeddedDvdFrameData
 from ...rff import apply_rff_array, apply_rff_video
@@ -19,6 +21,7 @@ from .parsedvd import (
     IFO0,
     IFOX,
     IFO0Title,
+    Region,
     SectorReadHelper,
 )
 from .title import Title
@@ -216,7 +219,7 @@ class IsoFile:
         # (the splitting logic assumes though that there is a chapter at the start and end)
         # TODO: verify these claims and check the splitting logic and figure out what the best solution is
         # you could either always add the end as chapter or stretch the last chapter till the end
-        # Guess #1: We only need to handle the case where last is not acually a chapter so stretching
+        # Guess #1: We only need to handle the case where last is not actually a chapter so stretching
         # is the only correct solution there, adding would be wrong
         output_chapters = [0, *output_chapters]
         lastchpt = len(rnode)
@@ -339,12 +342,12 @@ def dvdsrc_parse_vts(
 
     if not disable_rff:
         rnode = apply_rff_video(rawnode, staff.rff, staff.tff, staff.prog, staff.progseq)
-        _vobids = apply_rff_array(staff.vobids, staff.rff, staff.tff, staff.progseq)
+        vobids = apply_rff_array(staff.vobids, staff.rff, staff.tff, staff.progseq)
     else:
         rnode = rawnode
-        _vobids = staff.vobids
+        vobids = staff.vobids
 
-    return rnode, staff.rff, _vobids, vts_indices
+    return rnode, staff.rff, vobids, vts_indices
 
 
 def dvdsrc_extract_data(rawnode: vs.VideoNode) -> AllNeddedDvdFrameData:
