@@ -1,41 +1,36 @@
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
-from hashlib import md5
+from functools import cache
+from logging import getLogger
 from os import name as os_name
-from shutil import which
-from subprocess import Popen
-from tempfile import gettempdir
 from typing import Any, Callable, ClassVar, Iterable, Literal, Protocol, Sequence
 
+from jetpytools import MISSING, CustomRuntimeError, SPath, SPathLike, inject_self, to_arr
+
 from vstools import (
-    MISSING,
     ChromaLocation,
     ColorRangeLike,
-    CustomRuntimeError,
-    DataType,
     FieldBasedLike,
     MatrixLike,
     MissingT,
     PackageStorage,
     PrimariesLike,
-    SPath,
-    SPathLike,
     TransferLike,
     core,
     initialize_clip,
-    inject_self,
-    to_arr,
     vs,
 )
 
 from ..dataclasses import IndexFileType
 
-__all__ = ["ExternalIndexer", "Indexer", "VSSourceFunc"]
+__all__ = ["CacheIndexer", "ExternalIndexer", "Indexer", "VSSourceFunc"]
+
+log = getLogger(__name__)
 
 
 class VSSourceFunc(Protocol):
-    def __call__(self, path: DataType, *args: Any, **kwargs: Any) -> vs.VideoNode: ...
+    def __call__(self, path: str | bytes | bytearray, *args: Any, **kwargs: Any) -> vs.VideoNode: ...
 
 
 class Indexer(ABC):
@@ -61,13 +56,16 @@ class Indexer(ABC):
 
     @classmethod
     def get_videos_hash(cls, files: list[SPath]) -> str:
+        from hashlib import md5
+
         length = sum(file.stat().st_size for file in files)
         to_hash = length.to_bytes(32, "little") + cls.get_joined_names(files).encode()
         return md5(to_hash).hexdigest()
 
     @classmethod
-    def source_func(cls, path: DataType | SPathLike, *args: Any, **kwargs: Any) -> vs.VideoNode:
-        return cls._source_func(str(path), *args, **kwargs)
+    def source_func(cls, path: SPathLike, **kwargs: Any) -> vs.VideoNode:
+        log.debug("%s: indexing %r; arguments: %r", cls, path, kwargs)
+        return cls._source_func(str(path), **kwargs)
 
     @classmethod
     def normalize_filenames(cls, file: SPathLike | Sequence[SPathLike]) -> list[SPath]:
@@ -113,6 +111,13 @@ class Indexer(ABC):
         idx_props: bool = True,
         **kwargs: Any,
     ) -> vs.VideoNode:
+        """
+        Load one or more input files using the indexer and return a processed clip.
+
+        The returned clip is passed through [initialize_clip][vstools.initialize_clip] to apply bit depth conversion
+        and frame props initialization.
+        """
+
         nfiles = self.normalize_filenames(file)
         clip = self._source(
             [self.source_func(f.to_str(), **self.indexer_kwargs | kwargs) for f in nfiles],
@@ -130,6 +135,33 @@ class Indexer(ABC):
         return clip
 
 
+@cache
+def _get_indexer_cache_storage() -> PackageStorage:
+    return PackageStorage(package_name=f"{__name__}")
+
+
+class CacheIndexer(Indexer):
+    """Indexer interface with cache storage logic."""
+
+    _cache_arg_name: ClassVar[str]
+    _ext: ClassVar[str | None]
+
+    @staticmethod
+    def get_cache_path(file_name: SPathLike, ext: str | None = None) -> SPath:
+        storage = _get_indexer_cache_storage()
+
+        return storage.get_file(file_name, ext=ext)
+
+    @classmethod
+    def source_func(cls, path: SPathLike, **kwargs: Any) -> vs.VideoNode:
+        path = SPath(path)
+
+        if cls._cache_arg_name not in kwargs:
+            kwargs[cls._cache_arg_name] = cls.get_cache_path(path.name, cls._ext)
+
+        return super().source_func(path, **kwargs)
+
+
 class ExternalIndexer(Indexer):
     _bin_path: ClassVar[str]
     _ext: ClassVar[str]
@@ -176,11 +208,15 @@ class ExternalIndexer(Indexer):
         raise NotImplementedError
 
     def _get_bin_path(self) -> SPath:
+        from shutil import which
+
         if not (bin_path := which(str(self.bin_path))):
             raise FileNotFoundError(f"Indexer: `{self.bin_path}` was not found{' in PATH' if os_name == 'nt' else ''}!")
         return SPath(bin_path)
 
     def _run_index(self, files: list[SPath], output: SPath, cmd_args: Sequence[str]) -> None:
+        from subprocess import Popen
+
         output.mkdirp()
 
         proc = Popen(
@@ -215,6 +251,8 @@ class ExternalIndexer(Indexer):
             return SPath(file).get_folder() if file else self.get_out_folder(False)
 
         if not output_folder:
+            from tempfile import gettempdir
+
             return SPath(gettempdir())
 
         return SPath(output_folder)
