@@ -1,84 +1,17 @@
 from __future__ import annotations
 
 from fractions import Fraction
-from typing import Callable, Iterator, Literal, NamedTuple
+from typing import Callable, Iterator, Literal, NamedTuple, Self
 
-import vapoursynth as vs
-from jetpytools import Coordinate, CustomIntEnum, CustomStrEnum, Position, Sentinel, SentinelT, Size
-from typing_extensions import Self
+from jetpytools import CustomIntEnum, CustomRuntimeError, Sentinel, SentinelT
 
-from ..types import ConstantFormatVideoNode, HoldsPropValue, VideoNodeT
+from ..types import HoldsPropValue
+from ..vs_proxy import vs
 
-__all__ = ["Coordinate", "Dar", "Direction", "Position", "Region", "Resolution", "Sar", "SceneChangeMode", "Size"]
+__all__ = ["Dar", "Resolution", "Sar", "SceneChangeMode"]
 
 
-class Direction(CustomIntEnum):
-    """
-    Enum to simplify the direction argument.
-    """
-
-    HORIZONTAL = 0
-    VERTICAL = 1
-
-    LEFT = 2
-    RIGHT = 3
-    UP = 4
-    DOWN = 5
-
-    @property
-    def is_axis(self) -> bool:
-        """
-        Whether the Direction represents an axis (horizontal/vertical).
-        """
-
-        return self <= self.VERTICAL
-
-    @property
-    def is_way(self) -> bool:
-        """
-        Whether the Direction is one of the 4 arrow directions.
-        """
-
-        return self > self.VERTICAL
-
-    @property
-    def string(self) -> str:
-        """
-        A string representation of the Direction.
-        """
-
-        return self._name_.lower()
-
-
-class _Xar(Fraction):
-    @classmethod
-    def from_param(cls, value: Self | bool | float | None, fallback: Self | float) -> Self | None:
-        """
-        Get the Xar from a Xar, a boolean, a float or a None object.
-
-        Args:
-            value: Value identifier.
-            fallback: Fallback value.
-
-        Returns:
-            Xar object or None.
-        """
-        if value is False:
-            return cls(fallback)
-
-        if value is True:
-            return None
-
-        if isinstance(value, cls):
-            return value
-
-        if value is not None:
-            return cls.from_float(float(value))
-
-        return None
-
-
-class Dar(_Xar):
+class Dar(Fraction):
     """
     A Fraction representing the Display Aspect Ratio.
 
@@ -142,7 +75,7 @@ class Dar(_Xar):
         return Sar.from_ar(active_area, height, self)
 
 
-class Sar(_Xar):
+class Sar(Fraction):
     """
     A Fraction representing the Sample Aspect Ratio.
 
@@ -188,7 +121,7 @@ class Sar(_Xar):
 
         return cls(dar / (Fraction(active_area) / height))
 
-    def apply(self, clip: VideoNodeT) -> VideoNodeT:
+    def apply(self, clip: vs.VideoNode) -> vs.VideoNode:
         """
         Apply the SAR values as _SARNum and _SARDen frame properties to a clip.
         """
@@ -196,93 +129,6 @@ class Sar(_Xar):
         return vs.core.std.SetFrameProps(clip, _SARNum=self.numerator, _SARDen=self.denominator)
 
 
-class Region(CustomStrEnum):
-    """
-    StrEnum signifying an analog television region.
-    """
-
-    UNKNOWN = "unknown"
-    """
-    Unknown region.
-    """
-
-    NTSC = "NTSC"
-    """
-    The first American standard for analog television broadcast was developed by
-    National Television System Committee (NTSC) in 1941.
-
-    For more information see `this <https://en.wikipedia.org/wiki/NTSC>`_.
-    """
-
-    NTSCi = "NTSCi"
-    """
-    Interlaced NTSC.
-    """
-
-    PAL = "PAL"
-    """
-    Phase Alternating Line (PAL) colour encoding system.
-
-    For more information see `this <https://en.wikipedia.org/wiki/PAL>`_.
-    """
-
-    PALi = "PALi"
-    """
-    Interlaced PAL.
-    """
-
-    FILM = "FILM"
-    """
-    True 24fps content.
-    """
-
-    NTSC_FILM = "NTSC (FILM)"
-    """
-    NTSC 23.976fps content.
-    """
-
-    @property
-    def framerate(self) -> Fraction:
-        """
-        Obtain the Region's framerate.
-        """
-
-        return _region_framerate_map[self]
-
-    @classmethod
-    def from_framerate(cls, framerate: float | Fraction, strict: bool = False) -> Self:
-        """
-        Determine the Region using a given framerate.
-        """
-
-        key = Fraction(framerate)
-
-        if strict:
-            return cls(_framerate_region_map[key])
-
-        if key not in _framerate_region_map:
-            diffs = [(k, abs(float(key) - float(v))) for k, v in _region_framerate_map.items()]
-
-            diffs.sort(key=lambda x: x[1])
-
-            return cls(diffs[0][0])
-
-        return cls(_framerate_region_map[key])
-
-
-_region_framerate_map = {
-    Region.UNKNOWN: Fraction(0),
-    Region.NTSC: Fraction(30000, 1001),
-    Region.NTSCi: Fraction(60000, 1001),
-    Region.PAL: Fraction(25, 1),
-    Region.PALi: Fraction(50, 1),
-    Region.FILM: Fraction(24, 1),
-    Region.NTSC_FILM: Fraction(24000, 1001),
-}
-
-_framerate_region_map = {r.framerate: r for r in Region}
-
-
 class Resolution(NamedTuple):
     """
     Tuple representing a resolution.
@@ -296,10 +142,6 @@ class Resolution(NamedTuple):
         """
         Create a Resolution object using a given clip's dimensions.
         """
-        from ..functions import check_variable_resolution
-
-        assert check_variable_resolution(clip, cls.from_video)
-
         return cls(clip.width, clip.height)
 
     def transpose(self) -> Self:
@@ -359,17 +201,13 @@ class SceneChangeMode(CustomIntEnum):
             SceneChangeMode.WWXD_SCXVID_INTERSECTION,
         )
 
-    def ensure_presence(self, clip: vs.VideoNode) -> ConstantFormatVideoNode:
+    def ensure_presence(self, clip: vs.VideoNode) -> vs.VideoNode:
         """
         Ensures all the frame properties necessary for scene change detection are created.
         """
-        from ..exceptions import CustomRuntimeError
-        from ..functions import check_variable_format
         from ..utils import merge_clip_props
 
-        assert check_variable_format(clip, self.ensure_presence)
-
-        stats_clip = list[ConstantFormatVideoNode]()
+        stats_clip = list[vs.VideoNode]()
 
         if self.is_SCXVID:
             if not hasattr(vs.core, "scxvid"):
@@ -409,7 +247,7 @@ class SceneChangeMode(CustomIntEnum):
     def lambda_cb(self) -> Callable[[int, vs.VideoFrame], SentinelT | int]:
         return lambda n, f: Sentinel.check(n, bool(f[0][0, 0]))
 
-    def prepare_clip(self, clip: vs.VideoNode, height: int | Literal[False] = 360) -> ConstantFormatVideoNode:
+    def prepare_clip(self, clip: vs.VideoNode, height: int | Literal[False] = 360) -> vs.VideoNode:
         """
         Prepare a clip for scene change metric calculations.
 
