@@ -2,9 +2,10 @@ from __future__ import annotations
 
 from typing import Any
 
-import vapoursynth as vs
 from jetpytools import CustomPermissionError, CustomValueError, FuncExcept, SupportsString
 
+from ..vs_proxy import vs
+
 __all__ = [
     "InvalidColorspacePathError",
     "InvalidMatrixError",
@@ -13,6 +14,7 @@ __all__ = [
     "ReservedMatrixError",
     "ReservedPrimariesError",
     "ReservedTransferError",
+    "UndefinedColorRangeError",
     "UndefinedMatrixError",
     "UndefinedPrimariesError",
     "UndefinedTransferError",
@@ -179,6 +181,12 @@ class InvalidPrimariesError(CustomValueError):
 # ColorRange
 
 
+class UndefinedColorRangeError(CustomValueError):
+    """
+    Raised when a undefined color range value is passed.
+    """
+
+
 class UnsupportedColorRangeError(CustomValueError):
     """
     Raised when a unsupported color range value is passed.
