@@ -1,21 +1,32 @@
 from __future__ import annotations
 
 import re
+from abc import abstractmethod
 from dataclasses import dataclass
 from fractions import Fraction
 from functools import cache
 from pathlib import Path
-from typing import Any, ClassVar, Iterable, Literal, NamedTuple, TypeVar, overload
+from typing import Any, ClassVar, Iterable, Literal, NamedTuple, Self, cast, overload
 
-import vapoursynth as vs
-from jetpytools import CustomValueError, FilePathType, FuncExcept, LinearRangeLut, Sentinel, SPath, inject_self
-from typing_extensions import Self
+from jetpytools import (
+    CustomValueError,
+    FilePathType,
+    FuncExcept,
+    LinearRangeLut,
+    Sentinel,
+    SPath,
+    check_perms,
+    classproperty,
+    fallback,
+    inject_self,
+)
 
 from ..enums import Matrix, SceneChangeMode
-from ..exceptions import FramesLengthError, InvalidTimecodeVersionError
-from ..types import VideoNodeT
-from .file import PackageStorage
-from .render import clip_async_render
+from ..exceptions import FramesLengthError, UnsupportedTimecodeVersionError
+from ..utils import DynamicClipsCache, PackageStorage
+from ..vs_proxy import VSObject, vs
+from .ranges import replace_ranges
+from .render import clip_async_render, clip_data_gather
 
 __all__ = ["Keyframes", "LWIndex", "Timecodes"]
 
@@ -126,9 +137,6 @@ class Timecodes(list[FrameDur]):
         """
         Convert from normalized ranges to a list of frame duration.
         """
-
-        from .funcs import fallback
-
         norm_timecodes = [assume] * length if assume else list[Fraction]()
 
         for (startn, endn), fps in timecodes.items():
@@ -245,7 +253,7 @@ class Timecodes(list[FrameDur]):
 
         denominator = den or fb_den or 1001
 
-        version, *_timecodes = file.read_text().splitlines()
+        version, *timecodes = file.read_text().splitlines()
 
         if "v1" in version:
 
@@ -256,20 +264,20 @@ class Timecodes(list[FrameDur]):
 
             timecodes_d = dict[tuple[int | None, int | None], Fraction]()
 
-            for line in _timecodes:
+            for line in timecodes:
                 if line.startswith("#"):
                     continue
 
                 if line.startswith("Assume"):
-                    assume = _norm(_timecodes[0][7:])
+                    assume = _norm(timecodes[0][7:])
                     continue
 
-                starts, ends, _fps = line.split(",")
-                timecodes_d[(int(starts), int(ends) + 1)] = _norm(_fps)
+                starts, ends, fps = line.split(",")
+                timecodes_d[int(starts), int(ends) + 1] = _norm(fps)
 
             norm_timecodes = cls.normalize_range_timecodes(timecodes_d, length, assume)
         elif "v2" in version:
-            timecodes_l = [float(t) for t in _timecodes if not t.startswith("#")]
+            timecodes_l = [float(t) for t in timecodes if not t.startswith("#")]
             norm_timecodes = [
                 Fraction(denominator, int(denominator / float(f"{round((x - y) * 100, 4) / 100000:.08f}"[:-1])))
                 for x, y in zip(timecodes_l[1:], timecodes_l[:-1])
@@ -287,7 +295,7 @@ class Timecodes(list[FrameDur]):
 
         return cls(FrameDur(i, f.numerator, f.denominator) for i, f in enumerate(norm_timecodes))
 
-    def assume_vfr(self, clip: VideoNodeT, func: FuncExcept | None = None) -> VideoNodeT:
+    def assume_vfr(self, clip: vs.VideoNode, func: FuncExcept | None = None) -> vs.VideoNode:
         """
         Force the given clip to be assumed to be vfr by other applications.
 
@@ -298,8 +306,6 @@ class Timecodes(list[FrameDur]):
         Returns:
             Clip that should always be assumed to be vfr by other applications.
         """
-        from ..utils import replace_ranges
-
         func = func or self.assume_vfr
 
         major_time, minor_fps = self.accumulate_norm_timecodes(self)
@@ -326,15 +332,13 @@ class Timecodes(list[FrameDur]):
             out: Path to write the file to.
             format: Format to write the file to.
         """
-        from ..utils import check_perms
-
         func = func or self.to_file
 
         out_path = Path(str(out)).resolve()
 
         check_perms(out_path, "w+", func=func)
 
-        InvalidTimecodeVersionError.check(self.to_file, format)
+        UnsupportedTimecodeVersionError.check(self.to_file, format)
 
         out_text = [f"# timecode format v{format}"]
 
@@ -403,9 +407,9 @@ class Keyframes(list[int]):
         not empty, the keyframes are loaded from the file. Otherwise, they are
         detected from the clip and then written to the file.
 
-        Examples:
-            When working on a TV series, the episode number can be a convenient
-            key (e.g. `"01"` for episode 1, `"02"` for episode 2, etc.):
+        Example:
+            When working on a TV series, the episode number can be a convenient key
+            (e.g. `"01"` for episode 1, `"02"` for episode 2, etc.):
             ```py
             keyframes = Keyframes.unique(clip, "01")
             ```
@@ -458,7 +462,7 @@ class Keyframes(list[int]):
         prop_key: str = next(iter(SceneChangeMode.SCXVID.prop_keys)),
         scene_idx_prop: bool = False,
     ) -> vs.VideoNode:
-        from ..utils import replace_ranges
+        from .ranges import replace_ranges
 
         propset_clip = clip.std.SetFrameProp(prop_key, True)
 
@@ -528,8 +532,6 @@ class Keyframes(list[int]):
         header: bool = True,
         force: bool = False,
     ) -> None:
-        from ..utils import check_perms
-
         func = func or self.to_file
 
         out_path = Path(str(out)).resolve()
@@ -577,7 +579,64 @@ class Keyframes(list[int]):
         return cls(param)
 
 
-KeyframesBoundT = TypeVar("KeyframesBoundT", bound=Keyframes)
+class SceneBasedDynamicCache(DynamicClipsCache[int]):
+    def __init__(self, clip: vs.VideoNode, keyframes: Keyframes | str, cache_size: int = 5) -> None:
+        super().__init__(cache_size)
+
+        self.clip = clip
+        self.keyframes = Keyframes.from_param(clip, keyframes)
+
+    @abstractmethod
+    def get_clip(self, key: int) -> vs.VideoNode: ...
+
+    def get_eval(self) -> vs.VideoNode:
+        return self.clip.std.FrameEval(lambda n: self[self.keyframes.scenes.indices[n]])
+
+    @classmethod
+    def from_clip(cls, clip: vs.VideoNode, keyframes: Keyframes | str, *args: Any, **kwargs: Any) -> vs.VideoNode:
+        return cls(clip, keyframes, *args, **kwargs).get_eval()
+
+
+class SceneAverageStats(SceneBasedDynamicCache):
+    _props_keys = ("Min", "Max", "Average")
+
+    class _Cache(VSObject, dict[int, tuple[float, float, float]]):
+        def __init__(self, clip: vs.VideoNode, keyframes: Keyframes, plane: int) -> None:
+            self.props = clip.std.PlaneStats(plane=plane)
+            self.keyframes = keyframes
+
+        def __getitem__(self, idx: int) -> tuple[float, float, float]:
+            if idx not in self:
+                frame_range = self.keyframes.scenes[idx]
+                cut_clip = self.props[frame_range.start : frame_range.stop]
+
+                frames_min_max_avg = clip_data_gather(
+                    cut_clip,
+                    None,
+                    lambda n, f: tuple(cast(float, f.props[f"PlaneStats{p}"]) for p in SceneAverageStats._props_keys),
+                )
+
+                frames_min, frames_max, frames_avgs = [[x[i] for x in frames_min_max_avg] for i in (0, 1, 2)]
+
+                self[idx] = (min(frames_min), max(frames_max), sum(frames_avgs) / len(frames_avgs))
+
+            return super().__getitem__(idx)
+
+    def __init__(
+        self,
+        clip: vs.VideoNode,
+        keyframes: Keyframes | str,
+        prop: str = "SceneStats",
+        plane: int = 0,
+        cache_size: int = 5,
+    ) -> None:
+        super().__init__(clip, keyframes, cache_size)
+
+        self.prop_keys = tuple(f"{prop}{x}" for x in self._props_keys)
+        self.scene_avgs = self._Cache(self.clip, self.keyframes, plane)
+
+    def get_clip(self, key: int) -> vs.VideoNode:
+        return self.clip.std.SetFrameProps(**dict(zip(self.prop_keys, self.scene_avgs[key])))
 
 
 @dataclass
@@ -587,20 +646,29 @@ class LWIndex:
     keyframes: Keyframes
 
     class Regex:
-        frame_first = re.compile(
-            r"Index=(?P<Index>-?[0-9]+),POS=(?P<POS>-?[0-9]+),PTS=(?P<PTS>-?[0-9]+),"
-            r"DTS=(?P<DTS>-?[0-9]+),EDI=(?P<EDI>-?[0-9]+)"
-        )
+        @classproperty
+        @classmethod
+        def frame_first(cls) -> re.Pattern[str]:
+            return re.compile(
+                r"Index=(?P<Index>-?[0-9]+),POS=(?P<POS>-?[0-9]+),PTS=(?P<PTS>-?[0-9]+),"
+                r"DTS=(?P<DTS>-?[0-9]+),EDI=(?P<EDI>-?[0-9]+)"
+            )
 
-        frame_second = re.compile(
-            r"Key=(?P<Key>-?[0-9]+),Pic=(?P<Pic>-?[0-9]+),POC=(?P<POC>-?[0-9]+),"
-            r"Repeat=(?P<Repeat>-?[0-9]+),Field=(?P<Field>-?[0-9]+)"
-        )
+        @classproperty
+        @classmethod
+        def frame_second(cls) -> re.Pattern[str]:
+            return re.compile(
+                r"Key=(?P<Key>-?[0-9]+),Pic=(?P<Pic>-?[0-9]+),POC=(?P<POC>-?[0-9]+),"
+                r"Repeat=(?P<Repeat>-?[0-9]+),Field=(?P<Field>-?[0-9]+)"
+            )
 
-        streaminfo = re.compile(
-            r"Codec=(?P<Codec>[0-9]+),TimeBase=(?P<TimeBase>[0-9\/]+),Width=(?P<Width>[0-9]+),"
-            r"Height=(?P<Height>[0-9]+),Format=(?P<Format>[0-9a-zA-Z]+),ColorSpace=(?P<ColorSpace>[0-9]+)"
-        )
+        @classproperty
+        @classmethod
+        def streaminfo(cls) -> re.Pattern[str]:
+            return re.compile(
+                r"Codec=(?P<Codec>[0-9]+),TimeBase=(?P<TimeBase>[0-9\/]+),Width=(?P<Width>[0-9]+),"
+                r"Height=(?P<Height>[0-9]+),Format=(?P<Format>[0-9a-zA-Z]+),ColorSpace=(?P<ColorSpace>[0-9]+)"
+            )
 
     class StreamInfo(NamedTuple):
         codec: int
