@@ -1,33 +1,25 @@
 from __future__ import annotations
 
-from functools import partial, wraps
+import operator
+from functools import partial, reduce, wraps
 from types import NoneType
-from typing import Callable, Iterable, Mapping, Sequence, SupportsIndex, Union, overload
+from typing import Any, Callable, Iterable, Mapping, Self, Sequence, SupportsIndex, Union, overload
 from weakref import WeakValueDictionary
 
-import vapoursynth as vs
-from jetpytools import (
-    CustomIndexError,
-    CustomStrEnum,
-    CustomTypeError,
-    CustomValueError,
-    FuncExcept,
-    P,
-    normalize_seq,
-    to_arr,
-)
+from jetpytools import CustomIndexError, CustomStrEnum, CustomTypeError, FuncExcept, normalize_seq, to_arr
 
 from ..enums import ColorRange, ColorRangeLike
-from ..exceptions import ClipLengthError, InvalidColorFamilyError
-from ..types import ConstantFormatVideoNode, HoldsVideoFormat, Planes, VideoFormatLike, VideoNodeIterableT
-from .check import check_variable_format
-from .clip import shift_clip
+from ..exceptions import ClipLengthError, UnsupportedColorFamilyError
+from ..types import HoldsVideoFormat, Planes, VideoFormatLike, VideoNodeIterable
+from ..utils import flatten, get_depth, get_video_format
+from ..vs_proxy import vs
 
 __all__ = [
     "EXPR_VARS",
     "DitherType",
     "depth",
-    "depth_func",
+    "expect_bits",
+    "flatten_vnodes",
     "frame2clip",
     "get_b",
     "get_g",
@@ -39,7 +31,6 @@ __all__ = [
     "join",
     "limiter",
     "plane",
-    "sc_detect",
     "split",
     "stack_clips",
 ]
@@ -108,91 +99,93 @@ class DitherType(CustomStrEnum):
     Floyd-Steinberg error diffusion.
     """
 
-    ERROR_DIFFUSION_FMTC = "error_diffusion_fmtc"
+    ERROR_DIFFUSION_FMTC = "error_diffusion_fmtc", 6
     """
     Floyd-Steinberg error diffusion.
-    Modified for serpentine scan (avoids worm artefacts).
+    Modified for serpentine scan (avoids worm artifacts).
     """
 
-    SIERRA_2_4A = "sierra_2_4a"
+    SIERRA_2_4A = "sierra_2_4a", 3
     """
     Another type of error diffusion.
     Quick and excellent quality, similar to Floyd-Steinberg.
     """
 
-    STUCKI = "stucki"
+    STUCKI = "stucki", 4
     """
     Another error diffusion kernel.
     Preserves delicate edges better but distorts gradients.
     """
 
-    ATKINSON = "atkinson"
+    ATKINSON = "atkinson", 5
     """
     Another error diffusion kernel.
     Generates distinct patterns but keeps clean the flat areas (noise modulation).
     """
 
-    OSTROMOUKHOV = "ostromoukhov"
+    OSTROMOUKHOV = "ostromoukhov", 7
     """
     Another error diffusion kernel.
-    Slow, available only for integer input at the moment. Avoids usual F-S artefacts.
+    Slow, available only for integer input at the moment. Avoids usual F-S artifacts.
     """
 
-    VOID = "void"
+    VOID = "void", 8
     """
     A way to generate blue-noise dither and has a much better visual aspect than ordered dithering.
     """
 
-    QUASIRANDOM = "quasirandom"
+    QUASIRANDOM = "quasirandom", 9
     """
     Dither using quasirandom sequences.
     Good intermediary between void, cluster, and error diffusion algorithms.
     """
 
+    _fmtc_dmode: int | None
+
+    def __new__(cls, value: Any, fmtc_dmode: int | None = None) -> Self:
+        obj = str.__new__(cls, value)
+        obj._value_ = value
+
+        obj._fmtc_dmode = fmtc_dmode
+
+        return obj
+
     def apply(
-        self, clip: vs.VideoNode, fmt_out: vs.VideoFormat, range_in: ColorRange, range_out: ColorRange
-    ) -> ConstantFormatVideoNode:
+        self, clip: vs.VideoNode, out_fmt: vs.VideoFormat, range_in: ColorRange, range_out: ColorRange
+    ) -> vs.VideoNode:
         """
         Apply the given DitherType to a clip.
         """
-
-        from ..utils import get_video_format
-
-        assert self != DitherType.AUTO, CustomValueError("Cannot apply AUTO.", self.__class__)
+        if self is DitherType.AUTO:
+            self = DitherType.VOID if DitherType.should_dither(clip, out_fmt, range_in, range_out) else DitherType.NONE
 
         fmt = get_video_format(clip)
         clip = ColorRange.ensure_presence(clip, range_in)
 
-        if not self.is_fmtc:
+        if not self.is_fmtc():
             return clip.resize.Point(
-                format=fmt_out.id,
+                format=out_fmt.id,
                 dither_type=self.value.lower(),
                 range_in=range_in.value_zimg,
                 range=range_out.value_zimg,
             )
 
-        if fmt.sample_type is vs.FLOAT:
-            if self == DitherType.OSTROMOUKHOV:
-                raise CustomValueError("Ostromoukhov can't be used for float input.", self.__class__)
-
-            # Workaround because fmtc doesn't support FLOAT 16 input
-            if fmt.bits_per_sample < 32:
-                clip = DitherType.NONE.apply(clip, fmt.replace(bits_per_sample=32), range_in, range_out)
+        # Workaround because fmtc doesn't support FLOAT 16 input
+        if fmt.sample_type is vs.FLOAT and fmt.bits_per_sample == 16:
+            clip = DitherType.NONE.apply(clip, fmt.replace(bits_per_sample=32), range_in, range_out)
 
         return clip.fmtc.bitdepth(
-            dmode=_dither_fmtc_types.get(self),
-            bits=fmt_out.bits_per_sample,
+            dmode=self._fmtc_dmode,
+            bits=out_fmt.bits_per_sample,
             fulls=range_in is ColorRange.FULL,
             fulld=range_out is ColorRange.FULL,
         )
 
-    @property
     def is_fmtc(self) -> bool:
         """
         Whether the DitherType is applied through fmtc.
         """
-
-        return self in _dither_fmtc_types
+        return self._fmtc_dmode is not None
 
     @overload
     @staticmethod
@@ -206,19 +199,6 @@ class DitherType(CustomStrEnum):
         """
         Automatically determines whether dithering is needed for a given depth/range/sample type conversion.
 
-        If an input range is specified, an output range *should* be specified, otherwise it assumes a range conversion.
-
-        For an explanation of when dithering is needed:
-            - Dithering is NEVER needed if the conversion results in a float sample type.
-            - Dithering is ALWAYS needed for a range conversion (i.e. full to limited or vice-versa).
-            - Dithering is ALWAYS needed to convert a float sample type to an integer sample type.
-            - Dithering is needed when upsampling full range content except when one depth is a multiple of the other,
-              when the upsampling is a simple integer multiplication, e.g. for 8 -> 16: (0-255) * 257 -> (0-65535).
-            - Dithering is needed when downsampling limited or full range.
-
-        Dithering is theoretically needed when converting from an integer depth greater than 10 to half float,
-        despite the higher bit depth, but zimg's internal resampler currently does not dither for float output.
-
         Args:
             in_fmt: Input clip, frame or video format.
             out_fmt: Output clip, frame or video format.
@@ -243,26 +223,13 @@ class DitherType(CustomStrEnum):
         """
         Automatically determines whether dithering is needed for a given depth/range/sample type conversion.
 
-        If an input range is specified, an output range *should* be specified, otherwise it assumes a range conversion.
-
-        For an explanation of when dithering is needed:
-            - Dithering is NEVER needed if the conversion results in a float sample type.
-            - Dithering is ALWAYS needed for a range conversion (i.e. full to limited or vice-versa).
-            - Dithering is ALWAYS needed to convert a float sample type to an integer sample type.
-            - Dithering is needed when upsampling full range content except when one depth is a multiple of the other,
-              when the upsampling is a simple integer multiplication, e.g. for 8 -> 16: (0-255) * 257 -> (0-65535).
-            - Dithering is needed when downsampling limited or full range.
-
-        Dithering is theoretically needed when converting from an integer depth greater than 10 to half float,
-        despite the higher bit depth, but zimg's internal resampler currently does not dither for float output.
-
         Args:
             in_bits: Input bitdepth.
             out_bits: Output bitdepth.
-            in_sample_type: Input sample type.
-            out_sample_type: Output sample type.
             in_range: Input color range.
             out_range: Output color range.
+            in_sample_type: Input sample type.
+            out_sample_type: Output sample type.
 
         Returns:
             Whether the clip should be dithered.
@@ -278,13 +245,39 @@ class DitherType(CustomStrEnum):
         in_sample_type: vs.SampleType | None = None,
         out_sample_type: vs.SampleType | None = None,
     ) -> bool:
-        from ..utils import get_video_format
+        """
+        Automatically determines whether dithering is needed for a given depth/range/sample type conversion.
 
+        If an input range is specified, an output range *should* be specified, otherwise it assumes a range conversion.
+
+        For an explanation of when dithering is needed:
+
+        - Dithering is NEVER needed if the conversion results in a float sample type.
+        - Dithering is ALWAYS needed for a range conversion (i.e. full to limited or vice-versa).
+        - Dithering is ALWAYS needed to convert a float sample type to an integer sample type.
+        - Dithering is needed when upsampling full range content except when one depth is a multiple of the other,
+            when the upsampling is a simple integer multiplication, e.g. for 8 -> 16: (0-255) * 257 -> (0-65535).
+        - Dithering is needed when downsampling limited or full range.
+
+        Dithering is theoretically needed when converting from an integer depth greater than 10 to half float,
+        despite the higher bit depth, but zimg's internal resampler currently does not dither for float output.
+
+        Args:
+            in_bits_or_fmt: Input bitdepth, clip, frame or video format.
+            out_bits_or_fmt: Output bitdepth, clip, frame or video format.
+            in_range: Input color range.
+            out_range: Output color range.
+            in_sample_type: Input sample type.
+            out_sample_type: Output sample type.
+
+        Returns:
+            Whether the clip should be dithered.
+        """
         in_fmt = get_video_format(in_bits_or_fmt, sample_type=in_sample_type)
         out_fmt = get_video_format(out_bits_or_fmt, sample_type=out_sample_type)
 
-        in_range = ColorRange.from_param(in_range, (DitherType.should_dither, "in_range"))
-        out_range = ColorRange.from_param(out_range, (DitherType.should_dither, "out_range"))
+        in_range = ColorRange.from_param_with_fallback(in_range)
+        out_range = ColorRange.from_param_with_fallback(out_range)
 
         if out_fmt.sample_type is vs.FLOAT:
             return False
@@ -307,17 +300,6 @@ class DitherType(CustomStrEnum):
         return in_range == ColorRange.FULL and bool(out_bits % in_bits)
 
 
-_dither_fmtc_types: dict[DitherType, int] = {
-    DitherType.SIERRA_2_4A: 3,
-    DitherType.STUCKI: 4,
-    DitherType.ATKINSON: 5,
-    DitherType.ERROR_DIFFUSION_FMTC: 6,
-    DitherType.OSTROMOUKHOV: 7,
-    DitherType.VOID: 8,
-    DitherType.QUASIRANDOM: 9,
-}
-
-
 def depth(
     clip: vs.VideoNode,
     bitdepth: VideoFormatLike | HoldsVideoFormat | int | None = None,
@@ -327,49 +309,43 @@ def depth(
     range_in: ColorRangeLike | None = None,
     range_out: ColorRangeLike | None = None,
     dither_type: str | DitherType = DitherType.AUTO,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     A convenience bitdepth conversion function using only internal plugins if possible.
 
     This uses exclusively internal plugins except for specific dither_types.
-    To check whether your DitherType uses fmtc, use `DitherType.is_fmtc`.
+    To check whether your DitherType uses fmtc, use [DitherType.is_fmtc][vstools.DitherType.is_fmtc].
 
-        >>> src_8 = vs.core.std.BlankClip(format=vs.YUV420P8)
-        >>> src_10 = depth(src_8, 10)
-        >>> src_10.format.name
-        'YUV420P10'
+    Example:
+        ```py
+        rc_8 = vs.core.std.BlankClip(format=vs.YUV420P8)
+        rc_10 = depth(src_8, 10)
+        print(rc_10.format.name)  # YUV420P10
 
-        >>> src2_10 = vs.core.std.BlankClip(format=vs.RGB30)
-        >>> src2_8 = depth(src2_10, 8, dither_type=Dither.RANDOM)  # override default dither behavior
-        >>> src2_8.format.name
-        'RGB24'
+        rc2_10 = vs.core.std.BlankClip(format=vs.RGB30)
+        rc2_8 = depth(src2_10, 8, dither_type=Dither.RANDOM)  # override default dither behavior
+        print(rc2_8.format.name)  # RGB24
+        ```
 
     Args:
         clip: Input clip.
         bitdepth: Desired bitdepth of the output clip.
-        sample_type: Desired sample type of output clip. Allows overriding default float/integer behavior. Accepts
-            ``vapoursynth.SampleType`` enums ``vapoursynth.INTEGER`` and ``vapoursynth.FLOAT`` or their values, ``0``
-            and ``1`` respectively.
+        sample_type: Desired sample type of output clip. Allows overriding default float/integer behavior.
         range_in: Input pixel range (defaults to input `clip`'s range).
         range_out: Output pixel range (defaults to input `clip`'s range).
         dither_type: Dithering algorithm. Allows overriding default dithering behavior.
-            See [Dither][vstools.DitherType].
+            See [DitherType][vstools.DitherType].
 
             When integer output is desired but the conversion may produce fractional values,
-            defaults to DitherType.VOID if it is available via the fmtc VapourSynth plugin,
-            or to Floyd-Steinberg DitherType.ERROR_DIFFUSION for 8-bit output
-            or DitherType.ORDERED for higher bit depths.
-            In other cases, defaults to DitherType.NONE, or round to nearest.
+            defaults to [VOID][vstools.DitherType.VOID].
+
+            In other cases, defaults to no dither.
+
             See [DitherType.should_dither][vstools.DitherType.should_dither] for more information.
 
     Returns:
-        Converted clip with desired bit depth and sample type. ``ColorFamily`` will be same as input.
+        Converted clip with desired bit depth and sample type. `ColorFamily` will be same as input.
     """
-
-    from ..utils import get_video_format
-
-    assert check_variable_format(clip, depth)
-
     in_fmt = get_video_format(clip)
     out_fmt = get_video_format(bitdepth or clip, sample_type=sample_type)
 
@@ -383,26 +359,37 @@ def depth(
     ):
         return clip
 
-    dither_type = DitherType(dither_type)
-
-    if dither_type is DitherType.AUTO:
-        should_dither = DitherType.should_dither(in_fmt, out_fmt, range_in, range_out)
-
-        if hasattr(clip, "fmtc"):
-            dither_type = DitherType.VOID
-        else:
-            dither_type = DitherType.ERROR_DIFFUSION if out_fmt.bits_per_sample == 8 else DitherType.ORDERED
-        dither_type = dither_type if should_dither else DitherType.NONE
-
     new_format = in_fmt.replace(bits_per_sample=out_fmt.bits_per_sample, sample_type=out_fmt.sample_type)
 
-    return dither_type.apply(clip, new_format, range_in, range_out)
+    return DitherType.from_param(dither_type, depth).apply(clip, new_format, range_in, range_out)
 
 
-_f2c_cache = WeakValueDictionary[int, ConstantFormatVideoNode]()
+def expect_bits(clip: vs.VideoNode, /, expected_depth: int = 16, **kwargs: Any) -> tuple[vs.VideoNode, int]:
+    """
+    Expected output bitdepth for a clip.
+
+    This function is meant to be used when a clip may not match the expected input bitdepth.
+    Both the dithered clip and the original bitdepth are returned.
+
+    Args:
+        clip: Input clip.
+        expected_depth: Expected bitdepth. Default: 16.
+
+    Returns:
+        Tuple containing the clip dithered to the expected depth and the original bitdepth.
+    """
+    bits = get_depth(clip)
+
+    if bits != expected_depth:
+        clip = depth(clip, expected_depth, **kwargs)
+
+    return clip, bits
 
 
-def frame2clip(frame: vs.VideoFrame) -> ConstantFormatVideoNode:
+_f2c_cache = WeakValueDictionary[int, vs.VideoNode]()
+
+
+def frame2clip(frame: vs.VideoFrame) -> vs.VideoNode:
     """
     Convert a VideoFrame to a VideoNode.
 
@@ -427,7 +414,7 @@ def frame2clip(frame: vs.VideoFrame) -> ConstantFormatVideoNode:
     return vs.core.std.ModifyFrame(blank_clip, blank_clip, lambda n, f: frame_cp)
 
 
-def get_y(clip: vs.VideoNode, /) -> ConstantFormatVideoNode:
+def get_y(clip: vs.VideoNode, /) -> vs.VideoNode:
     """
     Extract the luma (Y) plane of the given clip.
 
@@ -438,15 +425,15 @@ def get_y(clip: vs.VideoNode, /) -> ConstantFormatVideoNode:
         Y plane of the input clip.
 
     Raises:
-        CustomValueError: Clip is not GRAY or YUV.
+        UnsupportedColorFamilyError: Clip is not GRAY or YUV.
     """
 
-    InvalidColorFamilyError.check(clip, [vs.YUV, vs.GRAY], get_y)
+    UnsupportedColorFamilyError.check(clip, [vs.YUV, vs.GRAY], get_y)
 
     return plane(clip, 0)
 
 
-def get_u(clip: vs.VideoNode, /) -> ConstantFormatVideoNode:
+def get_u(clip: vs.VideoNode, /) -> vs.VideoNode:
     """
     Extract the first chroma (U) plane of the given clip.
 
@@ -457,15 +444,15 @@ def get_u(clip: vs.VideoNode, /) -> ConstantFormatVideoNode:
         Y plane of the input clip.
 
     Raises:
-        CustomValueError: Clip is not YUV.
+        UnsupportedColorFamilyError: Clip is not YUV.
     """
 
-    InvalidColorFamilyError.check(clip, vs.YUV, get_u)
+    UnsupportedColorFamilyError.check(clip, vs.YUV, get_u)
 
     return plane(clip, 1)
 
 
-def get_v(clip: vs.VideoNode, /) -> ConstantFormatVideoNode:
+def get_v(clip: vs.VideoNode, /) -> vs.VideoNode:
     """
     Extract the second chroma (V) plane of the given clip.
 
@@ -476,15 +463,15 @@ def get_v(clip: vs.VideoNode, /) -> ConstantFormatVideoNode:
         V plane of the input clip.
 
     Raises:
-        CustomValueError: Clip is not YUV.
+        UnsupportedColorFamilyError: Clip is not YUV.
     """
 
-    InvalidColorFamilyError.check(clip, vs.YUV, get_v)
+    UnsupportedColorFamilyError.check(clip, vs.YUV, get_v)
 
     return plane(clip, 2)
 
 
-def get_r(clip: vs.VideoNode, /) -> ConstantFormatVideoNode:
+def get_r(clip: vs.VideoNode, /) -> vs.VideoNode:
     """
     Extract the red plane of the given clip.
 
@@ -495,15 +482,15 @@ def get_r(clip: vs.VideoNode, /) -> ConstantFormatVideoNode:
         R plane of the input clip.
 
     Raises:
-        CustomValueError: Clip is not RGB.
+        UnsupportedColorFamilyError: Clip is not RGB.
     """
 
-    InvalidColorFamilyError.check(clip, vs.RGB, get_r)
+    UnsupportedColorFamilyError.check(clip, vs.RGB, get_r)
 
     return plane(clip, 0)
 
 
-def get_g(clip: vs.VideoNode, /) -> ConstantFormatVideoNode:
+def get_g(clip: vs.VideoNode, /) -> vs.VideoNode:
     """
     Extract the green plane of the given clip.
 
@@ -514,15 +501,15 @@ def get_g(clip: vs.VideoNode, /) -> ConstantFormatVideoNode:
         G plane of the input clip.
 
     Raises:
-        CustomValueError: Clip is not RGB.
+        UnsupportedColorFamilyError: Clip is not RGB.
     """
 
-    InvalidColorFamilyError.check(clip, vs.RGB, get_g)
+    UnsupportedColorFamilyError.check(clip, vs.RGB, get_g)
 
     return plane(clip, 1)
 
 
-def get_b(clip: vs.VideoNode, /) -> ConstantFormatVideoNode:
+def get_b(clip: vs.VideoNode, /) -> vs.VideoNode:
     """
     Extract the blue plane of the given clip.
 
@@ -533,17 +520,15 @@ def get_b(clip: vs.VideoNode, /) -> ConstantFormatVideoNode:
         B plane of the input clip.
 
     Raises:
-        CustomValueError: Clip is not RGB.
+        UnsupportedColorFamilyError: Clip is not RGB.
     """
 
-    InvalidColorFamilyError.check(clip, vs.RGB, get_b)
+    UnsupportedColorFamilyError.check(clip, vs.RGB, get_b)
 
     return plane(clip, 2)
 
 
-def insert_clip(
-    clip: vs.VideoNode, /, insert: vs.VideoNode, start_frame: int, strict: bool = True
-) -> ConstantFormatVideoNode:
+def insert_clip(clip: vs.VideoNode, /, insert: vs.VideoNode, start_frame: int, strict: bool = True) -> vs.VideoNode:
     """
     Replace frames of a longer clip with those of a shorter one.
 
@@ -589,7 +574,7 @@ def insert_clip(
 @overload
 def join(
     luma: vs.VideoNode, chroma: vs.VideoNode, /, *, prop_src: vs.VideoNode | SupportsIndex | None = ...
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Combine a luma and chroma clip into a YUV clip.
 
@@ -613,7 +598,7 @@ def join(
     *,
     family: vs.ColorFamily = vs.ColorFamily.YUV,
     prop_src: vs.VideoNode | SupportsIndex | None = ...,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Combine three single-plane clips into one, with optional alpha.
 
@@ -637,7 +622,7 @@ def join(
     /,
     *,
     prop_src: vs.VideoNode | SupportsIndex | None = ...,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Combine a sequence of single-plane clips into one.
 
@@ -658,7 +643,7 @@ def join(
     /,
     *,
     prop_src: vs.VideoNode | SupportsIndex | None = ...,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
     Combine a sequence of single-plane clips into one.
 
@@ -701,8 +686,6 @@ def join(
         Clip with combined planes.
     """
     if isinstance(clips, Mapping):
-        from ..functions import flatten_vnodes
-
         clips_map = dict[int, vs.VideoNode]()
 
         for p_key, node in clips.items():
@@ -731,7 +714,7 @@ def join(
         raise CustomTypeError(func=join)
 
     if not plane2:
-        InvalidColorFamilyError.check(
+        UnsupportedColorFamilyError.check(
             (family, plane1_or_family),
             vs.YUV,
             join,
@@ -756,7 +739,7 @@ def join(
     return joined
 
 
-def plane(clip: vs.VideoNode, index: SupportsIndex, /, strict: bool = True) -> ConstantFormatVideoNode:
+def plane(clip: vs.VideoNode, index: SupportsIndex, /, strict: bool = True) -> vs.VideoNode:
     """
     Extract a plane from the given clip.
 
@@ -768,8 +751,6 @@ def plane(clip: vs.VideoNode, index: SupportsIndex, /, strict: bool = True) -> C
     Returns:
         Grayscale clip of the clip's plane.
     """
-    assert check_variable_format(clip, plane)
-
     if clip.format.num_planes == 1 and index.__index__() == 0:
         return clip
 
@@ -779,7 +760,7 @@ def plane(clip: vs.VideoNode, index: SupportsIndex, /, strict: bool = True) -> C
     return vs.core.std.ShufflePlanes(clip, index.__index__(), vs.GRAY)
 
 
-def split(clip: vs.VideoNode, /, strict: bool = True) -> list[ConstantFormatVideoNode]:
+def split(clip: vs.VideoNode, /, strict: bool = True) -> list[vs.VideoNode]:
     """
     Split a clip into a list of individual planes.
 
@@ -790,15 +771,29 @@ def split(clip: vs.VideoNode, /, strict: bool = True) -> list[ConstantFormatVide
     Returns:
         List of individual planes.
     """
-    assert check_variable_format(clip, split)
-
     return [clip] if clip.format.num_planes == 1 else [plane(clip, i, strict) for i in range(clip.format.num_planes)]
 
 
-depth_func = depth
+def flatten_vnodes(*clips: VideoNodeIterable, split_planes: bool = False) -> Sequence[vs.VideoNode]:
+    """
+    Flatten an array of VideoNodes.
+
+    Args:
+        *clips: An array of clips to flatten into a list.
+        split_planes: Optionally split the VideoNodes into their individual planes as well. Default: False.
+
+    Returns:
+        Flattened list of VideoNodes.
+    """
+    nodes = list[vs.VideoNode](flatten(clips))
+
+    if not split_planes:
+        return nodes
+
+    return reduce(operator.iadd, map(split, nodes), [])
 
 
-def stack_clips(clips: Iterable[VideoNodeIterableT[vs.VideoNode]]) -> vs.VideoNode:
+def stack_clips(clips: Iterable[VideoNodeIterable]) -> vs.VideoNode:
     """
     Recursively stack clips in alternating directions: horizontal → vertical → horizontal → ...
 
@@ -884,10 +879,10 @@ def limiter(
     mask: bool = False,
     planes: Planes = None,
     func: FuncExcept | None = None,
-) -> ConstantFormatVideoNode:
+) -> vs.VideoNode:
     """
-    Wraps `vs-zip <https://github.com/dnjulek/vapoursynth-zip>`.Limiter but only processes
-    if clip format is not integer, a min/max val is specified or tv_range is True.
+    Wraps [vszip.Limiter](https://github.com/dnjulek/vapoursynth-zip/wiki/Limiter)
+    but only processes if clip format is not integer, a min/max val is specified or tv_range is True.
 
     Args:
         clip: Clip to process.
@@ -897,7 +892,7 @@ def limiter(
             individually.
         tv_range: Changes min/max defaults values to LIMITED.
         mask: Float chroma range from -0.5/0.5 to 0.0/1.0.
-        planes: Planes to process.
+        planes: Which planes to process.
         func: Function returned for custom error handling. This should only be set by VS package developers.
 
     Returns:
@@ -906,8 +901,8 @@ def limiter(
 
 
 @overload
-def limiter(
-    _func: Callable[P, ConstantFormatVideoNode],
+def limiter[**P](
+    _func: Callable[P, vs.VideoNode],
     /,
     min_val: float | Sequence[float] | None = None,
     max_val: float | Sequence[float] | None = None,
@@ -916,10 +911,10 @@ def limiter(
     mask: bool = False,
     planes: Planes = None,
     func: FuncExcept | None = None,
-) -> Callable[P, ConstantFormatVideoNode]:
+) -> Callable[P, vs.VideoNode]:
     """
-    Wraps `vs-zip <https://github.com/dnjulek/vapoursynth-zip>`.Limiter but only processes
-    if clip format is not integer, a min/max val is specified or tv_range is True.
+    Wraps [vszip.Limiter](https://github.com/dnjulek/vapoursynth-zip/wiki/Limiter)
+    but only processes if clip format is not integer, a min/max val is specified or tv_range is True.
 
     This is the decorator implementation.
 
@@ -931,7 +926,7 @@ def limiter(
             individually.
         tv_range: Changes min/max defaults values to LIMITED.
         mask: Float chroma range from -0.5/0.5 to 0.0/1.0.
-        planes: Planes to process.
+        planes: Which planes to process.
         func: Function returned for custom error handling. This should only be set by VS package developers.
 
     Returns:
@@ -940,7 +935,7 @@ def limiter(
 
 
 @overload
-def limiter(
+def limiter[**P](
     *,
     min_val: float | Sequence[float] | None = None,
     max_val: float | Sequence[float] | None = None,
@@ -948,10 +943,10 @@ def limiter(
     mask: bool = False,
     planes: Planes = None,
     func: FuncExcept | None = None,
-) -> Callable[[Callable[P, ConstantFormatVideoNode]], Callable[P, ConstantFormatVideoNode]]:
+) -> Callable[[Callable[P, vs.VideoNode]], Callable[P, vs.VideoNode]]:
     """
-    Wraps `vs-zip <https://github.com/dnjulek/vapoursynth-zip>`.Limiter but only processes
-    if clip format is not integer, a min/max val is specified or tv_range is True.
+    Wraps [vszip.Limiter](https://github.com/dnjulek/vapoursynth-zip/wiki/Limiter)
+    but only processes if clip format is not integer, a min/max val is specified or tv_range is True.
 
     This is the decorator implementation.
 
@@ -962,7 +957,7 @@ def limiter(
             individually.
         tv_range: Changes min/max defaults values to LIMITED.
         mask: Float chroma range from -0.5/0.5 to 0.0/1.0.
-        planes: Planes to process.
+        planes: Which planes to process.
         func: Function returned for custom error handling. This should only be set by VS package developers.
 
     Returns:
@@ -970,8 +965,8 @@ def limiter(
     """
 
 
-def limiter(
-    clip_or_func: vs.VideoNode | Callable[P, ConstantFormatVideoNode] | None = None,
+def limiter[**P](
+    clip_or_func: vs.VideoNode | Callable[P, vs.VideoNode] | None = None,
     /,
     min_val: float | Sequence[float] | None = None,
     max_val: float | Sequence[float] | None = None,
@@ -981,13 +976,13 @@ def limiter(
     planes: Planes = None,
     func: FuncExcept | None = None,
 ) -> Union[
-    ConstantFormatVideoNode,
-    Callable[P, ConstantFormatVideoNode],
-    Callable[[Callable[P, ConstantFormatVideoNode]], Callable[P, ConstantFormatVideoNode]],
+    vs.VideoNode,
+    Callable[P, vs.VideoNode],
+    Callable[[Callable[P, vs.VideoNode]], Callable[P, vs.VideoNode]],
 ]:
     """
-    Wraps `vs-zip <https://github.com/dnjulek/vapoursynth-zip>`.Limiter but only processes
-    if clip format is not integer, a min/max val is specified or tv_range is True.
+    Wraps [vszip.Limiter](https://github.com/dnjulek/vapoursynth-zip/wiki/Limiter)
+    but only processes if clip format is not integer, a min/max val is specified or tv_range is True.
 
     Args:
         clip_or_func: Clip to process or function that returns a VideoNode to be processed.
@@ -997,25 +992,25 @@ def limiter(
             individually.
         tv_range: Changes min/max defaults values to LIMITED.
         mask: Float chroma range from -0.5/0.5 to 0.0/1.0.
-        planes: Planes to process.
+        planes: Which planes to process.
         func: Function returned for custom error handling. This should only be set by VS package developers.
 
     Returns:
         Clamped clip.
     """
     if callable(clip_or_func):
-        _func = clip_or_func
+        func_ = clip_or_func
 
-        @wraps(_func)
-        def _wrapper(*args: P.args, **kwargs: P.kwargs) -> ConstantFormatVideoNode:
+        @wraps(func_)
+        def _wrapper(*args: P.args, **kwargs: P.kwargs) -> vs.VideoNode:
             return limiter(
-                _func(*args, **kwargs),
+                func_(*args, **kwargs),
                 min_val,
                 max_val,
                 tv_range=tv_range,
                 mask=mask,
                 planes=planes,
-                func=func or _func,
+                func=func or func_,
             )
 
         return _wrapper
@@ -1026,8 +1021,6 @@ def limiter(
     if clip is None:
         return partial(limiter, min_val=min_val, max_val=max_val, tv_range=tv_range, planes=planes, func=func)
 
-    assert check_variable_format(clip, func)
-
     if all([clip.format.sample_type == vs.INTEGER, min_val is None, max_val is None, tv_range is False]):
         return clip
 
@@ -1038,17 +1031,3 @@ def limiter(
         max_val = normalize_seq(max_val or get_peak_values(clip, clip), clip.format.num_planes)
 
     return clip.vszip.Limiter(min_val, max_val, tv_range, mask, planes)
-
-
-def sc_detect(clip: vs.VideoNode, threshold: float = 0.1) -> ConstantFormatVideoNode:
-    assert check_variable_format(clip, sc_detect)
-
-    stats = vs.core.std.PlaneStats(shift_clip(clip, -1), clip)
-
-    return vs.core.akarin.PropExpr(
-        [clip, stats, stats[1:]],
-        lambda: {
-            "_SceneChangePrev": f"y.PlaneStatsDiff {threshold} > 1 0 ?",
-            "_SceneChangeNext": f"z.PlaneStatsDiff {threshold} > 1 0 ?",
-        },
-    )
