@@ -4,21 +4,30 @@ from contextlib import AbstractContextManager
 from fractions import Fraction
 from math import floor
 from types import TracebackType
-from typing import TYPE_CHECKING, Any, Callable, Iterable, Sequence, cast, overload
+from typing import Any, Iterable, Iterator, Self, Sequence, overload
 
-import vapoursynth as vs
-from jetpytools import MISSING, MissingT, P
-from typing_extensions import Self
+from jetpytools import MISSING, CustomValueError, MissingT, normalize_seq, to_arr
+from jetpytools import flatten as jetp_flatten
 
-from ..enums import Align
-from ..exceptions import InvalidSubsamplingError
-from ..functions import Keyframes, check_variable_format, clip_data_gather
-from ..types import ConstantFormatVideoNode, VideoNodeT
-from ..utils.cache import SceneBasedDynamicCache
-from .info import get_video_format
+from ..enums import Align, Matrix
+from ..types import AudioNodeIterable, Planes, RawNodeIterable, VideoNodeIterable
+from ..vs_proxy import core, vs
+from .check import check_variable
+from .info import get_subsampling
 from .props import get_props
+from .scale import get_lowest_values, get_neutral_values, get_peak_values
 
-__all__ = ["SceneAverageStats", "change_fps", "match_clip", "padder", "padder_ctx", "pick_func_stype", "set_output"]
+__all__ = [
+    "change_fps",
+    "flatten",
+    "invert_planes",
+    "match_clip",
+    "normalize_param_planes",
+    "normalize_planes",
+    "padder",
+    "padder_ctx",
+    "set_output",
+]
 
 
 def change_fps(clip: vs.VideoNode, fps: Fraction) -> vs.VideoNode:
@@ -71,9 +80,6 @@ def match_clip(
             clip. Default: True.
         length: Whether to adjust the length of the reference clip to match the original clip.
     """
-    from ..enums import Matrix
-    from ..functions import check_variable
-
     assert check_variable(clip, match_clip)
     assert check_variable(ref, match_clip)
 
@@ -96,13 +102,16 @@ def match_clip(
     return clip.std.AssumeFPS(fpsnum=ref.fps.numerator, fpsden=ref.fps.denominator)
 
 
-class padder_ctx(AbstractContextManager["padder_ctx"]):  # noqa: N801
+# ruff: noqa: N802, N801
+class padder_ctx(AbstractContextManager["padder_ctx"]):
     """
     Context manager for the padder class.
     """
 
     def __init__(self, mod: int = 8, min: int = 0, align: Align = Align.MIDDLE_CENTER) -> None:
         """
+        Initializes the class
+
         Args:
             mod: The modulus used for calculations or constraints. Defaults to 8.
             min: The minimum value allowed or used as a base threshold. Defaults to 0.
@@ -113,9 +122,7 @@ class padder_ctx(AbstractContextManager["padder_ctx"]):  # noqa: N801
         self.align = align
         self.pad_ops = list[tuple[tuple[int, int, int, int], tuple[int, int]]]()
 
-    def CROP(  # noqa: N802
-        self, clip: vs.VideoNode, crop_scale: float | tuple[float, float] | None = None
-    ) -> ConstantFormatVideoNode:
+    def CROP(self, clip: vs.VideoNode, crop_scale: float | tuple[float, float] | None = None) -> vs.VideoNode:
         """
         Crop a clip with the padding values.
 
@@ -135,7 +142,7 @@ class padder_ctx(AbstractContextManager["padder_ctx"]):  # noqa: N801
 
         return clip.std.Crop(*(x * y for x, y in zip(padding, crop_pad)))
 
-    def MIRROR(self, clip: VideoNodeT) -> VideoNodeT:  # noqa: N802
+    def MIRROR(self, clip: vs.VideoNode) -> vs.VideoNode:
         """
         Pad a clip with reflect mode. This will reflect the clip on each side.
 
@@ -150,7 +157,7 @@ class padder_ctx(AbstractContextManager["padder_ctx"]):  # noqa: N801
         self.pad_ops.append((padding, (out.width, out.height)))
         return out
 
-    def REPEAT(self, clip: vs.VideoNode) -> ConstantFormatVideoNode:  # noqa: N802
+    def REPEAT(self, clip: vs.VideoNode) -> vs.VideoNode:
         """
         Pad a clip with repeat mode. This will simply repeat the last row/column till the end.
 
@@ -165,18 +172,19 @@ class padder_ctx(AbstractContextManager["padder_ctx"]):  # noqa: N801
         self.pad_ops.append((padding, (out.width, out.height)))
         return out
 
-    def COLOR(  # noqa: N802
-        self, clip: vs.VideoNode, color: int | float | bool | None | Sequence[int | float | bool | None] = (False, None)
-    ) -> ConstantFormatVideoNode:
+    def COLOR(self, clip: vs.VideoNode, color: float | None | Sequence[float | None] = (False, None)) -> vs.VideoNode:
         """
         Pad a clip with a constant color.
 
         Args:
             clip: Input clip.
-            color: Constant color that should be added on the sides: * number: This will be treated as such and not
-                converted or clamped. * False: Lowest value for this clip format and color range. * True: Highest value
-                for this clip format and color range. * None: Neutral value for this clip format. * MISSING:
-                Automatically set to False if RGB, else None.
+            color: Constant color that should be added on the sides:
+
+                   * number: This will be treated as such and not converted or clamped.
+                   * False: Lowest value for this clip format and color range.
+                   * True: Highest value for this clip format and color range.
+                   * None: Neutral value for this clip format.
+                   * MISSING: Automatically set to False if RGB, else None.
 
         Returns:
             Padded clip with colored borders.
@@ -195,42 +203,38 @@ class padder_ctx(AbstractContextManager["padder_ctx"]):  # noqa: N801
         return None
 
 
-class padder:  # noqa: N801
+class padder:
     """
     Pad out the pixels on the sides by the given amount of pixels.
     """
 
     ctx = padder_ctx
+    """Context manager for the padder class."""
 
     @staticmethod
     def _base(
         clip: vs.VideoNode, left: int = 0, right: int = 0, top: int = 0, bottom: int = 0
     ) -> tuple[int, int, vs.VideoFormat, int, int]:
-        from ..functions import check_variable
-
-        assert check_variable(clip, "padder")
-
         width = clip.width + left + right
         height = clip.height + top + bottom
 
-        fmt = get_video_format(clip)
+        w_sub, h_sub = 2**clip.format.subsampling_w, 2**clip.format.subsampling_h
 
-        w_sub, h_sub = 1 << fmt.subsampling_w, 1 << fmt.subsampling_h
-
-        if width % w_sub and height % h_sub:
-            raise InvalidSubsamplingError(
-                "padder", fmt, "Values must result in a mod congruent to the clip's subsampling ({subsampling})!"
+        if width % w_sub or height % h_sub:
+            raise CustomValueError(
+                "Values must result in a mod congruent to the clip's subsampling ({subsampling}).",
+                "padder",
+                subsampling=get_subsampling(clip),
             )
 
-        return width, height, fmt, w_sub, h_sub
+        return width, height, clip.format, w_sub, h_sub
 
     @classmethod
-    def MIRROR(cls, clip: VideoNodeT, left: int = 0, right: int = 0, top: int = 0, bottom: int = 0) -> VideoNodeT:  # noqa: N802
+    def MIRROR(cls, clip: vs.VideoNode, left: int = 0, right: int = 0, top: int = 0, bottom: int = 0) -> vs.VideoNode:
         """
         Pad a clip with reflect mode. This will reflect the clip on each side.
 
         Visual example:
-
             ```
             >>> |ABCDE
             >>> padder.MIRROR(left=3)
@@ -248,8 +252,6 @@ class padder:  # noqa: N801
             Padded clip with reflected borders.
         """
 
-        from ..utils import core
-
         width, height, *_ = cls._base(clip, left, right, top, bottom)
 
         padded = core.resize.Point(
@@ -264,14 +266,11 @@ class padder:  # noqa: N801
         return core.std.CopyFrameProps(padded, clip)
 
     @classmethod
-    def REPEAT(  # noqa: N802
-        cls, clip: vs.VideoNode, left: int = 0, right: int = 0, top: int = 0, bottom: int = 0
-    ) -> ConstantFormatVideoNode:
+    def REPEAT(cls, clip: vs.VideoNode, left: int = 0, right: int = 0, top: int = 0, bottom: int = 0) -> vs.VideoNode:
         """
         Pad a clip with repeat mode. This will simply repeat the last row/column till the end.
 
         Visual example:
-
             ```
             >>> |ABCDE
             >>> padder.REPEAT(left=3)
@@ -330,20 +329,19 @@ class padder:  # noqa: N801
         )
 
     @classmethod
-    def COLOR(  # noqa: N802
+    def COLOR(
         cls,
         clip: vs.VideoNode,
         left: int = 0,
         right: int = 0,
         top: int = 0,
         bottom: int = 0,
-        color: int | float | bool | None | MissingT | Sequence[int | float | bool | None | MissingT] = (False, MISSING),
-    ) -> ConstantFormatVideoNode:
+        color: float | None | MissingT | Sequence[float | None | MissingT] = (False, MISSING),
+    ) -> vs.VideoNode:
         """
         Pad a clip with a constant color.
 
         Visual example:
-
             ```
             >>> |ABCDE
             >>> padder.COLOR(left=3, color=Z)
@@ -367,15 +365,9 @@ class padder:  # noqa: N801
         Returns:
             Padded clip with colored borders.
         """
-
-        from ..functions import normalize_seq
-        from ..utils import core, get_lowest_values, get_neutral_values, get_peak_values
-
-        assert check_variable_format(clip, "padder")
-
         cls._base(clip, left, right, top, bottom)
 
-        def _norm(colr: int | float | bool | None | MissingT) -> Sequence[int | float]:
+        def _norm(colr: float | bool | None | MissingT) -> Sequence[float]:
             if colr is MISSING:
                 colr = False if clip.format.color_family is vs.RGB else None
 
@@ -452,52 +444,14 @@ class padder:  # noqa: N801
         return padding, tuple(x * crop_scale[0 if i < 2 else 1] for x, i in enumerate(padding))  # type: ignore
 
 
-def pick_func_stype(
-    clip: vs.VideoNode, func_int: Callable[P, VideoNodeT], func_float: Callable[P, VideoNodeT]
-) -> Callable[P, VideoNodeT]:
-    """
-    Pick the function matching the sample type of the clip's format.
-
-    Args:
-        clip: Input clip.
-        func_int: Function to run on integer clips.
-        func_float: Function to run on float clips.
-
-    Returns:
-        Function matching the sample type of your clip's format.
-    """
-
-    assert check_variable_format(clip, pick_func_stype)
-
-    return func_float if clip.format.sample_type == vs.FLOAT else func_int
-
-
 @overload
-def set_output(node: vs.VideoNode, index: int = ..., /, *, alpha: vs.VideoNode | None = ..., **kwargs: Any) -> None:
-    """
-    Set output node with optional index, and if available, use vspreview set_output.
-
-    Args:
-        node: Output node
-        index: Index number, defaults to current maximum index number + 1 or 0 if no ouput exists yet
-        alpha: Alpha planes node, defaults to None
-        **kwargs: Additional arguments to be passed to `vspreview.set_output`.
-    """
+def set_output(node: vs.VideoNode, index: int = ..., /, *, alpha: vs.VideoNode | None = ..., **kwargs: Any) -> None: ...
 
 
 @overload
 def set_output(
     node: vs.VideoNode, name: str | bool | None = ..., /, *, alpha: vs.VideoNode | None = ..., **kwargs: Any
-) -> None:
-    """
-    Set output node with optional name, and if available, use vspreview set_output.
-
-    Args:
-        node: Output node
-        name: Node's name, defaults to variable name
-        alpha: Alpha planes node, defaults to None
-        **kwargs: Additional arguments to be passed to `vspreview.set_output`.
-    """
+) -> None: ...
 
 
 @overload
@@ -508,111 +462,33 @@ def set_output(
     /,
     alpha: vs.VideoNode | None = ...,
     **kwargs: Any,
-) -> None:
-    """
-    Set output node with optional index and name, and if available, use vspreview set_output.
-
-    Args:
-        node: Output node.
-        index: Index number, defaults to current maximum index number + 1 or 0 if no ouput exists yet.
-        name: Node's name, defaults to variable name
-        alpha: Alpha planes node, defaults to None.
-        **kwargs: Additional arguments to be passed to `vspreview.set_output`.
-    """
-
-
-@overload
-def set_output(node: vs.AudioNode, index: int = ..., /, **kwargs: Any) -> None:
-    """
-    Set output node with optional index, and if available, use vspreview set_output.
-
-    Args:
-        node: Output node.
-        index: Index number, defaults to current maximum index number + 1 or 0 if no ouput exists yet.
-        **kwargs: Additional arguments to be passed to `vspreview.set_output`.
-    """
-
-
-@overload
-def set_output(node: vs.AudioNode, name: str | bool | None = ..., /, **kwargs: Any) -> None:
-    """
-    Set output node with optional name, and if available, use vspreview set_output.
-
-    Args:
-        node: Output node.
-        name: Node's name, defaults to variable name
-        **kwargs: Additional arguments to be passed to `vspreview.set_output`.
-    """
-
-
-@overload
-def set_output(node: vs.AudioNode, index: int = ..., name: str | bool | None = ..., /, **kwargs: Any) -> None:
-    """
-    Set output node with optional index and name, and if available, use vspreview set_output.
-
-    Args:
-        node: Output node.
-        index: Index number, defaults to current maximum index number + 1 or 0 if no ouput exists yet.
-        name: Node's name, defaults to variable name.
-        **kwargs: Additional arguments to be passed to `vspreview.set_output`.
-    """
+) -> None: ...
 
 
 @overload
 def set_output(
-    node: Iterable[vs.RawNode | Iterable[vs.RawNode | Iterable[vs.RawNode]]],
-    index: int | Sequence[int] = ...,
-    /,
-    **kwargs: Any,
-) -> None:
-    """
-    Set output node with optional index, and if available, use vspreview set_output.
-
-    Args:
-        node: Output node.
-        index: Index number, defaults to current maximum index number + 1 or 0 if no ouput exists yet.
-        **kwargs: Additional arguments to be passed to `vspreview.set_output`.
-    """
+    node: VideoNodeIterable | AudioNodeIterable | RawNodeIterable, index: int | Sequence[int] = ..., /, **kwargs: Any
+) -> None: ...
 
 
 @overload
 def set_output(
-    node: Iterable[vs.RawNode | Iterable[vs.RawNode | Iterable[vs.RawNode]]],
-    name: str | bool | None = ...,
-    /,
-    **kwargs: Any,
-) -> None:
-    """
-    Set output node with optional name, and if available, use vspreview set_output.
-
-    Args:
-        node: Output node.
-        name: Node's name, defaults to variable name.
-        **kwargs: Additional arguments to be passed to `vspreview.set_output`.
-    """
+    node: VideoNodeIterable | AudioNodeIterable | RawNodeIterable, name: str | bool | None = ..., /, **kwargs: Any
+) -> None: ...
 
 
 @overload
 def set_output(
-    node: Iterable[vs.RawNode | Iterable[vs.RawNode | Iterable[vs.RawNode]]],
+    node: VideoNodeIterable | AudioNodeIterable | RawNodeIterable,
     index: int | Sequence[int] = ...,
     name: str | bool | None = ...,
     /,
     **kwargs: Any,
-) -> None:
-    """
-    Set output node with optional index and name, and if available, use vspreview set_output.
-
-    Args:
-        node: Output node.
-        index: Index number, defaults to current maximum index number + 1 or 0 if no ouput exists yet
-        name: Node's name, defaults to variable name
-        **kwargs: Additional arguments to be passed to `vspreview.set_output`.
-    """
+) -> None: ...
 
 
 def set_output(
-    node: vs.RawNode | Iterable[vs.RawNode | Iterable[vs.RawNode | Iterable[vs.RawNode]]],
+    node: vs.VideoNode | VideoNodeIterable | AudioNodeIterable | RawNodeIterable,
     index_or_name: int | Sequence[int] | str | bool | None = None,
     name: str | bool | None = None,
     /,
@@ -620,80 +496,105 @@ def set_output(
     **kwargs: Any,
 ) -> None:
     """
-    Set output node with optional index and name, and if available, use vspreview set_output.
+    Wrapper around vspreview.set_output if available, falling back to basic VapourSynth output.
 
     Args:
-        node: Output node.
-        index_or_name: Index number, defaults to current maximum index number + 1 or 0 if no ouput exists yet.
-        name: Node's name, defaults to variable name
-        alpha: Alpha planes node, defaults to None.
-        **kwargs: Additional arguments to be passed to `vspreview.set_output`.
+        node: Output node(s).
+        index_or_name: Index number or name, defaults to current maximum index number + 1.
+        name: Node's display name, defaults to variable name if True.
+        alpha: Optional alpha planes node.
+        **kwargs: Extra arguments passed through to vspreview.set_output.
     """
-    from ..functions import flatten, to_arr
-
-    if isinstance(index_or_name, (str, bool)):
-        index = None
-        if not TYPE_CHECKING and isinstance(name, vs.VideoNode):
-            # Backward compatible with older api
-            alpha = name
-        name = index_or_name
-    else:
-        index = index_or_name
-
-    ouputs = vs.get_outputs()
-    nodes = list[vs.RawNode](flatten(node)) if isinstance(node, Iterable) else [node]
-
-    index = to_arr(index) if index is not None else [max(ouputs, default=-1) + 1]
-
-    while len(index) < len(nodes):
-        index.append(index[-1] + 1)
-
     try:
         from vspreview import set_output as vsp_set_output
 
-        vsp_set_output(nodes, index, name, alpha=alpha, f_back=2, force_preview=True, **kwargs)
     except ModuleNotFoundError:
-        for idx, n in zip(index, nodes):
+        index = None if isinstance(index_or_name, (str, bool)) else index_or_name
+
+        outputs = vs.get_outputs()
+        nodes = list(flatten(node))
+        indices = to_arr(index) if index is not None else [max(outputs, default=-1) + 1]
+        indices = normalize_seq(indices, len(nodes))
+
+        for idx, n in zip(indices, nodes):
             n.set_output(idx)
+    else:
+        kwargs.setdefault("frame_depth", 2)
+
+        return vsp_set_output(node, index_or_name, name, alpha=alpha, **kwargs)  # type: ignore[arg-type]
 
 
-class SceneAverageStats(SceneBasedDynamicCache):
-    _props_keys = ("Min", "Max", "Average")
+def normalize_planes(clip: vs.VideoNode, planes: Planes = None) -> list[int]:
+    """
+    Normalize a sequence of planes.
 
-    class cache(dict[int, tuple[float, float, float]]):  # noqa: N801
-        def __init__(self, clip: vs.VideoNode, keyframes: Keyframes, plane: int) -> None:
-            self.props = clip.std.PlaneStats(plane=plane)
-            self.keyframes = keyframes
+    Args:
+        clip: Input clip.
+        planes: Array of planes. If None, returns all planes of the input clip's format. Default: None.
 
-        def __getitem__(self, idx: int) -> tuple[float, float, float]:
-            if idx not in self:
-                frame_range = self.keyframes.scenes[idx]
-                cut_clip = self.props[frame_range.start : frame_range.stop]
+    Returns:
+        Sorted list of planes.
+    """
 
-                frames_min_max_avg = clip_data_gather(
-                    cut_clip,
-                    None,
-                    lambda n, f: tuple(cast(float, f.props[f"PlaneStats{p}"]) for p in SceneAverageStats._props_keys),
-                )
+    assert clip.format
 
-                frames_min, frames_max, frames_avgs = [[x[i] for x in frames_min_max_avg] for i in (0, 1, 2)]
+    planes = list(range(clip.format.num_planes)) if planes is None or planes == 4 else to_arr(planes)
 
-                self[idx] = (min(frames_min), max(frames_max), sum(frames_avgs) / len(frames_avgs))
+    return sorted(set(planes).intersection(range(clip.format.num_planes)))
 
-            return super().__getitem__(idx)
 
-    def __init__(
-        self,
-        clip: vs.VideoNode,
-        keyframes: Keyframes | str,
-        prop: str = "SceneStats",
-        plane: int = 0,
-        cache_size: int = 5,
-    ) -> None:
-        super().__init__(clip, keyframes, cache_size)
+def invert_planes(clip: vs.VideoNode, planes: Planes = None) -> list[int]:
+    """
+    Invert a sequence of planes.
 
-        self.prop_keys = tuple(f"{prop}{x}" for x in self._props_keys)
-        self.scene_avgs = self.__class__.cache(self.clip, self.keyframes, plane)
+    Args:
+        clip: Input clip.
+        planes: Array of planes. If None, selects all planes of the input clip's format.
 
-    def get_clip(self, key: int) -> vs.VideoNode:
-        return self.clip.std.SetFrameProps(**dict(zip(self.prop_keys, self.scene_avgs[key])))
+    Returns:
+        Sorted inverted list of planes.
+    """
+    return sorted(set(normalize_planes(clip, None)) - set(normalize_planes(clip, planes)))
+
+
+def normalize_param_planes[T](clip: vs.VideoNode, param: T | Sequence[T], planes: Planes, null: T) -> list[T]:
+    """
+    Normalize a value or sequence to a list mapped to the clip's planes.
+
+    For any plane not included in `planes`, the corresponding output value is set to `null`.
+
+    Args:
+        clip: The input clip whose format and number of planes will be used to determine mapping.
+        param: A single value or a sequence of values to normalize across the clip's planes.
+        planes: The planes to apply the values to. Other planes will receive `null`.
+        null: The default value to use for planes that are not included in `planes`.
+
+    Returns:
+        A list of length equal to the number of planes in the clip, with `param` values or `null`.
+    """
+    planes = normalize_planes(clip, planes)
+
+    return [p if i in planes else null for i, p in enumerate(normalize_seq(param, clip.format.num_planes))]
+
+
+@overload
+def flatten[T](items: Iterable[Iterable[T]]) -> Iterator[T]: ...
+
+
+@overload
+def flatten(items: Iterable[Any]) -> Iterator[Any]: ...
+
+
+@overload
+def flatten(items: Any) -> Iterator[Any]: ...
+
+
+def flatten(items: Any) -> Iterator[Any]:
+    """
+    Flatten an array of values, clips and frames included.
+    """
+
+    if isinstance(items, (vs.RawNode, vs.RawFrame)):
+        yield items
+    else:
+        yield from jetp_flatten(items)
